#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import json
import glob
from powervc_ttv.resources import BrocadeFabricResource
from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import translate_message


output_dict = {
    "ValidationID": "BROCADEFABRIC_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "brocade_fabric",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}

logFile = "brocadefabric_validator.log"


def main_func():
    rc = 0
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()
    if rc == 1:
        return 1

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.BROCADE_FABRIC_VALIDATIONS,
        desired_locale)

    resource_dict = readConf(logger)
    brocade = "brocade_fabric"
    validationProp = utils.readJSONFile(
        "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json")[brocade]

    for resource in resource_dict:
        try:
            validateResource(validationProp, resource, logger)
        except Exception as e:
            logger.exception(e)
    print(json.dumps({"ValidationResult": output_dict}))
    return rc

'''
Brocade fabric zoning and role checker
'''


class ZoningAndRoleChecker():

    def __init__(self, resource, expzoning, exprole):
        self.resource = resource
        self.expzoning = expzoning
        self.exprole = exprole
        self.result = []

    def check(self):
        status = ""
        val_check_id = 'BROCADE_FABRIC_ZONING_PROPERTY'
        host = self.resource.ip
        (zoning, role) = self.resource.getZoningAndRoleStatus()
        if len(zoning.strip()) > 0:
            if zoning.lower() == self.expzoning.lower():
                camsgplaceholders = [host]
                correctiveAction = ""
                msgplaceholders = [host]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.BROCADE_FABRIC_ZONING_MATCH,
                    desired_locale)
                messageID = ""
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
            else:
                camsgplaceholders = [self.expzoning]
                correctiveAction = translate_message(
                    ttvCatalog.CA_BROCADE_FABRIC_ZONING_MISMATCH,
                    desired_locale)
                msgplaceholders = [self.expzoning]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.BROCADE_FABRIC_ZONING_MISMATCH,
                    desired_locale)
                messageID = messageIDCatalog.BROCADE_FABRIC_CONFIGURATION_ERROR
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
        if len(role.strip()) > 0:
            val_check_id = 'BROCADE_FABRIC_SWITCH_ROLE'
            if role.lower().startswith(self.exprole.lower()):
                camsgplaceholders = [host]
                correctiveAction = ""
                msgplaceholders = [host]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.BROCADE_FABRIC_ROLE_MATCH,
                    desired_locale)
                messageID = ""
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
            else:
                camsgplaceholders = [role]
                correctiveAction = translate_message(
                    ttvCatalog.CA_BROCADE_FABRIC_ROLE_MISMATCH,
                    desired_locale)
                msgplaceholders = [role]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.BROCADE_FABRIC_ROLE_MISMATCH,
                    desired_locale)
                messageID = messageIDCatalog.BROCADE_FABRIC_CONFIGURATION_ERROR
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     val_check_id))
        return (status, self.result)


'''
Brocade fabric firmware checker
'''


class FirmwareChecker():

    def __init__(self, resource, expfirmware):
        self.resource = resource
        self.expfirmware = expfirmware
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        firmware = self.resource.getFirmware()
        if utils.compareVersion(firmware, self.expfirmware):
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.BROCADE_FABRIC_FIRMWARE_MATCH,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'BROCADE_FABRIC_FIRMWARE'))
        else:
            camsgplaceholders = [self.expfirmware]
            correctiveAction = translate_message(
                ttvCatalog.CA_BROCADE_FABRIC_FIRMWARE_MISMATCH,
                desired_locale)
            msgplaceholders = [self.expfirmware, firmware]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.BROCADE_FABRIC_FIRMWARE_MISMATCH,
                desired_locale)
            messageID = messageIDCatalog.STORAGE_FIRMWARE_INCORRECT
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'BROCADE_FABRIC_FIRMWARE'))
        return (status, self.result)


'''
Brocade fabric credential checker
'''


class CredentialChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        credential = self.resource.checkCredential()
        if credential == 0:
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_VALID,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'BROCADE_FABRIC_CREDENTIAL'))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.BROCADE_FABRIC_CONFIGURATION_ERROR
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'BROCADE_FABRIC_CREDENTIAL'))

        return (status, self.result)


'''
Brocade fabric reachability checker
'''


class ReachabilityChecker():

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        reachability = utils.checkReachability(host)
        if reachability == 0:
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'BROCADE_FABRIC_REACHABILITY'
                                                 ))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = messageIDCatalog.\
                BROCADE_FABRIC_CONFIGURATION_ERROR
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 'BROCADE_FABRIC_REACHABILITY'
                                                 ))
        return (status, self.result)


def validateResource(validationProp, resource, logger=None):
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]
    exception = False
    host = resource["hostname"]
    brocaderesourcemgr = BrocadeFabricResource.BrocadeFabricResource(
        resource, logger)
    checkers = []
    # checkers.append(ReachabilityChecker(brocaderesourcemgr))
    # checkers.append(CredentialChecker(brocaderesourcemgr))
    checkers.append(
        FirmwareChecker(brocaderesourcemgr, validationProp["firmware"]))
    checkers.append(ZoningAndRoleChecker(
        brocaderesourcemgr, validationProp["zoning"], validationProp["role"]))
    status = ""
    result = []
    try:
        (status, result) = ReachabilityChecker(brocaderesourcemgr).check()
        chkResultList.extend(result)
    except Exception as e:
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        messageID = messageIDCatalog.BROCADE_FABRIC_CONFIGURATION_ERROR
        msgSubstitues = ""
        status = "Critical"
        verbose = ""
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               'BROCADE_FABRIC_REACHABILITY'))
        logger.exception(e)
    if status == "Success" and not exception:
        try:
            status = ""
            result = []
            (status, result) = CredentialChecker(brocaderesourcemgr).check()
            chkResultList.extend(result)
        except Exception as e:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = "HOST_NAME_IP_RESOL_OK"
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            chkResultList.append(utils.checkResult(host,
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   'BROCADE_FABRIC_CREDENTIAL'
                                                   ))
            logger.exception(e)
        if status == "Success":
            for checker in checkers:
                status = ""
                result = []
                try:
                    (status, result) = checker.check()
                    chkResultList.extend(result)
                except Exception as e:
                    logger.exception(e)
                    exception = True

    if exception:
        correctiveAction = "an exception occurred while performing some or"\
            " all validations of this resource. check the log for more details"
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.CA_FABRIC_FETCH_ERROR,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.FABRIC_FETCH_ERROR,
            desired_locale)
        messageID = "HOST_NAME_IP_RESOL_NOTOK"
        msgSubstitues = ""
        status = "Critical"
        verbose = "verbose"
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               'BROCADE_FABRIC_VALIDATION'))


'''
Read Brocade fabric info from fabrics.conf files
'''


def readConf(logger):
    env = {}
    env["brocade-fabric"] = []
    for conf in glob.glob("/etc/cinder/fabrics.conf"):
        try:
            fabric_conf = ConfigParser.ConfigParser()
            fabric_conf.read(conf)
            fabricnames = fabric_conf.get("DEFAULT", "fc_fabric_names")
            for fabric in fabricnames.split(","):
                if fabric_conf.has_option("DEFAULT", "fabric_type_" + fabric):
                    if fabric_conf.get("DEFAULT", "fabric_type_" + fabric) == \
                            "brocade".lower():
                        fabrics = {}
                        fabrics["fc_fabric_type"] = "BROCADEFABRIC"
                        fabrics["hostname"] = fabric_conf.get(
                            "DEFAULT", "fc_fabric_address_" + fabric)
                        fabrics["fabric_name"] = fabric
                        fabrics["fc_fabric_user"] = fabric_conf.get(
                            "DEFAULT", "fc_fabric_user_" + fabric)
                        fabrics["fc_fabric_password"] = fabric_conf.get(
                            "DEFAULT", "fc_fabric_password_" + fabric)
                        fabrics["key-file"] = None
                        env["brocade-fabric"].append(fabrics)
        except Exception as e:
            logger.exception(e)
    return env["brocade-fabric"]


def main():
    rc = 0
    # Parse the command line arguments
    rc = main_func()
    exit(rc)

if __name__ == "__main__":
    main()
