#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------

from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv.resources import os_server_groups
from ttv_validation.ttv_manager.utils import security_utils
import json
from powervc_ttv import _
from powervc_ttv import translate_message

logFile = "collocation_policies_validator.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)

COLLOCATION_POLICIES_NOT_VIOLATED = "COLLOCATION_POLICIES_NOT_VIOLATED"
COLLOCATION_POLICIES_VIOLATED = "COLLOCATION_POLICIES_VIOLATED"


class collocation_policies_checker():

    # Intakes HMC ip, user id and password
    # for connecting and retrieving the HMC details
    def __init__(self, auth_token, tenant_id, desired_locale, logger):
        self.auth_token = auth_token
        self.tenant_id = tenant_id
        self.logger = logger
        self.desired_locale = desired_locale

    # Each validation check should implement this interface
    # to perform the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed
    # using __init__ method
    def check(self):

        chkResultList = []
        collocation_policies_list = os_server_groups.get_server_groups(
            self.auth_token, self.tenant_id, self.logger)
        if (len(collocation_policies_list) == 0):
            return chkResultList
        else:
            self.logger.info(_("Collocation policies have been set."))
            for entry in collocation_policies_list:
                collocation_policy_id = entry["id"]
                collocation_policy_name = entry["name"]
                is_policy_violated = entry["policy_violated"]
                members_info = entry["members"]
                policy_types = entry["policies"]
                correctiveAction = ""
                camsgplaceholders = []
                msgSubstitues = ""
                msgplaceholders = [policy_types[0], collocation_policy_name]
                verbose = ""
                if is_policy_violated:
                    self.logger.info(_("Policies have been violated"))
                    correctiveAction = entry["policy_violated_reason"]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.COLLOCATION_POLICIES_VIOLATED,
                        self.desired_locale)
                    messageID = messageIDCatalog.COLLOCATION_POLICIES_VIOLATED
                    status = "Warning"
                    val_chk_id = COLLOCATION_POLICIES_VIOLATED
                else:
                    self.logger.info(_("Policies have not been violated"))
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.COLLOCATION_POLICIES_NOT_VIOLATED,
                        self.desired_locale)
                    messageID = messageIDCatalog.\
                        COLLOCATION_POLICIES_NOT_VIOLATED
                    status = "Success"
                    val_chk_id = COLLOCATION_POLICIES_NOT_VIOLATED
                chkResult = utils.checkResult(collocation_policy_id,
                                              correctiveAction,
                                              camsgplaceholders,
                                              ttvcatalogmsgID,
                                              msgplaceholders,
                                              messageID,
                                              msgSubstitues,
                                              status, verbose,
                                              val_chk_id)
                chkResultList.append(chkResult)
            return chkResultList


def main():

    # Variable Initialization
    rc = 0
    output_dict = {
        "ValidationID": "COLLOCATION_POLICIES_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": ttvCatalog.COLLOCATION_POLICIES_VALIDATIONS,
        "TargetedResourceType": "Server",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]

    # Logger Initialization
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()

    # I think this validator sud execute in both PowerVM/PowerKVM env.

    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    token = args.auth_token
    auth_token = security_utils.unobfuscate(token)
    tenant_id = args.tenant_id
    if (auth_token is not None and len(auth_token) != 0 and
            tenant_id is not None and len(tenant_id) != 0):
        # Create object for each checker
        collocation_chk = collocation_policies_checker(auth_token,
                                                       tenant_id,
                                                       desired_locale,
                                                       logger)
        chkResultList = collocation_chk.check()
        if (len(chkResultList) == 0):
            logger.info(_("Collocation server groups list is empty."))
        else:
            logger.info(_("Collocation server groups list is not empty."))
            resultList.extend(chkResultList)
    else:
        logger.info(_("Either auth_token or tenant_id is null or empty."))
    logger.info(_("Final Result is:-> {0}".format(
        json.dumps({"ValidationResult": output_dict}))))
    print(json.dumps({"ValidationResult": output_dict}))
    exit(rc)

if __name__ == "__main__":
    main()
