#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import os
import json
import glob
from powervc_ttv.utils import utils
import xml.etree.ElementTree as ET
from powervc_ttv.utils.exceptions import ConfigFileError
import scg_npiv_validation
import scg_vscsi_validation
# from powervc_keystone.encrypthandler import EncryptHandler
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import _
from powervc_ttv import translate_message


base_nova_conf_file = "/etc/nova/nova.conf"

output_dict = {
    "ValidationID": "VIOS_CONNECTIVITY_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "hmc_vios",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}
logger = None
# Log file name. Could be /var/log/ttv_validation/mem_cpu_val.log
logFile = "vios_connectivity_validator.log"


def main_func():
    rc = 0
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    global logger
    logger = chkLogger.getLogger()
    if rc == 1:
        return 1

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.VIOS_CONNECTIVITY_VALIDATOR,
        desired_locale)

    storageinfo = readAllStoragesConf()
    viosinfodict = readVIOSInfoFromDB()
    scg_npiv_vios_list = []
    scg_npiv_vios_dict = viosinfodict["scg_npiv_vios_dict"]
    for vios in scg_npiv_vios_dict.keys():
        scg_npiv_vios_list.append(scg_npiv_vios_dict.get(vios))

    scg_vscsi_vios_list = []
    scg_vscsi_vios_dict = viosinfodict["scg_vscsi_vios_dict"]
    for vios in scg_vscsi_vios_dict.keys():
        scg_vscsi_vios_list.append(scg_vscsi_vios_dict.get(vios))

    fabricinfo = readAllFabricsConf()

    try:
        # config = propertiesforValidation(storageinfo,
        # scg_npiv_vios_list, fabricinfo)
        '''
        call the npiv validation only when at least one type of storage type
        and one type of fabric is registered.
        '''
        if(scg_npiv_vios_dict and (storageinfo["svc-storage"] or
                                   storageinfo["xiv-storage"] or
                                   storageinfo["emcvmax-storage"] or
                                   storageinfo["emcvnx-storage"])):
            npivresult = scg_npiv_validation.validateSCGNPIVConnectivity(
                scg_npiv_vios_list, storageinfo, fabricinfo)
            npivresultdict = json.loads(npivresult)
            readNPIVOuputAfterValidation(npivresultdict)

        '''
        call the vscsi validation only when at least one type of storage type.
        '''
        if(scg_vscsi_vios_dict and (storageinfo["svc-storage"] or
                                    storageinfo["xiv-storage"] or
                                    storageinfo["emcvmax-storage"] or
                                    storageinfo["emcvnx-storage"])):
            vscsiresult = scg_vscsi_validation.validateSCGVSCSIConnectivity(
                scg_vscsi_vios_list, storageinfo)
            vscsiresultdict = json.loads(vscsiresult)
            readVSCSIOuputAfterValidation(vscsiresultdict)

    except Exception as e:
        rc = 1
        logger.exception(e)
    logger.info(_("Consolidated Results: {0}".format(
        json.dumps({"ValidationResult": output_dict}))))
    print(json.dumps({"ValidationResult": output_dict}))
    return rc


'''
Read all the storages information from conf files
'''


def readAllStoragesConf():
    global logger
    env = {}
    env["svc-storage"] = []
    env["xiv-storage"] = []
    env["emcvnx-storage"] = []
    env["emcvmax-storage"] = []
    for conf in glob.glob("/etc/cinder/cinder-*.conf"):
        host_type = ''
        try:
            logger.info("reading cinder.conf file %s", conf)
            cinder_conf = ConfigParser.ConfigParser()
            cinder_conf.read(conf)
            if cinder_conf.has_option('DEFAULT', 'host_type'):
                host_type = cinder_conf.get('DEFAULT', 'host_type').lower()
            if host_type == 'xiv':
                xiv_storage = {}
                xiv_storage["hostname"] = cinder_conf.get("DEFAULT", "san_ip")
                xiv_storage["userid"] = cinder_conf.get("DEFAULT", "san_login")
                xiv_storage["passwd"] = None
                xiv_storage["key-file"] = None
                if cinder_conf.has_option("DEFAULT", "san_private_key"):
                    if os.path.exists(cinder_conf.get("DEFAULT",
                                                      "san_private_key")):
                        keystr = "san_private_key"
                        xiv_storage[
                            "key-file"] = utils.\
                            getClearTextString(cinder_conf.get("DEFAULT",
                                                               keystr))
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    xiv_storage["passwd"] = cinder_conf.get(
                        "DEFAULT", "san_password")
                for prop in ('san_clustername', 'san_ssh_port'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        xiv_storage[prop] = cinder_conf.get('DEFAULT', prop)
                env["xiv-storage"].append(xiv_storage)

            elif host_type == 'svc':
                svc_storage = {}
                svc_storage["hostname"] = cinder_conf.get("DEFAULT", "san_ip")
                svc_storage["userid"] = cinder_conf.get("DEFAULT", "san_login")
                svc_storage["passwd"] = None
                svc_storage["key-file"] = None
                if cinder_conf.has_option("DEFAULT", "san_private_key"):
                    if os.path.exists(cinder_conf.get("DEFAULT",
                                                      "san_private_key")):
                        key = cinder_conf.get("DEFAULT", "san_private_key")
                        svc_storage["key-file"] = utils.getClearTextString(key)
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    svc_storage["passwd"] = cinder_conf.get(
                        "DEFAULT", "san_password")
                for prop in ('storwize_svc_volpool_name', 'san_ssh_port'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        svc_storage[prop] = cinder_conf.get('DEFAULT', prop)
                env["svc-storage"].append(svc_storage)

            elif host_type == 'emc_vmax':
                emcvmax_storage = {}
                emc_conf_file = cinder_conf.get(
                    "DEFAULT", 'cinder_emc_config_file')
                tree = ET.parse(emc_conf_file)
                doc = tree.getroot()
                emcvmax_storage["hostname"] = doc.find('EcomServerIp').text
                emcvmax_storage["userid"] = doc.find('EcomUserName').text
                emcvmax_storage["passwd"] = utils.getClearTextString(
                    doc.find('EcomPassword').text)
                emcvmax_storage["port"] = doc.find('EcomServerPort').text
                emcvmax_storage["key-file"] = None
                env["emcvmax-storage"].append(emcvmax_storage)

            elif host_type == 'emc_vnx':
                emcnvx_storage = {}
                emcnvx_storage["hostname"] = cinder_conf.get(
                    "DEFAULT", "san_ip")
                emcnvx_storage["userid"] = cinder_conf.get(
                    "DEFAULT", "san_login")
                emcnvx_storage["passwd"] = None
                emcnvx_storage["key-file"] = None
                if cinder_conf.has_option("DEFAULT",
                                          "storage_vnx_security_file_dir"):
                    if os.path.exists(cinder_conf.
                                      get("DEFAULT",
                                          "storage_vnx_security_file_dir")):
                        emcnvx_storage["key-file"] = utils.getClearTextString(
                            cinder_conf.get("DEFAULT",
                                            "storage_vnx_security_file_dir"))
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    emcnvx_storage["passwd"] = cinder_conf.get(
                        "DEFAULT", "san_password")
                for prop in ('storage_vnx_pool_names', 'san_ssh_port',
                             'naviseccli_path', 'host_uniqueid',
                             'volume_backend_name'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        emcnvx_storage[prop] = cinder_conf.get('DEFAULT', prop)
                env["emcvnx-storage"].append(emcnvx_storage)
        except ConfigParser.NoOptionError as e:
            logger.error(e)
        except ConfigFileError as e:
            logger.warning(e)
    return env


'''
Read all the fabrics information from conf files
'''


def readAllFabricsConf():
    global logger
    env = {}
    env["FABRICS"] = []
    # fabric.conf file(s)
    for conf in glob.glob("/etc/cinder/fabrics.conf"):
        try:
            logger.info(_("reading fabrics.conf file %s"), conf)
            fabric_conf = ConfigParser.ConfigParser()
            fabric_conf.read(conf)
            fabricnames = fabric_conf.get("DEFAULT", "fc_fabric_names")
            for fabric in fabricnames.split(","):
                fabrics = {}
                if fabric_conf.has_option(
                        "DEFAULT", "fabric_type_" + fabric):
                    if fabric_conf.get("DEFAULT", "fabric_type_" + fabric) ==\
                            "cisco".lower():
                        fabrics["fc_fabric_type"] = "CISCOFABRIC"
                    else:
                        fabrics["fc_fabric_type"] = "BROCADEFABRIC"
                if fabric_conf.has_option(
                        "DEFAULT", "fc_fabric_address_" + fabric):
                    fabrics["hostname"] = fabric_conf.get(
                        "DEFAULT", "fc_fabric_address_" + fabric)
                fabrics["fabric_name"] = fabric
                fabrics["key-file"] = None
                if fabric_conf.has_option(
                        "DEFAULT", "fc_fabric_user_" + fabric):
                    fabrics["fc_fabric_user"] = fabric_conf.get(
                        "DEFAULT", "fc_fabric_user_" + fabric)
                if fabric_conf.has_option(
                        "DEFAULT", "fc_fabric_password_" + fabric):
                    fabrics["fc_fabric_password"] = fabric_conf.get(
                        "DEFAULT", "fc_fabric_password_" + fabric)
                if fabric_conf.has_option(
                        "DEFAULT", "fabric_type_" + fabric):
                    if fabric_conf.\
                        get("DEFAULT",
                            "fabric_type_" + fabric) == "cisco".lower():
                        if fabric_conf.has_option("DEFAULT",
                                                  "fc_fabric_vsan_" +
                                                  fabric):
                            fabrics["fc_fabric_vsan"] = fabric_conf.get(
                                "DEFAULT", "fc_fabric_vsan_" + fabric)
                        if fabric_conf.has_option("DEFAULT",
                                                  "fc_fabric_uniqueid_" +
                                                  fabric):
                            fabrics["fc_fabric_uniqueid"] = fabric_conf.get(
                                "DEFAULT", "fc_fabric_uniqueid_" + fabric)
                env["FABRICS"].append(fabrics)

        except ConfigParser.NoOptionError as e:
            logger.error(e)
        except ConfigFileError as e:
            logger.warning(e)
    return env


'''
Prepare the vscsi output messages after the validation
'''


def readVSCSIOuputAfterValidation(vscsiresultdict):
    val_chk_id = 'VIOS_STORAGE_VSCSI_CONNECTIVITY'
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]
    for key in vscsiresultdict.keys():
        if "ALL_ZONED_NO_HOST_OBJECT" in key:
            '''
            if all the wwpns are zoned and there is no host object
            created on storage
            '''
            msgValue = vscsiresultdict.get(key).split("!!!")
            viosStorageComb = key.split("!!!")

            camsgplaceholders = [""]
            correctiveAction = translate_message(
                ttvCatalog.CA_VIOS_SCG_CONNECTIVITY_ALL_ZONED_NO_HOST_OBJECT,
                desired_locale)
            msgplaceholders = [viosStorageComb[0],
                               viosStorageComb[1].split("!")[0],
                               viosStorageComb[1].split("!")[1], msgValue[0]]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_CONNECTIVITY_ALL_ZONED_NO_HOST_OBJECT,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_CONNECTIVITY_ALL_ZONED_NO_HOST_OBJECT
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            chkResultList.append(utils.checkResult(viosStorageComb[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))
        elif "ALL_ZONED" in key:
            '''
            if all the wwpns are zoned and there is host object
            created on storage
            '''
            msgValue = vscsiresultdict.get(key).split("!!!")
            viosStorageComb = key.split("!!!")

            camsgplaceholders = [""]
            correctiveAction = ''
            msgplaceholders = [viosStorageComb[0],
                               viosStorageComb[1].split("!")[0],
                               viosStorageComb[1].split("!")[1], msgValue[0]]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_CONNECTIVITY_ALL_ZONED,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_CONNECTIVITY_ALL_ZONED
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            chkResultList.append(utils.checkResult(viosStorageComb[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))
        elif "NONE_ZONED" in key:
            '''
            if none of the wwpns of the host are zoned to the storage
            '''
            msgValue = vscsiresultdict.get(key).split("!!!")
            viosStorageComb = key.split("!!!")

            camsgplaceholders = [""]
            correctiveAction = translate_message(
                ttvCatalog.CA_VIOS_SCG_CONNECTIVITY_NONE_ZONED,
                desired_locale)
            msgplaceholders = [viosStorageComb[0],
                               viosStorageComb[1].split("!")[0],
                               viosStorageComb[1].split("!")[1], msgValue[0]]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_CONNECTIVITY_NONE_ZONED,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_CONNECTIVITY_NONE_ZONED
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            chkResultList.append(utils.checkResult(viosStorageComb[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))
        elif "ZONED_NO_HOST_OBJECT" in key:
            '''
            if some of the wwpns are zoned and there is no host object
            created on storage
            '''
            msgValue = vscsiresultdict.get(key).split("!!!")
            viosStorageComb = key.split("!!!")

            camsgplaceholders = [""]
            correctiveAction = translate_message(
                ttvCatalog.CA_VIOS_SCG_CONNECTIVITY_ZONED_NO_HOST_OBJECT,
                desired_locale)
            msgplaceholders = [viosStorageComb[0],
                               viosStorageComb[1].split("!")[0],
                               viosStorageComb[1].split("!")[1],
                               msgValue[0], msgValue[1]]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_CONNECTIVITY_ZONED_NO_HOST_OBJECT,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_CONNECTIVITY_ZONED_NO_HOST_OBJECT
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            chkResultList.append(utils.checkResult(viosStorageComb[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))
        elif "NOT_ZONED" in key:
            '''
            if none of the wwpns are zoned to storage
            '''
            msgValue = vscsiresultdict.get(key).split("!!!")
            viosStorageComb = key.split("!!!")

            camsgplaceholders = [""]
            correctiveAction = translate_message(
                ttvCatalog.CA_VIOS_SCG_CONNECTIVITY_NOT_ZONED,
                desired_locale)
            msgplaceholders = [viosStorageComb[0],
                               viosStorageComb[1].split("!")[0],
                               viosStorageComb[1].split("!")[1],
                               msgValue[0], msgValue[1]]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_CONNECTIVITY_NOT_ZONED,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_CONNECTIVITY_NOT_ZONED
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            chkResultList.append(utils.checkResult(viosStorageComb[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))
        elif "SOME_ZONED" in key:
            '''
            if some of the wwpns are zoned and there is host object
            created on storage
            '''
            msgValue = vscsiresultdict.get(key).split("!!!")
            viosStorageComb = key.split("!!!")

            camsgplaceholders = [""]
            correctiveAction = ''
            msgplaceholders = [viosStorageComb[0],
                               viosStorageComb[1].split("!")[0],
                               viosStorageComb[1].split("!")[1],
                               msgValue[0], msgValue[1]]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_CONNECTIVITY_SOME_ZONED,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_CONNECTIVITY_SOME_ZONED
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            chkResultList.append(utils.checkResult(viosStorageComb[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))

'''
Prepare the NPIV output messages after the validation
'''


def readNPIVOuputAfterValidation(npivresultdict):
    val_chk_id = 'VIOS_STORAGE_NPIV_CONNECTIVITY'
    chkResultList = output_dict["ValidationDetail"]["ChecksList"]
    for key in npivresultdict.keys():
        if len(key.split("!!!")) == 2 and key.split("!!!")[1] ==\
                "NOTCONNECTED":
            '''
            if there is no common fabric between vios and any storage
            '''
            msgValue = key.split("!!!")[0]
            switches = npivresultdict.get(key).split("!!!")[0]
            scgs = npivresultdict.get(key).split("!!!")[1]
            viosStorageComb = key.split("!!!")
            camsgplaceholders = [""]
            correctiveAction = translate_message(
                ttvCatalog.CA_VIOS_SCG_NPIV_CONNECTIVITY_NONECONNECTED,
                desired_locale)
            msgplaceholders = [scgs]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_NPIV_CONNECTIVITY_NONECONNECTED,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_NPIV_CONNECTIVITY_NONECONNECTED
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            chkResultList.append(utils.checkResult(viosStorageComb[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))

        elif len(key.split("!!!")) == 3 and key.split("!!!")[2] ==\
                "NOTCONNECTED":
            '''
            if some fabric is not having common path between
            vios and any storage
            '''
            msgValue = key.split("!!!")
            switches = npivresultdict.get(key).split("!!!")[0]
            scgs = npivresultdict.get(key).split("!!!")[1]

            viosStorageComb = key.split("!!!")

            camsgplaceholders = [""]
            correctiveAction = ''
            msgplaceholders = [msgValue[0], msgValue[1], switches, scgs]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_NPIV_CONNECTIVITY_NOTCONNECTED,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_NPIV_CONNECTIVITY_NOTCONNECTED
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            chkResultList.append(utils.checkResult(msgValue[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))

        elif len(key.split("!!!")) == 3 and key.split("!!!")[2] == "CONNECTED":
            '''
            if some fabric is having common path between vios and any storage.
            '''
            msgValue = key.split("!!!")
            switches = npivresultdict.get(key).split("!!!")[0]
            scgs = npivresultdict.get(key).split("!!!")[1]

            camsgplaceholders = [""]
            correctiveAction = ''
            msgplaceholders = [msgValue[0], msgValue[1], switches, scgs]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.VIOS_SCG_NPIV_CONNECTIVITY_CONNECTED,
                desired_locale)
            messageID = messageIDCatalog.\
                VIOS_SCG_NPIV_CONNECTIVITY_CONNECTED
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            chkResultList.append(utils.checkResult(msgValue[0],
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   val_chk_id))

'''
read the vios and fabric mapping info from DB
'''


def readVIOSInfoFromDB():
    env = {}
    conf = glob.glob(base_nova_conf_file)
    base_nova_conf = ConfigParser.ConfigParser()
    base_nova_conf.read(conf)

    compute_host_name_dict = {}
    for conf in glob.glob("/etc/nova/nova-*.conf"):
        logger.info(_("reading %s file") % conf)
        nova_conf = ConfigParser.ConfigParser()
        nova_conf.read(conf)

        host = nova_conf.get("DEFAULT", "host") if nova_conf.has_option(
            "DEFAULT", "host") else ''
        if nova_conf.has_option(
                "DEFAULT", "host_display_name"):
            host_display_name = nova_conf.get("DEFAULT",
                                              "host_display_name")
        else:
            host_display_name = ''
        compute_host_name_dict[host] = host_display_name

    # Check if there are any duplcate host names present
    conn_text_str = base_nova_conf.get("database", "connection")
    nova_sql_connection = utils.getClearTextString(conn_text_str)

    compute_nodes_dict = {}
    compute_nodes_query = "select compute_nodes.id, compute_nodes." +\
        "hypervisor_hostname from compute_nodes where deleted = 0"
    hyp_query_result = utils.executeOnDB(
        nova_sql_connection, compute_nodes_query)
    for record in hyp_query_result:
        id = record['id']
        compute_nodes_dict[id] = {}
        compute_nodes_dict.get(
            id)["hypervisor_hostname"] = record["hypervisor_hostname"]
    # print"compute_nodes_dict", compute_nodes_dict

    vios_dict = {}
    server_vio_query = "select  server_vio.pk_id, server_vio.lpar_name," +\
        " server_vio.compute_node_id from server_vio where deleted = 0"
    hyp_query_result = utils.executeOnDB(nova_sql_connection, server_vio_query)
    for record in hyp_query_result:
        key = compute_nodes_dict[record["compute_node_id"]].get(
            "hypervisor_hostname")
        # printkey
        if key in compute_host_name_dict is not None:
            id = record['pk_id']
            vios_dict[id] = {}
            vios_dict.get(id)["lpar_name"] = record["lpar_name"]
            vios_dict.get(id)["compute_node_name"] = compute_host_name_dict[
                compute_nodes_dict[record["compute_node_id"]].
                get("hypervisor_hostname")]
        # print"vios_dict", vios_dict

    scg_dict = {}
    storage_connectivity_group_query = "select pk_id, display_name," +\
        " port_tag, npiv_fabric, boot_connectivity, data_connectivity" +\
        " from storage_connectivity_group where enabled = 1 and deleted = 0"
    hyp_query_result = utils.executeOnDB(
        nova_sql_connection, storage_connectivity_group_query)
    for record in hyp_query_result:
        scg_dict[record['pk_id']] = {}
        id = record['pk_id']
        scg_dict.get(id)["display_name"] = record["display_name"]
        scg_dict.get(id)["port_tag"] = record["port_tag"]
        scg_dict.get(id)["npiv_fabric"] = record["npiv_fabric"]
        scg_dict.get(id)["boot_connectivity"] = record["boot_connectivity"]
        scg_dict.get(id)["data_connectivity"] = record["data_connectivity"]
    # print "scg_dict", scg_dict

    vios_scg_dict = {}
    scg_vios_association_query = "select scg_vios_association.scg_pk_id," +\
        " scg_vios_association.vios_pk_id from scg_vios_association" +\
        " where deleted = 0"
    hyp_query_result = utils.executeOnDB(
        nova_sql_connection, scg_vios_association_query)
    for record in hyp_query_result:
        # print record
        id = record['vios_pk_id']
        if id in vios_scg_dict.keys():
            scg_list = vios_scg_dict.get(record['vios_pk_id'])["scg_list"]
            scg_name_list = vios_scg_dict.get(
                record['vios_pk_id'])["scg_name_list"]
            # if record.get('scg_pk_id') not in scg_list:
            scg_list.append(record['scg_pk_id'])
            scg_name_list.append(
                scg_dict.get(record['scg_pk_id'])["display_name"])
        else:
            scg_list = []
            scg_name_list = []
            vios_scg_dict[id] = {}
            scg_list.append(record['scg_pk_id'])
            scg_name_list.append(
                scg_dict.get(record['scg_pk_id'])["display_name"])
            vios_scg_dict.get(id)["scg_list"] = scg_list
            vios_scg_dict.get(id)["scg_name_list"] = scg_name_list
    # print "vios_scg_dict", vios_scg_dict

    vscsi_vios_wwpn_dict = {}
    port_fc_query = "select id,name, wwpn, adapter_id, port_tag as tag," +\
        " fabric, fc_protocol,pk_id, vios_pk_id from port_fc where" +\
        " enabled=1 and deleted=0 and status like 'OK%' " +\
        "and (fc_protocol='vscsi'" +\
        " or fc_protocol='any')"
    hyp_query_result = utils.executeOnDB(nova_sql_connection, port_fc_query)
    for record in hyp_query_result:
        logger.debug("vscsi vios")
        logger.debug(record)
        id = record['vios_pk_id']
        if id in vscsi_vios_wwpn_dict.keys() and id in vios_dict:
            if vscsi_vios_wwpn_dict.get(id)['fc_names'] is not None:
                fc_names = vscsi_vios_wwpn_dict.get(id)['fc_names']
                fc_names.append(record['name'])
            else:
                fc_names = []
                fc_names.append(record['name'])
                vscsi_vios_wwpn_dict.get(id)['fc_names'] = fc_names

            if vscsi_vios_wwpn_dict.get(id)['wwpns'] is not None:
                wwpns = vscsi_vios_wwpn_dict.get(id)['wwpns']
                # if record['wwpn'] not in wwpn_list:
                wwpns.append(record['wwpn'])
            else:
                wwpns = []
                wwpns.append(record['wwpn'])
                vscsi_vios_wwpn_dict.get(id)['wwpns'] = wwpns

            if vscsi_vios_wwpn_dict.get(id)['adapter_ids'] is not None:
                adapter_ids = vscsi_vios_wwpn_dict.get(id)['adapter_ids']
                adapter_ids.append(record['adapter_id'])
            else:
                adapter_ids = []
                adapter_ids.append(record['adapter_id'])
                vscsi_vios_wwpn_dict.get(id)['adapter_ids'] = adapter_ids

            if vscsi_vios_wwpn_dict.get(id)['port_tags'] is not None:
                port_tags = vscsi_vios_wwpn_dict.get(id)['port_tags']
                port_tags.append(record['tag'])
            else:
                port_tags = []
                port_tags.append(record['tag'])
                vscsi_vios_wwpn_dict.get(id)['port_tags'] = port_tags

            if vscsi_vios_wwpn_dict.get(id)['fabrics'] is not None:
                fabrics = vscsi_vios_wwpn_dict.get(id)['fabrics']
                fabrics.append(record['fabric'])
            else:
                fabrics = []
                fabrics.append(record['fabric'])
                vscsi_vios_wwpn_dict.get(id)['fabrics'] = fabrics

            if vscsi_vios_wwpn_dict.get(id)['fc_protocols'] is not None:
                fc_protocols = vscsi_vios_wwpn_dict.get(id)['fc_protocols']
                fc_protocols.append(record['fc_protocol'])
            else:
                fc_protocols = []
                fc_protocols.append(record['fc_protocol'])
                vscsi_vios_wwpn_dict.get(id)['fc_protocols'] = fc_protocols

        elif id in vios_dict:
            vscsi_vios_wwpn_dict[id] = {}
            fc_names = []
            fc_names.append(record['name'])
            vscsi_vios_wwpn_dict.get(id)['fc_names'] = fc_names

            wwpns = []
            wwpns.append(record['wwpn'])
            vscsi_vios_wwpn_dict.get(id)['wwpns'] = wwpns

            port_tags = []
            port_tags.append(record['tag'])
            vscsi_vios_wwpn_dict.get(id)['port_tags'] = port_tags

            adapter_ids = []
            adapter_ids.append(record['adapter_id'])
            vscsi_vios_wwpn_dict.get(id)['adapter_ids'] = adapter_ids

            fabric = []
            fabric.append(record['fabric'])
            vscsi_vios_wwpn_dict.get(id)['fabrics'] = fabric

            fc_protocols = []
            fc_protocols.append(record['fc_protocol'])
            vscsi_vios_wwpn_dict.get(id)['fc_protocols'] = fc_protocols

        if id in vios_dict and id in vios_scg_dict:
            vscsi_vios_wwpn_dict.get(
                id)["name"] = vios_dict.get(id)["lpar_name"]
            vscsi_vios_wwpn_dict.get(id)["host"] = vios_dict.get(
                id)["compute_node_name"]
            vscsi_vios_wwpn_dict.get(id)["scgs"] = vios_scg_dict.get(id)

    npiv_vscsi_vios_wwpn_dict = {}

    port_fc_query = "select id,name, wwpn, adapter_id, port_tag as tag," +\
        " fabric, fc_protocol,pk_id, vios_pk_id from port_fc" +\
        " where enabled=1 and deleted=0 and status like 'OK%'" +\
        " and (fc_protocol='npiv' or" +\
        " fc_protocol='any') and fabric <> 'None'"
    hyp_query_result = utils.executeOnDB(nova_sql_connection, port_fc_query)
    for record in hyp_query_result:
        logger.debug("npiv vios")
        logger.debug(record)
        id = record['vios_pk_id']
        if id in npiv_vscsi_vios_wwpn_dict.keys() and id in vios_dict:
            if npiv_vscsi_vios_wwpn_dict.get(id)['fc_names'] is not None:
                fc_names = npiv_vscsi_vios_wwpn_dict.get(id)['fc_names']
                fc_names.append(record['name'])
            else:
                fc_names = []
                fc_names.append(record['name'])
                npiv_vscsi_vios_wwpn_dict.get(id)['fc_names'] = fc_names

            if npiv_vscsi_vios_wwpn_dict.get(id)['wwpns'] is not None:
                wwpns = npiv_vscsi_vios_wwpn_dict.get(id)['wwpns']
                # if record['wwpn'] not in wwpn_list:
                wwpns.append(record['wwpn'])
            else:
                wwpns = []
                wwpns.append(record['wwpn'])
                npiv_vscsi_vios_wwpn_dict.get(id)['wwpns'] = wwpns

            if npiv_vscsi_vios_wwpn_dict.get(id)['adapter_ids'] is not None:
                adapter_ids = npiv_vscsi_vios_wwpn_dict.get(id)['adapter_ids']
                adapter_ids.append(record['adapter_id'])
            else:
                adapter_ids = []
                adapter_ids.append(record['adapter_id'])
                npiv_vscsi_vios_wwpn_dict.get(id)['adapter_ids'] = adapter_ids

            if npiv_vscsi_vios_wwpn_dict.get(id)['port_tags'] is not None:
                port_tags = npiv_vscsi_vios_wwpn_dict.get(id)['port_tags']
                port_tags.append(record['tag'])
            else:
                port_tags = []
                port_tags.append(record['tag'])
                npiv_vscsi_vios_wwpn_dict.get(id)['port_tags'] = port_tags

            if npiv_vscsi_vios_wwpn_dict.get(id)['fabrics'] is not None:
                fabrics = npiv_vscsi_vios_wwpn_dict.get(id)['fabrics']
                fabrics.append(record['fabric'])
            else:
                fabrics = []
                fabrics.append(record['fabric'])
                npiv_vscsi_vios_wwpn_dict.get(id)['fabrics'] = fabrics

            if npiv_vscsi_vios_wwpn_dict.get(id)['fc_protocols'] is not None:
                fc_protocols = npiv_vscsi_vios_wwpn_dict.get(
                    id)['fc_protocols']
                fc_protocols.append(record['fc_protocol'])
            else:
                fc_protocols = []
                fc_protocols.append(record['fc_protocol'])
                npiv_vscsi_vios_wwpn_dict.get(
                    id)['fc_protocols'] = fc_protocols

        elif id in vios_dict:
            npiv_vscsi_vios_wwpn_dict[id] = {}
            fc_names = []
            fc_names.append(record['name'])
            npiv_vscsi_vios_wwpn_dict.get(id)['fc_names'] = fc_names

            wwpns = []
            wwpns.append(record['wwpn'])
            npiv_vscsi_vios_wwpn_dict.get(id)['wwpns'] = wwpns

            port_tags = []
            port_tags.append(record['tag'])
            npiv_vscsi_vios_wwpn_dict.get(id)['port_tags'] = port_tags

            adapter_ids = []
            adapter_ids.append(record['adapter_id'])
            npiv_vscsi_vios_wwpn_dict.get(id)['adapter_ids'] = adapter_ids

            fabrics = []
            fabrics.append(record['fabric'])
            npiv_vscsi_vios_wwpn_dict.get(id)['fabrics'] = fabrics

            fc_protocols = []
            fc_protocols.append(record['fc_protocol'])
            npiv_vscsi_vios_wwpn_dict.get(id)['fc_protocols'] = fc_protocols

        if id in vios_dict and id in vios_scg_dict:
            npiv_vscsi_vios_wwpn_dict.get(
                id)["name"] = vios_dict.get(id)["lpar_name"]
            npiv_vscsi_vios_wwpn_dict.get(
                id)["host"] = vios_dict.get(id)["compute_node_name"]
            npiv_vscsi_vios_wwpn_dict.get(id)["scgs"] = vios_scg_dict.get(id)

    env["scg_npiv_vios_dict"] = npiv_vscsi_vios_wwpn_dict
    logger.debug('env["scg_npiv_vios_dict"]')
    logger.debug(env["scg_npiv_vios_dict"])

    # print "vscsi_vios_wwpn_dict",vscsi_vios_wwpn_dict
    # print "npiv_vscsi_vios_wwpn_dict", npiv_vscsi_vios_wwpn_dict

    env["scg_vscsi_vios_dict"] = vscsi_vios_wwpn_dict
    logger.debug('env["scg_vscsi_vios_dict"]')
    logger.debug(env["scg_vscsi_vios_dict"])
    return env


def main():
    rc = 0
    # Parse the command line arguments
    rc = main_func()
    exit(rc)

if __name__ == "__main__":
    main()
