#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import json
import glob
import re
from powervc_ttv.utils import utils
import xml.etree.ElementTree as ET
from powervc_ttv.resources import EMCVMAXStorageResource
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import translate_message

logger = None
logFile = "emc_vmax.log"
base_nova_conf_file = "/etc/nova/nova.conf"
PING_COMMAND = "ping -c 4 -w 10 "
PING6_COMMAND = "ping6 -c 4 -w 10 "
DEFAULT_SECTION = "DEFAULT"
emcvmax_validations_dict = {}
RequiredSMISFirmware = 'RequiredSMISFirmware'
EMCVMAXCredentialValid = 'EMCVMAXCredentialValid'
EMCVMAXReachable = 'EMCVMAXReachable'

self_version = None
RequiredSMISFirmware = "RequiredSMISFirmware"
SymmetricArrayModelValid = "SymmetricArrayModelValid"
LicensesValid = "LicensesValid"
EMC_VMAX_CRED_CHK = "EMC_VMAX_CRED_CHK"
EMC_VMAX_REACHABILITY_CHK = "EMC_VMAX_REACHABILITY_CHK"
EMC_VMAX_CHK = "EMC_VMAX_CHK"
EMC_VMAX_SYMMETRIC_ARRAY_CHK = "EMC_VMAX_SYMMETRIC_ARRAY_CHK"
EMC_VMAX_LICENSE_CHK = "EMC_VMAX_LICENSE_CHK"
NETWORK_QUALITY_CHK = "NETWORK_QUALITY_CHK"


output_dict = {
    "ValidationID": "EMC_VMAX_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "hmc",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}
chkResultList = output_dict["ValidationDetail"]["ChecksList"]


class CredentialChecker():
    # checks for the credentials of the resources
    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        credential = self.resource.checkCredential()
        if credential == 0:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_VALID,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 EMC_VMAX_CRED_CHK))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 EMC_VMAX_CRED_CHK))
        return (status, self.result)


class ReachabilityChecker():
    # checks for the reachability of the resources from management server
    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        reachability = utils.checkReachability(host)
        if reachability == 0:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.resource.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 EMC_VMAX_REACHABILITY_CHK))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = "RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER"
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(self.resource.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 EMC_VMAX_REACHABILITY_CHK))
        return (status, self.result)


def readConfCinder(logger):
    env = {}
    env["emcvmax-storage"] = []
    for conf in glob.glob("/etc/cinder/cinder-*.conf"):
        if conf == "/etc/cinder/cinder-health.conf":
            continue
        try:
            cinder_conf = ConfigParser.ConfigParser()
            cinder_conf.read(conf)
            if cinder_conf.has_option('DEFAULT', 'host_type'):
                host_type = cinder_conf.get('DEFAULT', 'host_type')
            if host_type.lower() == 'emc_vmax':
                emcvmax_storage = {}
                emc_conf_file = cinder_conf.get("DEFAULT",
                                                'cinder_emc_config_file')
                tree = ET.parse(emc_conf_file)
                doc = tree.getroot()
                ip = doc.find('EcomServerIp')
                # resolveHostname(doc.find('EcomServerIp').text)
                emcvmax_storage["hostname"] = doc.find('EcomServerIp').text
                emcvmax_storage["ip"] = ip
                emcvmax_storage["userid"] = doc.find('EcomUserName').text
                emcvmax_storage["passwd"] = utils.getClearTextString(
                    doc.find('EcomPassword').text)
                emcvmax_storage["port"] = doc.find('EcomServerPort').text
                env["emcvmax-storage"].append(emcvmax_storage)
        except Exception as e:
            logger.exception(e)

    return env["emcvmax-storage"]


def validateResource(validationProp, individual_dict,
                     resource_dict_cinder, vmax_resource_obj, con,
                     logger=None):
    exception = False
    host = individual_dict['hostname']
    checkers = []

    checkers.append(
        validateEMCVMAXChecker(
            individual_dict,
            vmax_resource_obj,
            validationProp,
            con))
    checkers.append(
        validateEMCSymmetricArrayChecker(
            individual_dict,
            vmax_resource_obj,
            validationProp,
            con))
    checkers.append(
        validateEMCVMAXLicenseChecker(
            individual_dict,
            vmax_resource_obj,
            validationProp,
            con))

    try:
        # checks for the reachability of the resources from management server
        rc = ReachabilityChecker(vmax_resource_obj)
        (status, result) = rc.check()
        chkResultList.extend(result)
    except Exception as e:
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        messageID = messageIDCatalog.\
            MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
        msgSubstitues = ""
        status = "Critical"
        verbose = ""
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose))
        logger.exception(e)
    if status == "Success" and not exception:
        try:
            status = ""
            result = []
            """checks for the network quality
            from the management server to the resource"""
            (status, result) = NetworkQualityChecker(
                individual_dict["hostname"]).check()
            chkResultList.extend(result)
            """checks for the credentials
            of the resource resource"""
            (status, result) = CredentialChecker(vmax_resource_obj).check()
            chkResultList.extend(result)
        except Exception as e:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            chkResultList.append(utils.checkResult(host,
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose))
            logger.exception(e)

        if status == "Success":
            for checker in checkers:
                status = ""
                result = []
                try:
                    (status, result) = checker.check()
                    chkResultList.extend(result)
                except Exception as e:
                    logger.exception(e)
                    exception = True

    if exception:
        correctiveAction = "an exception occurred while performing some or"\
            " all validations of this resource. check the log for more details"
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.CA_FABRIC_FETCH_ERROR,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.FABRIC_FETCH_ERROR,
            desired_locale)
        messageID = messageIDCatalog.\
            MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
        msgSubstitues = ""
        status = "Critical"
        verbose = "verbose"
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose))


"""checks for the network quality
from the management server to the resource"""


class NetworkQualityChecker():

    def __init__(self, ip):
        self.ip = ip
        self.result = []

    def check(self):
        status = ""
        host = self.ip
        quality = utils.checkNetworkQuality(self.ip)
        quality = int(quality)
        # if all the packets are dropped
        if quality == 100:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_BAD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        # if packets drop count is more than 5%
        elif quality >= 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_DEGRADED,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        # if packets drop count is less than 5%
        elif quality < 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_GOOD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))

        return (status, self.result)


def main_func():
    rc = 0
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    global logger
    logger = chkLogger.getLogger()

    global desired_locale
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.EMC_VMAX_VALIDATIONS,
        desired_locale)

    conf = glob.glob(base_nova_conf_file)
    base_nova_conf = ConfigParser.ConfigParser()
    base_nova_conf.read(conf)
    powervc_edition = ''

    if base_nova_conf.has_option("DEFAULT", "powervm_mgr_type"):
        powervc_edition = base_nova_conf.get("DEFAULT", "powervm_mgr_type")
    else:
        powervc_edition = base_nova_conf.get("DEFAULT", "host_type")
    if powervc_edition == 'HMC' or powervc_edition == 'hmc':

        "cinder nodes info"
        resource_dict_cinder = readConfCinder(logger)
        ev = "emc_vmax"
        validationProp = utils.readJSONFile(
            "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json")[ev]

        for index in range(len(resource_dict_cinder)):
            try:
                individual_dict = resource_dict_cinder[index]

                emcvmaxip = individual_dict['ip']
                emcvmaxUser = individual_dict['userid']
                if individual_dict['passwd'] is not None:
                    emcvmaxPasswd = utils.getClearTextString(
                        individual_dict['passwd'])

                port = int(individual_dict['port']) \
                    if individual_dict['port'] else 5989

                vmax_resource_obj = EMCVMAXStorageResource.\
                    EMCVMAXStorageResource(individual_dict, logger)
                con = vmax_resource_obj._get_ecom_connection(
                    emcvmaxip,
                    emcvmaxUser,
                    emcvmaxPasswd,
                    port)

                validateResource(
                    validationProp,
                    individual_dict,
                    resource_dict_cinder,
                    vmax_resource_obj,
                    con,
                    logger)

            except Exception as e:
                logger.exception(e)
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    else:
        print(json.dumps({"ValidationResult": output_dict}))
        return rc


def validate_version(versionObj, exp_firmware_version):
    version_name = versionObj['VersionString']
    tempString = ''
    firstIteration = True
    if version_name[0] == 'V' or version_name[0] == 'v':
        for char in version_name:
            if firstIteration:
                firstIteration = False
            else:
                tempString = tempString + char
    version_name = tempString

    return utils.compareVersion(version_name, exp_firmware_version)


def validate_array(current_arrayObj, exp_array_pattern):

    is_pattern_found = False
    version_name = current_arrayObj['Version'].strip()
    match = re.search(exp_array_pattern, version_name)
    if match is not None:
        is_pattern_found = True
    return (is_pattern_found, version_name)


class validateEMCVMAXLicenseChecker():
    # checks for the EMC VMAX License
    def __init__(self, resource, vmax_res_obj, exp_values, conn):
        self.resource = resource
        self.exp_values = exp_values
        self.vmax_res_obj = vmax_res_obj
        self.conn = conn
        self.result = []

    def check(self):

        status = ""
        host = self.resource['hostname']
        is_valid, return_value = self.vmax_res_obj.get_vmax_license_info()
        if return_value == "Failed_to_get":
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVMAX_LICENSE_ERROR,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVMAX_LICENSE_ERROR,
                desired_locale)
            messageID = messageIDCatalog.EMCVMAX_LICENSE_ERROR
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(host,
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  EMC_VMAX_LICENSE_CHK))
        elif return_value == "Valid":
            camsgplaceholders = []
            correctiveAction = ""
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVMAX_LICENSE_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.EMCVMAX_LICENSE_SUCCESS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(
                utils.checkResult(host,
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  EMC_VMAX_LICENSE_CHK))
        elif return_value == "Invalid":
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVMAX_LICENSE_FAIL,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVMAX_LICENSEL_FAIL,
                desired_locale)
            messageID = messageIDCatalog.EMCVMAX_LICENSEL_FAIL
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(
                utils.checkResult(host,
                                  correctiveAction,
                                  camsgplaceholders,
                                  ttvcatalogmsgID,
                                  msgplaceholders,
                                  messageID,
                                  msgSubstitues,
                                  status, verbose,
                                  EMC_VMAX_LICENSE_CHK))

        return (status, self.result)


class validateEMCSymmetricArrayChecker():

    def __init__(self, resource, vmax_res_obj, exp_values, conn):
        self.resource = resource
        self.vmax_res_obj = vmax_res_obj
        self.conn = conn
        self.exp_symmetric_array_model = exp_values['SymmetricArrayModelValid']
        self.result = []

    def check(self):

        status = ""
        host = self.resource['hostname']

        # get symmetric array version
        is_conn_valid, current_symm_array = self.vmax_res_obj.\
            get_vmax_symmetric_array()

        if not is_conn_valid and current_symm_array == 'Failed_to_get':
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVMAX_SYMMETRIC_ARRAY_MODEL_ERROR,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVMAX_SYMMETRIC_ARRAY_MODEL_ERROR,
                desired_locale)
            messageID = messageIDCatalog.EMCVMAX_SYMMETRIC_ARRAY_MODEL_ERROR
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        else:
            is_pattern_found, current_symm_array_name =\
                validate_array(current_symm_array,
                               self.exp_symmetric_array_model)
            if is_pattern_found:
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.EMCVMAX_SYMMETRIC_ARRAY_MODEL_SUCCESS,
                    desired_locale)
                messageID =\
                    messageIDCatalog.EMCVMAX_SYMMETRIC_ARRAY_MODEL_SUCCESS
                msgSubstitues = ""
                status = "Success"
                verbose = ""
            else:
                camsgplaceholders = [self.exp_symmetric_array_model]
                correctiveAction = translate_message(
                    ttvCatalog.CA_EMCVMAX_SYMMETRIC_ARRAY_MODEL_FAIL,
                    desired_locale)
                msgplaceholders = [current_symm_array_name]
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.EMCVMAX_SYMMETRIC_ARRAY_MODEL_FAIL,
                    desired_locale)
                messageID = messageIDCatalog.EMCVMAX_SYMMETRIC_ARRAY_MODEL_FAIL
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
        self.result.append(
            utils.checkResult(host,
                              correctiveAction,
                              camsgplaceholders,
                              ttvcatalogmsgID,
                              msgplaceholders,
                              messageID,
                              msgSubstitues,
                              status, verbose,
                              EMC_VMAX_SYMMETRIC_ARRAY_CHK))

        return (status, self.result)


class validateEMCVMAXChecker():
    # checks for the EMC VMAX version
    def __init__(self, resource, vmax_res_obj, exp_values, conn):
        self.resource = resource
        self.vmax_res_obj = vmax_res_obj
        self.conn = conn
        self.exp_SMIS_firmware = exp_values['RequiredSMISFirmware']
        self.result = []

    def check(self):

        status = ""
        host = self.resource['hostname']

        '''validation for Firmware'''
        # read actual firmware version
        is_conn_valid, current_version = self.vmax_res_obj.get_vmax_version()
        if not is_conn_valid and current_version == 'Failed_to_get':
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_EMCVMAX_FIRMWARE_ERROR,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.EMCVMAX_FIRMWARE_ERROR,
                desired_locale)
            messageID = messageIDCatalog.EMCVMAX_FIRMWARE_ERROR
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
        else:
            is_firmware_version_valid =\
                validate_version(current_version, self.exp_SMIS_firmware)
            if is_firmware_version_valid:
                # Returned True
                camsgplaceholders = []
                correctiveAction = ""
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.EMCVMAX_FIRMWARE_SUCCESS,
                    desired_locale)
                messageID = messageIDCatalog.EMCVMAX_FIRMWARE_SUCCESS
                msgSubstitues = ""
                status = "Success"
                verbose = ""
            else:
                camsgplaceholders = [self.exp_SMIS_firmware]
                correctiveAction = translate_message(
                    ttvCatalog.CA_EMCVMAX_FIRMWARE_FAIL,
                    desired_locale)
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.EMCVMAX_FIRMWARE_FAIL,
                    desired_locale)
                messageID = messageIDCatalog.EMCVMAX_FIRMWARE_FAIL
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
        self.result.append(
            utils.checkResult(host,
                              correctiveAction,
                              camsgplaceholders,
                              ttvcatalogmsgID,
                              msgplaceholders,
                              messageID,
                              msgSubstitues,
                              status, verbose,
                              EMC_VMAX_CHK))

        return (status, self.result)


def main():
    rc = 0
    # Parse the command line arguments
    rc = main_func()
    exit(rc)


if __name__ == "__main__":
    main()
