#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_ttv.utils import utils
from base_validator import cmdHandler
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import translate_message
import json
import six

base_nova_conf_file = "/etc/nova/nova.conf"
NOT_RUNNING = ttvCatalog.MANAGEMENT_SERVER_OPERATING_SYSTEM_SERVICE_NOT_RUNNING
RUNNING = ttvCatalog.MANAGEMENT_SERVER_OPERATING_SYSTEM_SERVICE_RUNNING

# Class implements Ops Manager Runtime services check


'''Get management server service status
'''


class Ops_Manager_Runtime_Services():

    def __init__(self, id, logger, timeout):
        self.cmdHandle = cmdHandler(logger, timeout)
        self.id = id
        self.logger = logger

    # Each validation check should implement this method to perform the check
    # There should be no argument to this method.
    # Any argument / input to this method should be passed
    # using __init__ method
    # This method should return the list containing dicts
    # created by _checkResult
    def check(self):
        checkServiceList = []
        correctiveAction = ""
        camsgplaceholders = []
        ttvcatalogmsgID = ""
        msgplaceholders = []
        messageID = ""
        msgSubstitues = ""
        status = ""
        verbose = ""

        services = ["sshd", "openstack-nova-compute",
                    "rabbitmq-server", "openstack-nova-conductor",
                    "keystone-public", "keystone-admin",
                    "openstack-nova-ibm-notification",
                    "powervc-db2",
                    "openstack-nova-ibm-ego-ha-service",
                    "openstack-nova-scheduler",
                    "openstack-nova-api", "openstack-glance-registry",
                    "openstack-glance-api", "openstack-cinder-volume",
                    "openstack-cinder-scheduler", "openstack-cinder-api",
                    "openstack-ceilometer-collector",
                    "openstack-ceilometer-api",
                    "neutron-server",
                    "openstack-nova-ibm-ego-resource-optimization",
                    "powervc-nova-ibm-health",
                    "powervc-cinder-ibm-health"
                    ]

        for service in services:
            val_check_id = service.upper()
            if service in ("keystone-public", "keystone-admin"):
                cmd_service = "ps aux | awk '{print $1,$2,$11}'| grep keystone"
                " | grep %s | awk '{ print $2 }' " % (service)
                (rc, cmdOutput, cmd_err) = self.cmdHandle.cmd_run(cmd_service)

            elif service.strip() == "openstack-nova-compute":
                cmd = "ls /etc/init.d | grep openstack-nova-compute-*"
                (rc, cmdOutput, cmd_err) = self.cmdHandle.cmd_run(cmd)
                lists = cmdOutput.split('\n')
                for item in lists:
                    if item == '':
                        lists.remove(item)
                for l1 in lists:
                    cmd = "service %s  status" % (l1)
                    (rc, cmdOutput, cmd_err) = self.cmdHandle.cmd_run(cmd)

                    if rc == 0:
                        correctiveAction = ""
                        camsgplaceholders = [l1, l1]
                        ttvcatalogmsgID = six.text_type(
                            translate_message(
                                ttvCatalog.
                                MANAGEMENT_SERVER_OPEN_STACK_SERVICE,
                                desired_locale)) + " " + six.text_type(
                                    translate_message(RUNNING,
                                                      desired_locale))
                        msgplaceholders = [l1, l1]
                        messageID = messageIDCatalog.\
                            MANAGEMENT_SERVER_OPENSTACK_SERVICE
                        status = "Success"
                        verbose = ""
                    elif rc == 1:
                        if cmd_err != '':
                            correctiveAction =\
                                "Configure the service nova"
                            camsgplaceholders = ['', '']

                            ttvcatalogmsgID = "service nova is not recognized."
                            msgplaceholders = ['', '']
                            messageID =\
                                messageIDCatalog.\
                                MANAGEMENT_SERVER_SERVICES_ERROR
                            status = "Critical"
                            verbose = ""

                        else:
                            correctiveAction =\
                                "Try starting service nova"
                            camsgplaceholders = ['', '']

                            ttvcatalogmsgID = six.text_type(
                                translate_message(
                                    ttvCatalog.
                                    MANAGEMENT_SERVER_OPEN_STACK_SERVICE,
                                    desired_locale)) + " " + six.text_type(
                                        translate_message(
                                            NOT_RUNNING,
                                            desired_locale))
                            msgplaceholders = [l1, l1]

                            messageID =\
                                messageIDCatalog.\
                                MANAGEMENT_SERVER_SERVICES_ERROR
                            status = "Warning"
                            verbose = ""

                    else:

                        correctiveAction = "Check log for details"
                        camsgplaceholders = ['', '']

                        ttvcatalogmsgID = "Failed to fetch the status of " + \
                            l1 + "service"
                        msgplaceholders = ['', '']

                        messageID = messageIDCatalog.\
                            MANAGEMENT_SERVER_SERVICES_ERROR
                        status = "Critical"
                        msgSubstitues = ""
                        verbose = ""
                    chkResult = utils.checkResult(utils.get_host_name(),
                                                  correctiveAction,
                                                  camsgplaceholders,
                                                  ttvcatalogmsgID,
                                                  msgplaceholders,
                                                  messageID,
                                                  msgSubstitues,
                                                  status, verbose,
                                                  val_check_id)
                    checkServiceList.append(chkResult)

            elif service.strip() == "openstack-cinder-volume":
                cmd = "ls /etc/init.d | grep openstack-cinder-volume-*"
                (rc, cmdOutput, cmd_err) = self.cmdHandle.cmd_run(cmd)
                lists = cmdOutput.split('\n')
                for item in lists:
                    if item == '':
                        lists.remove(item)
                for l1 in lists:
                    cmd = "service %s  status" % (l1)
                    (rc, cmdOutput, cmd_err) = self.cmdHandle.cmd_run(cmd)

                    if rc == 0:
                        correctiveAction = ""
                        camsgplaceholders = [l1, l1]
                        ttvcatalogmsgID = six.text_type(
                            translate_message(
                                ttvCatalog.
                                MANAGEMENT_SERVER_OPEN_STACK_SERVICE,
                                desired_locale)) + " " + six.text_type(
                                    translate_message(RUNNING,
                                                      desired_locale))
                        msgplaceholders = [l1, l1]
                        messageID = messageIDCatalog.\
                            MANAGEMENT_SERVER_OPENSTACK_SERVICE
                        status = "Success"
                        verbose = ""
                    elif rc == 1:
                        if cmd_err != '':
                            correctiveAction =\
                                "Configure the service cinder"
                            camsgplaceholders = ['', '']
                            ttvcatalogmsgID = \
                                "service cinder is not recognized."
                            msgplaceholders = ['', '']
                            messageID =\
                                messageIDCatalog.\
                                MANAGEMENT_SERVER_SERVICES_ERROR
                            status = "Critical"
                            verbose = ""

                        else:
                            correctiveAction =\
                                "Try starting service cinder"
                            camsgplaceholders = ['', '']

                            ttvcatalogmsgID = six.text_type(
                                translate_message(
                                    ttvCatalog.
                                    MANAGEMENT_SERVER_OPEN_STACK_SERVICE,
                                    desired_locale)) + " " + six.text_type(
                                        translate_message(NOT_RUNNING,
                                                          desired_locale))
                            msgplaceholders = [l1, l1]

                            messageID =\
                                messageIDCatalog.\
                                MANAGEMENT_SERVER_SERVICES_ERROR
                            status = "Warning"
                            verbose = ""

                    else:

                        correctiveAction = "Check log for details"
                        camsgplaceholders = ['', '']

                        ttvcatalogmsgID = "Failed to fetch the status of " + \
                            l1 + "service"
                        msgplaceholders = ['', '']

                        messageID = messageIDCatalog.\
                            MANAGEMENT_SERVER_OS_SERVICE_ERROR
                        status = "Critical"
                        msgSubstitues = ""
                        verbose = ""

                    chkResult = utils.checkResult(utils.get_host_name(),
                                                  correctiveAction,
                                                  camsgplaceholders,
                                                  ttvcatalogmsgID,
                                                  msgplaceholders,
                                                  messageID,
                                                  msgSubstitues,
                                                  status, verbose,
                                                  val_check_id)
                    checkServiceList.append(chkResult)
            else:
                if service.strip() == "powervc-db2":
                    cmd = "sudo /usr/bin/systemctl status %s" % (service)
                else:
                    cmd = "service %s status" % (service)

                (rc, cmdOutput, cmd_err) = self.cmdHandle.cmd_run(cmd)

            if rc == 0 and service.strip(
            ) != "openstack-cinder-volume" \
                    and service.strip() != 'openstack-nova-compute':
                correctiveAction = ""
                if service.strip() == "sshd":
                    ttvcatalogmsgID = six.text_type(
                        translate_message(
                            ttvCatalog.
                            MANAGEMENT_SERVER_OPERATING_SYSTEM_SERVICE,
                            desired_locale)) + " " + six.text_type(
                                translate_message(RUNNING,
                                                  desired_locale))
                    messageID = messageIDCatalog.MANAGEMENT_SERVER_OS_SERVICE
                else:
                    ttvcatalogmsgID = six.text_type(
                        translate_message(
                            ttvCatalog.MANAGEMENT_SERVER_OPEN_STACK_SERVICE,
                            desired_locale)) + " " + six.text_type(
                                translate_message(RUNNING,
                                                  desired_locale))
                    messageID = messageIDCatalog.\
                        MANAGEMENT_SERVER_OPENSTACK_SERVICE

                msgplaceholders = [service, service]

                status = "Success"
            elif rc == 1 and service.strip() != 'openstack-cinder-volume' \
                    and service.strip() != 'openstack-nova-compute':
                correctiveAction = six.text_type(
                    translate_message(
                        ttvCatalog.CA_MANAGEMENT_SERVER_SERVICE_START,
                        desired_locale))
                camsgplaceholders = [service, service]
                if (service.strip() == "keystone-admin" or
                        service.strip() == "keystone-public"):
                    correctiveAction = six.text_type(
                        translate_message(
                            ttvCatalog.
                            CA_MANAGEMENT_SERVER_KEYSTONE_SERVICE_RESTART,
                            desired_locale))
                    camsgplaceholders = [service, "keystone", "httpd", "httpd"]
                elif (service.strip() == "openstack-cinder-scheduler" or
                      service.strip() == "openstack-cinder-api"):
                    camsgplaceholders = [service, "cinder"]
                elif (service.strip() == "powervc-db2"):
                    camsgplaceholders = [service, "db2"]
                if service.strip() == "sshd":
                    ttvcatalogmsgID = six.text_type(
                        translate_message(
                            ttvCatalog.
                            MANAGEMENT_SERVER_OPERATING_SYSTEM_SERVICE,
                            desired_locale)) + " " + six.text_type(
                                translate_message(NOT_RUNNING,
                                                  desired_locale))
                else:
                    ttvcatalogmsgID = six.text_type(
                        translate_message(
                            ttvCatalog.MANAGEMENT_SERVER_OPEN_STACK_SERVICE,
                            desired_locale)) + " " + six.text_type(
                                translate_message(NOT_RUNNING,
                                                  desired_locale))
                msgplaceholders = [service]
                messageID =\
                    messageIDCatalog.MANAGEMENT_SERVER_SERVICES_ERROR
                status = "Critical"
            else:
                if service.strip(
                ) != 'openstack-cinder-volume' \
                        and \
                        service.strip() != 'openstack-nova-compute':
                    correctiveAction = "Check log for details"

                    camsgplaceholders = []
                    ttvcatalogmsgID = "Failed to fetch the status of " + \
                        service.strip() + "service"
                    if service.strip() == "sshd":
                        messageID = messageIDCatalog.\
                            MANAGEMENT_SERVER_OS_SERVICE_ERROR
                    else:
                        messageID = messageIDCatalog.\
                            MANAGEMENT_SERVER_OPENSTACK_SERVICE_ERROR
                    status = "Critical"
                    msgSubstitues = ''

            if service.strip(
            ) != 'openstack-cinder-volume' \
                    and service.strip() != 'openstack-nova-compute':
                chkResult = utils.checkResult(utils.get_host_name(),
                                              correctiveAction,
                                              camsgplaceholders,
                                              ttvcatalogmsgID,
                                              msgplaceholders,
                                              messageID,
                                              msgSubstitues,
                                              status, verbose,
                                              val_check_id)
                checkServiceList.append(chkResult)

        return checkServiceList


def main():
    rc = 0
    output_dict = {
        "ValidationID": "MANAGEMENT_SERVER_SERVICES_VALIDATOR",
        "ValidationStatus": "Success",
        "Description": "",
        "TargetedResourceType": "Management Server",
        "IpAddress": "",
        "refs": [],
        "ValidationDetail": {
            "ChecksList": []
        },
    }
    resultList = output_dict["ValidationDetail"]["ChecksList"]

    logFile = "management_server_services_val.log"
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    logger = chkLogger.getLogger()
    timeout = 60
    id = "OPSM_RUNTIME_SERVICES"
    args = utils.parse_arguments()
    global desired_locale
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.MANAGEMENT_SERVER_SERVICES_VALIDATIONS,
        desired_locale)

    # Create object for each checke.r
    srvchk = Ops_Manager_Runtime_Services(id, logger, timeout)
    chkResult = srvchk.check()
    resultList.extend(chkResult)
    logger.info("Final Result is:-> {0}".format(
        json.dumps({"ValidationResult": output_dict})))
    print(json.dumps({"ValidationResult": output_dict}))

    exit(rc)

if __name__ == "__main__":
    main()
