#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

import ConfigParser
import os
import json
import glob
from powervc_ttv.resources import KVMResource
from powervc_ttv.utils import utils
from powervc_ttv.utils import ttvCatalog
from powervc_ttv.utils import messageIDCatalog
from powervc_ttv import translate_message
import re
import xml.etree.ElementTree as ET

output_dict = {
    "ValidationID": "KVM_VALIDATOR",
    "ValidationStatus": "Success",
    "Description": "",
    "TargetedResourceType": "kvm",
    "IpAddress": "",
    "refs": [],
    "ValidationDetail": {
        "ChecksList": []
    },
}

logger = None
# Log file name. Could be /var/log/ttv_validation/mem_cpu_val.log
logFile = "kvm_validator.log"
base_nova_conf_file = "/etc/nova/nova.conf"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"
iscsi_service_status = 'iscsi-service-status'
PING_COMMAND = "ping -c 4 -w 10 "
PING6_COMMAND = "ping6 -c 4 -w 10 "
powerkvm_validations_dict = {}
powerkvm_validations_list = []
DEFAULT_SECTION = "DEFAULT"
KVMHostCredentialValid = 'KVMHostCredentialValid'
KVMHostReachable = 'KVMHostReachable'
LPMConfiguration = 'LPMConfiguration'
LPMCertificatesFound = 'LPMCertificatesFound'
LPMNovaConfig = 'LPMNovaConfig'
LPMLibvirtConfig = 'LPMLibvirtConfig'
LPMFirewallConfig = 'LPMFirewallConfig'
openstack_nova_compute_status = 'openstack-nova-compute-status'
lsmod_kvm_status = "lsmod-kvm-status"
HostnameCheck = "HostnameCheck"
libvirtd_status = "libvirtd-status"
RequiredSystemRelease = "RequiredSystemRelease"
RequiredAvailableMemory = "RequiredAvailableMemory"
RequiredCPUs = "RequiredCPUs"
RequiredPythonVersion = "RequiredPythonVersion"
RequiredFirmware = "RequiredFirmware"
RequiredProcTypes = "RequiredProcTypes"
CMD_PROCTYPE = "cat /proc/cpuinfo | grep 'POWER'"
lshw_status = "lshw-status"
openvswitch_status = "openvswitch-status"
iptables_status = "iptables-status"
host_level_vswitch = "host-level-vswitch"
host_level_vswitch_external_port = "host-level-vswitch-external-port"
RequiredDiskSize = "RequiredDiskSize"
invalid_vswitch_names = "invalid-vswitch-names"
ephemeral_path_valid = "host-ephemeral-path-valid"
host_prop_eph_path = "kvm_host_ephemeral_path"
ephemeral_path_accessible = "host-ephemeral-path-accessible"
openstack_neutron_agent_status = "openstack-neutron-agent-status"
StorageConnectivity = "StorageConnectivity"
chkResultList = output_dict["ValidationDetail"]["ChecksList"]
FILE_SEP_CHAR = '/'
EPH_PATH_TEST_FILE = "/ephDiscVal.txt"
NETWORK_QUALITY_CHK = "NETWORK_QUALITY_CHK"
iscsi_success = ttvCatalog. HOST_TO_ALL_STORAGENODE_ISCSI_CONNECTIVITY_SUCCESS
iscsi_success_msgid = messageIDCatalog.\
    HOST_TO_ALL_STORAGENODE_ISCSI_CONNECTIVITY_SUCCESS
fail = correctiveAction =\
    ttvCatalog.CA_HOST_TO_ALL_STORAGENODE_ISCSI_CONNECTIVITY_FAIL,

neutron_success =\
    ttvCatalog.POWERKVM_OPENSTACK_NUETRON_AGENT_SERVICE_STATUS_SUCCUESS
neutron_success_msgid = messageIDCatalog.\
    POWERKVM_OPENSTACK_NUETRON_AGENT_SERVICE_STATUS_SUCCUESS
POWERKVM_CRED_CHK = "POWERKVM_CRED_CHK"
POWERKVM_REACHABILITY_CHK = "POWERKVM_REACHABILITY_CHK"
POWERKVM_STORAGE_CONNECTIVITY = "POWERKVM_STORAGE_CONNECTIVITY"
POWERKVM_HOSTNAME_CHK = "POWERKVM_HOSTNAME_CHK"
POWERKVM_LPM_CERTIFICATES = "POWERKVM_LPM_CERTIFICATES"
POWERKVM_LPM_NOVA_CONFIG = "POWERKVM_LPM_NOVA_CONFIG"
POWERKVM_LPM_LIBVIRT_CONFIG = "POWERKVM_LPM_LIBVIRT_CONFIG"
POWERKVM_REQUIRED_LPM_CERTIFICATES = "POWERKVM_REQUIRED_LPM_CERTIFICATES"
POWERKVM_LPM_FIREWALL_CONFIG = "POWERKVM_LPM_FIREWALL_CONFIG"
POWERKVM_HOST_KVM_DRIVER = "POWERKVM_HOST_KVM_DRIVER"
POWERKVM_OPENSTACK_NOVA_COMPUTE_STATUS = \
    "POWERKVM_OPENSTACK_NOVA_COMPUTE_STATUS"
POWERKVM_HOST_LIBVIRT = "POWERKVM_HOST_LIBVIRT"
POWERKVM_HOST_RELEASE_VERSION = "POWERKVM_HOST_RELEASE_VERSION"
POWERKVM_HOST_MIN_MEMORY_CHECK = "POWERKVM_HOST_MIN_MEMORY_CHECK"
POWERKVM_HOST_MINIMUM_CPU_CHECK = "POWERKVM_HOST_MINIMUM_CPU_CHECK"
POWERKVM_PYTHON_VERSION = "POWERKVM_PYTHON_VERSION"
POWERKVM_REQUIRED_FIRMWARE = "POWERKVM_REQUIRED_FIRMWARE"
POWERKVM_REQUIRED_PROC_TYPE = "POWERKVM_REQUIRED_PROC_TYPE"
POWERKVM_REQUIRED_TOTAL_DISK_SIZE = "POWERKVM_REQUIRED_TOTAL_DISK_SIZE"
POWERKVM_HOST_EPH_PATH = "POWERKVM_HOST_EPH_PATH"
POWERKVM_LSHW_COMMAND = "POWERKVM_LSHW_COMMAND"
POWERKVM_MIN_VSWITCH_CONFIGURED = "POWERKVM_MIN_VSWITCH_CONFIGURED"
POWERKVM_IPTABLE_SERVICE_STATUS = "POWERKVM_IPTABLE_SERVICE_STATUS"
POWERKVM_OPENSTACK_NEUTRON_AGENT_STATUS = \
    "POWERKVM_OPENSTACK_NEUTRON_AGENT_STATUS"
POWERKVM_OPENVSWITCH_SERVICE_STATUS = "POWERKVM_OPENVSWITCH_SERVICE_STATUS"
POWERKVM_ISCSI_SERVICE_STATUS = "POWERKVM_ISCSI_SERVICE_STATUS"
HSC = ttvCatalog.HOST_TO_ALL_STORAGENODE_ISCSI_CONNECTIVITY_FAIL


class CredentialChecker():
    # checks for the credentials of the resources

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        credential = self.resource.checkCredential()
        if credential == 0:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_VALID,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_CRED_CHK))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_CRED_CHK))
        return (status, self.result)


class NetworkQualityChecker():
    # checks for the network quality between the  management server and
    # resource

    def __init__(self, ip):
        self.ip = ip
        self.result = []

    def check(self):
        status = ""
        host = self.ip
        quality = utils.checkNetworkQuality(self.ip)
        quality = int(quality)
        # if all the packets are dropped
        if quality == 100:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_BAD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        # if packets drop count is more than 5%
        elif quality >= 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.NETWORK_QUALITY_DEGRADED,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Warning"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))
        # if packets drop count is less than 5%
        elif quality < 5:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [utils.get_host_name(), host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog. NETWORK_QUALITY_GOOD,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(self.ip,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 NETWORK_QUALITY_CHK))

        return (status, self.result)


class ReachabilityChecker():
    # checks for the reachability of the resource from management server

    def __init__(self, resource):
        self.resource = resource
        self.result = []

    def check(self):
        status = ""
        host = self.resource.ip
        reachability = utils.checkReachability(host)
        if reachability == 0:
            camsgplaceholders = [host]
            correctiveAction = ""
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog. RESOURCE_REACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = ""
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_REACHABILITY_CHK))
        else:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog. RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog. RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_REACHABILITY_CHK))
        return (status, self.result)


def readConfCinder(logger):
    # reads all the cinder files
    env = {}
    env["svc-storage"] = []
    for conf in glob.glob("/etc/cinder/cinder-*.conf"):
        if conf == "/etc/cinder/cinder-health.conf":
            continue
        try:
            logger.debug(env)
            cinder_conf = ConfigParser.SafeConfigParser()
            cinder_conf.read(conf)
            svc_storage = {}
            host_type = cinder_conf.get('DEFAULT', 'host_type')
            if host_type == 'svc' or host_type == 'SVC':
                svc_storage["hostname"] = cinder_conf.get("DEFAULT", "san_ip")
                svc_storage["userid"] = cinder_conf.get("DEFAULT", "san_login")
                svc_storage["key-file"] = None
                if cinder_conf.has_option("DEFAULT", "san_private_key"):
                    key_file = cinder_conf.get("DEFAULT", "san_private_key")
                    if os.path.exists(key_file):
                        svc_storage["key-file"] = key_file
                if cinder_conf.has_option("DEFAULT", "san_password"):
                    svc_storage["passwd"] = cinder_conf.get("DEFAULT",
                                                            "san_password")

                for prop in ('storwize_svc_volpool_name', 'san_ssh_port'):
                    if cinder_conf.has_option('DEFAULT', prop):
                        svc_storage[prop] = cinder_conf.get('DEFAULT', prop)
                env["svc-storage"].append(svc_storage)
        except Exception as e:
            logger.exception(e)

    return env["svc-storage"]


'''
Parse the service status from the corresponding line
'''


def parseServiceStatus(line):
    return line[line.find(":") + 1:line.find(")") + 1].strip()
'''
Compares versions
'''


def compare(a, b):
    try:
        actual = a.split(".")
        expected = b.split(".")
        actual = [int(i) for i in actual]
        expected = [int(i) for i in expected]
        if len(actual) > len(expected):
            diff = len(actual) - len(expected)
            for i in range(0, diff):
                expected.append(0)
        elif len(expected) > len(actual):
            diff = len(expected) - len(actual)
            for i in range(0, diff):
                actual.append(0)

        return actual >= expected
    except Exception as e:
        logger.exception(e)
        return False


def main_func():
    rc = 0
    home_dir = "/var/log/ttv-validation/validators"
    log_file = "{0}/{1}".format(home_dir.strip(), logFile)
    chkLogger = utils.checkerLogger(log_file)
    global logger
    global desired_locale
    logger = chkLogger.getLogger()
    args = utils.parse_arguments()
    desired_locale = args.locale if args is not None else None
    logger.info(desired_locale)
    output_dict["Description"] = translate_message(
        ttvCatalog.KVM_VALIDATIONS,
        desired_locale)
    conf = glob.glob(base_nova_conf_file)
    base_nova_conf = ConfigParser.ConfigParser()
    base_nova_conf.read(conf)
    powervc_edition = ''
    if base_nova_conf.has_option("DEFAULT", "powervm_mgr_type"):
        powervc_edition = base_nova_conf.get("DEFAULT", "powervm_mgr_type")
    else:
        powervc_edition = base_nova_conf.get("DEFAULT", "host_type")
    if powervc_edition == 'KVM' or powervc_edition == 'kvm':
        resource_dict = readConf(logger)
        kvm_hosts = "kvm_host"
        validationProp = utils.readJSONFile(
            "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json"
        )[kvm_hosts]

        "cinder nodes info"
        resource_dict_cinder = readConfCinder(logger)
        svc = "svc_storage"
        validationProp_Cinder = utils.readJSONFile(
            "/opt/ibm/powervc/ttv-validation/conf/solution_spec.json")[svc]
        for index in range(len(resource_dict)):
            try:
                individual_dict = resource_dict[index]

                validateResource(validationProp, individual_dict,
                                 resource_dict_cinder, validationProp_Cinder,
                                 logger)

            except Exception as e:
                logger.exception(e)

        for index in range(len(resource_dict)):

            individual_dict = resource_dict[index]

            host = individual_dict['ip']
            kvmresourcemgr = KVMResource.KVMResourceManager(individual_dict,
                                                            logger)
            if utils.is_ipv6_install():
                ping_cmd = PING6_COMMAND
            else:
                ping_cmd = PING_COMMAND
            for resource in resource_dict_cinder:
                try:
                    svcresourcemgr = KVMResource.SVCStorageResource(resource,
                                                                    logger)

                    """(status, result) = NetworkQualityChecker(
                        resource["hostname"]).check()
                    chkResultList.extend(result)"""
                    svcresourcemgr_output = svcresourcemgr.getNodeISCSIIPs()

                    connectivity_list = set()
                    svcdetailslist = svcresourcemgr_output.split("----")

                    svcdetailslist = [x for x in svcdetailslist if x != '']
                    svcip = svcdetailslist[0].strip()

                    if len(svcdetailslist) == 2:
                        nodedetailslist = svcdetailslist[
                            1].strip().split("###")
                        nodedetailslist = [
                            x for x in nodedetailslist if x != '']

                        connectedNodeIPDict = {}
                        notConnectedNodeIPDict = {}
                        for nodedata in nodedetailslist:
                            if len(nodedata.split("!!")) == 2:
                                nodeid = nodedata.split("!!")[0]
                                nodeipdata = nodedata.split("!!")[1]
                                nodeiplist = nodeipdata.split(",")
                                nodeiplist = [x for x in nodeiplist if x != '']
                                connected = False
                                if (len(nodeiplist) > 0):
                                    for nodeip in nodeiplist:
                                        # making kvm_ssh object
                                        kvm_ssh_obj = \
                                            kvmresourcemgr._getKVMConnection()

                                        returnCode, stdout, stderr = \
                                            utils.runSSHCommand(kvm_ssh_obj,
                                                                ping_cmd +
                                                                nodeip)

                                        if returnCode == 0:
                                            connected = True
                                            break
                                    if(connected):
                                        connectedNodeIPDict[nodeid] = "true"
                                    else:
                                        notConnectedNodeIPDict[nodeid]\
                                            = "false"
                                else:
                                    connectivity_list.\
                                        add(svcip + "!" + nodeid + "!NoIPs")
                            if(len(nodedetailslist) ==
                               len(connectedNodeIPDict)):
                                connectivity_list.add(svcip + "!Nodes" +
                                                      "!true")
                            elif (len(nodedetailslist) ==
                                  len(notConnectedNodeIPDict)):
                                connectivity_list.add(svcip + "!Nodes" +
                                                      "!false")
                            else:
                                for notConnectedNodeId in \
                                        notConnectedNodeIPDict:
                                    connectivity_list.add(
                                        svcip + "!" +
                                        notConnectedNodeId + "!false")
                                for connectedNodeId in connectedNodeIPDict:
                                    connectivity_list.add(
                                        svcip + "!" + connectedNodeId +
                                        "!true")
                        else:
                            connectivity_list.add(svcip + "!Nodes" + "!NoIPs")
                        if(len(connectivity_list) > 0):
                            powerkvm_validations_dict[
                                StorageConnectivity] = True
                            camsgplaceholders = []
                            correctiveAction = ""
                            msgplaceholders = [svcip]

                            ttvcatalogmsgID = translate_message(
                                iscsi_success,
                                desired_locale)
                            messageID = iscsi_success_msgid
                            msgSubstitues = ""
                            status = "Success"
                            verbose = ""
                            chkResultList.append(
                                utils.checkResult(
                                    host,
                                    correctiveAction,
                                    camsgplaceholders,
                                    ttvcatalogmsgID,
                                    msgplaceholders,
                                    messageID,
                                    msgSubstitues,
                                    status,
                                    verbose,
                                    POWERKVM_STORAGE_CONNECTIVITY))
                            powerkvm_validations_dict[
                                StorageConnectivity + "-value"] = ",".\
                                join(connectivity_list)

                        else:
                            powerkvm_validations_dict[
                                StorageConnectivity] = False
                            camsgplaceholders = [svcip]

                            correctiveAction = fail
                            msgplaceholders = [svcip]
                            ttvcatalogmsgID = \
                                translate_message(HSC, desired_locale)
                            messageID = messageIDCatalog.\
                                HOST_TO_ALL_STORAGENODE_ISCSI_CONNECTIVITY_FAIL
                            msgSubstitues = ""
                            status = "Critical"
                            verbose = ""
                            chkResultList.append(
                                utils.checkResult(
                                    host,
                                    correctiveAction,
                                    camsgplaceholders,
                                    ttvcatalogmsgID,
                                    msgplaceholders,
                                    messageID,
                                    msgSubstitues,
                                    status,
                                    verbose,
                                    POWERKVM_STORAGE_CONNECTIVITY))
                except Exception as e:
                    logger.exception(e)
        print(json.dumps({"ValidationResult": output_dict}))
        return rc
    else:
        print(json.dumps({"ValidationResult": output_dict}))
        return rc


class EphemeralDiskChecker():

    def __init__(self, resource, kvmobj, validationProp,
                 validationProp_Cinder):
        # pass the novaip-*.conf as arg to init
        self.resource = resource
        self.kvmobj = kvmobj
        self.validationProp = validationProp
        self.validationProp_Cinder = validationProp_Cinder
        self.result = []
        global expected_host_prop_eph_path

        expected_host_prop_eph_path = resource[host_prop_eph_path]

    def check(self):
        status = ""
        host = self.resource['ip']
        host_storage_type = self.resource['host_storage_type']
        powerkvm_validations_dict['selinux-nfs-flag-set'] = False
        powerkvm_validations_dict[ephemeral_path_accessible] = False
        powerkvm_validations_dict[ephemeral_path_valid] = False
        isNFS_EPH_path_found = False
        logger.debug("Host Storage Type -> {0}".format(
            host_storage_type))
        if (host_storage_type.lower().strip() == "nfs"):
            # validation for NFS mount reachability
            # and other file operations
            returnCode, stdout, stderr = self.kvmobj.getSelinuxFlag()
            if returnCode == 0:
                for line in stdout:
                    if line.startswith("virt_use_nfs ") and \
                       line.find("on") != -1:
                        powerkvm_validations_dict[
                            'selinux-nfs-flag-set'] = True
                        break
            if powerkvm_validations_dict['selinux-nfs-flag-set']:
                eph_disk_path = expected_host_prop_eph_path
                # iterate over the NFS mounts and check if any matches
                # the eph path
                returnCode, stdout, stderr = self.kvmobj.getCmdNfsCheck()
                if returnCode == 0:
                    for line in stdout:
                        ln_splt = line.split()
                        logger.debug("ln_splt {0}".format(ln_splt))
                        # If the file system type is NFS,get the mount path
                        if ln_splt[2] in ['nfs', 'nfs2', 'nfs3', 'nfs4']:
                            configuredPath = ln_splt[1]
                            logger.debug("configuredPath {0}".format(
                                configuredPath))
                            logger.debug("eph_disk_path {0}".format(
                                eph_disk_path))
                            if eph_disk_path.startswith(configuredPath):
                                isNFS_EPH_path_found = True
                                if len(eph_disk_path) > len(configuredPath):
                                    # check for the next char after
                                    # config path len
                                    if eph_disk_path[len(configuredPath)]\
                                            != FILE_SEP_CHAR:
                                        isNFS_EPH_path_found = False
                                if isNFS_EPH_path_found:
                                    break
        if(host_storage_type.lower().strip() == "local" or
           isNFS_EPH_path_found is True):
            eph_disk_path = expected_host_prop_eph_path
            createFileCommand = 'touch ' +\
                eph_disk_path + EPH_PATH_TEST_FILE
            returnCode, stdout, stderr = self.kvmobj.\
                getCreateFileCommand(createFileCommand)
            if returnCode == 0:
                removeFileCmd = 'rm -f ' + \
                    eph_disk_path + EPH_PATH_TEST_FILE
                returnCode, stdout, stderr = self.kvmobj.getRemoveFileCmd(
                    removeFileCmd)
                if returnCode == 0:
                    powerkvm_validations_dict[
                        ephemeral_path_valid] = True
                    camsgplaceholders = []
                    correctiveAction = ''
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog. POWERKVM_HOST_EPH_PATH_VALID,
                        desired_locale)
                    messageID = "POWERKVM_HOST_EPH_PATH_VALID"
                    msgSubstitues = ""
                    status = "Success"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status, verbose,
                            POWERKVM_HOST_EPH_PATH))

                    powerkvm_validations_dict[
                        ephemeral_path_accessible] = True
                    camsgplaceholders = []
                    correctiveAction = ''
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog. POWERKVM_HOST_EPH_PATH_ACCESSIBLE,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        POWERKVM_HOST_EPH_PATH_ACCESSIBLE
                    msgSubstitues = ""
                    status = "Success"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status, verbose,
                            POWERKVM_HOST_EPH_PATH))
        if powerkvm_validations_dict[ephemeral_path_valid] is False:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERKVM_HOST_EPH_PATH_STATUS_INVALID,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_HOST_EPH_PATH_INVALID,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_HOST_EPH_PATH_STATUS_INVALID
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_HOST_EPH_PATH))
        if powerkvm_validations_dict[ephemeral_path_accessible] is False:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERKVM_HOST_EPH_PATH_NOT_ACCESSIBLE,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_HOST_EPH_PATH_NOT_ACCESSIBLE,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_HOST_EPH_PATH_NOT_ACCESSIBLE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_HOST_EPH_PATH))
        return (status, self.result)


class StorageConnectivityChecker():

    def __init__(self, resource, svcobj, expfirmware, novaipFile):
        self.resource = resource
        self.expfirmware = expfirmware
        self.svcobj = svcobj
        self.result = []
        global expected_host_prop_eph_path
        expected_host_prop_eph_path = resource[host_prop_eph_path]

    def check(self):
        status = ""
        host = self.resource['ip']
        '''validation for iscsi service status'''
        expectedStatus = self.expfirmware["iscsi-service-status"]

        returnCode, stdout, stderr = self.svcobj.get_iscsi_service_status()
        powerkvm_validations_dict[iscsi_service_status] = False
        if returnCode == 3:
            for line in stdout:
                if "Active: " in line:
                    powerkvm_validations_dict[
                        iscsi_service_status + "-value"] = \
                        parseServiceStatus(line)
                    camsgplaceholders = ["host1", "host2"]
                    correctiveAction = translate_message(
                        ttvCatalog.
                        CA_HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_FAIL,
                        desired_locale)
                    msgplaceholders = ["host1", "host2"]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.
                        HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_FAIL,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_FAIL
                    msgSubstitues = ""
                    status = "Critical"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_ISCSI_SERVICE_STATUS))
                    break
        elif returnCode == 0:
            for line in stdout:
                if "Active: " in line:
                    powerkvm_validations_dict[
                        iscsi_service_status + "-value"] = \
                        parseServiceStatus(line)
                    if expectedStatus in line:
                        powerkvm_validations_dict[iscsi_service_status] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = ["host1", "host2"]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.
                            HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_ISCSI_SERVICE_STATUS))
                    break
        if powerkvm_validations_dict[iscsi_service_status] is False:
            camsgplaceholders = ["host1", "host2"]
            correctiveAction = translate_message(
                ttvCatalog. CA_HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_FAIL,
                desired_locale)
            msgplaceholders = ["host1", "host2"]
            ttvcatalogmsgID = translate_message(
                ttvCatalog. HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_FAIL,
                desired_locale)
            messageID = messageIDCatalog.\
                HOST_TO_STORAGENODE_ISCSI_CONNECTIVITY_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_ISCSI_SERVICE_STATUS))

        '''validation for kvm kernel module availability'''
        stdout = self.svcobj.getNodeISCSIIPs()
        return (status, self.result)


class PowerServerNetworkChecker():

    def __init__(self, resource, kvmobj, expfirmware):
        self.resource = resource
        self.expfirmware = expfirmware
        self.kvmobj = kvmobj
        self.result = []
        global expected_iptables_status
        expected_iptables_status = self.expfirmware['iptables-status']

        global expected_openstack_neutron_agent_status
        expected_openstack_neutron_agent_status = self.\
            expfirmware[openstack_neutron_agent_status]

    def check(self):
        status = ""
        host = self.resource['ip']

        bridge_driver_dict = {}
        '''validation for lshw command availability'''
        powerkvm_validations_dict[lshw_status] = False
        returnCode, stdout, stderr = self.kvmobj.getLshw()
        if returnCode == 0:
            powerkvm_validations_dict[lshw_status] = True
            doc = ET.fromstring(("".join(stdout)))
            for node in doc.findall("node"):
                logical_name = node.find('logicalname').text
                for setting in node.find('configuration').findall('setting'):
                    if setting.get('id') == 'driver':
                        bridge_driver_dict[logical_name] = setting.get('value')
            camsgplaceholders = []
            correctiveAction = ''
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_LSHW_COMMAND_SUCCUESS,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_LSHW_COMMAND_AVAILABILITY
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_LSHW_COMMAND))
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERKVM_LSHW_COMMAND_FAIL,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_LSHW_COMMAND_FAIL,
                desired_locale)
            messageID = messageIDCatalog.\
                POWERKVM_LSHW_COMMAND_AVAILABILITY_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_LSHW_COMMAND))

        '''validation for openvswitch availability and
        external port of vswitch'''
        powerkvm_validations_dict[openvswitch_status] = False
        powerkvm_validations_dict[host_level_vswitch] = False
        returnCode, stdout, stderr = self.kvmobj.getopenvswitch()
        if returnCode == 0:
            powerkvm_validations_dict[openvswitch_status] = True
            camsgplaceholders = []
            correctiveAction = ''
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog. POWERKVM_OPENVSWITCH_SERVICE_STATUS_SUCCUESS,
                desired_locale)
            messageID = "POWERKVM_OPENVSWITCH_SERVICE_STATUS_SUCCUESS"
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_OPENVSWITCH_SERVICE_STATUS))
            if powerkvm_validations_dict[openvswitch_status] is False:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog. CA_POWERKVM_OPENVSWITCH_SERVICE_STATUS_FAIL,
                    desired_locale)
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog. POWERKVM_OPENVSWITCH_SERVICE_STATUS_FAIL,
                    desired_locale)
                messageID = "POWERKVM_OPENVSWITCH_SERVICE_STATUS_FAIL"
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.append(
                    utils.checkResult(
                        host,
                        correctiveAction,
                        camsgplaceholders,
                        ttvcatalogmsgID,
                        msgplaceholders,
                        messageID,
                        msgSubstitues,
                        status,
                        verbose,
                        POWERKVM_OPENVSWITCH_SERVICE_STATUS))

            if len(bridge_driver_dict) > 0:
                bridge_dict = {}
                bridge_interfacelist = []

                bridge_name = None
                for line in stdout:
                    if "Bridge" in line and "br-int" in line:
                        bridge_interfacelist = []
                        bridge_name = None

                    elif "Bridge" in line and "br-int" not in line:
                        powerkvm_validations_dict[host_level_vswitch] = True
                        bridge_name = line.split()[1]
                        bridge_interfacelist = []
                        if powerkvm_validations_dict[host_level_vswitch] \
                                is True:
                            camsgplaceholders = []
                            correctiveAction = ''
                            msgplaceholders = []
                            ttvcatalogmsgID = translate_message(
                                ttvCatalog.
                                POWERKVM_MIN_VSWITCH_CONFIGURED_SUCCUESS,
                                desired_locale)
                            messageID = \
                                "POWERKVM_MIN_VSWITCH_CONFIGURED_SUCCUESS"
                            msgSubstitues = ""
                            status = "Success"
                            verbose = ""
                            self.result.append(
                                utils.checkResult(
                                    host,
                                    correctiveAction,
                                    camsgplaceholders,
                                    ttvcatalogmsgID,
                                    msgplaceholders,
                                    messageID,
                                    msgSubstitues,
                                    status,
                                    verbose,
                                    POWERKVM_MIN_VSWITCH_CONFIGURED))
                        else:
                            camsgplaceholders = []
                            correctiveAction = translate_message(
                                ttvCatalog.
                                CA_POWERKVM_MIN_VSWITCH_CONFIGURED_FAIL,
                                desired_locale)
                            msgplaceholders = []
                            ttvcatalogmsgID = translate_message(
                                ttvCatalog.
                                POWERKVM_MIN_VSWITCH_CONFIGURED_FAIL,
                                desired_locale)
                            messageID = "POWERKVM_MIN_VSWITCH_CONFIGURED_FAIL"
                            msgSubstitues = ""
                            status = "Critical"
                            verbose = ""
                            self.result.append(
                                utils.checkResult(
                                    host,
                                    correctiveAction,
                                    camsgplaceholders,
                                    ttvcatalogmsgID,
                                    msgplaceholders,
                                    messageID,
                                    msgSubstitues,
                                    status,
                                    verbose,
                                    POWERKVM_MIN_VSWITCH_CONFIGURED))
                    elif "Interface" in line and bridge_name is not None \
                            and bridge_name != line.split()[1].strip():
                        bridge_interfacelist.append(line.split()[1])
                        bridge_dict[bridge_name] = bridge_interfacelist

                for bridge_name in bridge_dict:
                    external_bridge_found = False
                    if bridge_name != "br-int":
                        for interface_name in bridge_dict.get(bridge_name):
                            interface_name = interface_name.replace('"', '')
                            if interface_name in bridge_driver_dict and\
                                bridge_driver_dict.\
                                    get(interface_name) != 'veth':
                                powerkvm_validations_dict[
                                    host_level_vswitch_external_port] \
                                    = bridge_name + ":" + interface_name
                                external_bridge_found = True
                                break
                    if external_bridge_found:
                        break

        '''validation for iptables service not running'''
        powerkvm_validations_dict[iptables_status] = True
        global CMD_SERVICE_IPTABLES
        expectedStatus = expected_iptables_status
        if utils.is_ipv6_install():
            CMD_SERVICE_IPTABLES = "service ip6tables status"

        returnCode, stdout, stderr = self.kvmobj.getCmdServiceIptables()
        global output_iptables_status
        if stdout is not None:
            for line in stdout:
                if "Active: " in line:
                    powerkvm_validations_dict[
                        iptables_status + "-value"] = parseServiceStatus(line)
                    output_iptables_status = parseServiceStatus(line)
                    if expectedStatus in line:
                        powerkvm_validations_dict[iptables_status] = False
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = [output_iptables_status]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.
                            POWERKVM_IPTABLE_SERVICE_STATUS_SUCCUESS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            POWERKVM_IPTABLE_SERVICE_STATUS_SUCCUESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_IPTABLE_SERVICE_STATUS))
                    break
        if powerkvm_validations_dict[iptables_status] is True:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog. CA_POWERKVM_IPTABLE_SERVICE_STATUS_FAIL,
                desired_locale)
            msgplaceholders = [output_iptables_status]
            ttvcatalogmsgID = translate_message(
                ttvCatalog. POWERKVM_IPTABLE_SERVICE_STATUS_FAIL,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_IPTABLE_SERVICE_STATUS_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_IPTABLE_SERVICE_STATUS))

        '''validation for openstack neutron agent status'''
        expectedStatus = expected_openstack_neutron_agent_status
        returnCode, stdout, stderr = self.kvmobj.\
            getCmdOpenstackNeutronAgentStatus()
        powerkvm_validations_dict[openstack_neutron_agent_status] = False
        if returnCode == 3:
            for line in stdout:
                if "Active: " in line:
                    powerkvm_validations_dict[
                        openstack_neutron_agent_status + "-value"] = \
                        parseServiceStatus(line)
                    break
        elif returnCode == 0:
            for line in stdout:
                if "Active: " in line:
                    powerkvm_validations_dict[
                        openstack_neutron_agent_status + "-value"] = \
                        parseServiceStatus(line)
                    if expectedStatus in line:
                        powerkvm_validations_dict[
                            openstack_neutron_agent_status] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = [parseServiceStatus(line)]

                        ttvcatalogmsgID = translate_message(
                            neutron_success,
                            desired_locale)
                        messageID = neutron_success_msgid
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_OPENSTACK_NEUTRON_AGENT_STATUS))
                    break
        if powerkvm_validations_dict[openstack_neutron_agent_status] is False:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.
                CA_POWERKVM_OPENSTACK_NUETRON_AGENT_SERVICE_STATUS_FAIL,
                desired_locale)
            msgplaceholders = [parseServiceStatus(line)]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.
                POWERKVM_OPENSTACK_NUETRON_AGENT_SERVICE_STATUS_FAIL,
                desired_locale)
            messageID = messageIDCatalog.\
                POWERKVM_OPENSTACK_NUETRON_AGENT_SERVICE_STATUS_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_OPENSTACK_NEUTRON_AGENT_STATUS))
        # If the openvswitch agent is running verify that vswitch names are
        # proper
        vswitch_invalid_names_list = []
        regex = re.compile('^[a-zA-Z0-9_-]*$')
        returnCode, stdout, stderr = self.kvmobj.getCmdvSwitchNames()
        if returnCode == 0:
            for line in stdout:
                if not regex.match(line):
                    vswitch_invalid_names_list.append(line)
            powerkvm_validations_dict[
                invalid_vswitch_names] = vswitch_invalid_names_list
        return (status, self.result)


class KVMHostChecker():

    def __init__(self, resource, kvmobj, expfirmware):
        self.resource = resource
        self.expfirmware = expfirmware
        self.kvmobj = kvmobj
        self.result = []
        global openstack_nova
        openstack_nova = expfirmware['openstack-nova-compute-status']
        global libvirtd_status
        libvirtd_status = expfirmware['libvirtd-status']
        global RequiredSystemRelease
        RequiredSystemRelease = expfirmware['RequiredSystemRelease']
        global expectedMemory
        expectedMemory = expfirmware['RequiredAvailableMemory']
        global expectedRequiredCPUs
        expectedRequiredCPUs = expfirmware['RequiredCPUs']
        global expectedRequiredPythonVersion
        expectedRequiredPythonVersion = expfirmware['RequiredPythonVersion']
        global expectedRequiredFirmware
        expectedRequiredFirmware = expfirmware['RequiredFirmware']
        global expectedRequiredProcTypes
        expectedRequiredProcTypes = expfirmware['RequiredProcTypes']
        global expectedRequiredDiskSize
        expectedRequiredDiskSize = expfirmware['RequiredDiskSize']

    def check(self):

        status = ""
        host = self.resource['ip']
        returnCode, stdout, stderr = self.kvmobj.getHostName()
        if returnCode == 0 and len(stdout) > 0:
            powerkvm_validations_dict[HostnameCheck] = True
            for line in stdout:
                line = line.strip()
                if "localhost" in line.lower():
                    powerkvm_validations_dict[HostnameCheck] = False
                    camsgplaceholders = []
                    correctiveAction = translate_message(
                        ttvCatalog.CA_KVM_HOSTNAME_CHECK_FAIL,
                        desired_locale)
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.KVM_HOSTNAME_CHECK_FAIL,
                        desired_locale)
                    messageID = messageIDCatalog.KVM_HOSTNAME_CHECK_FAIL
                    msgSubstitues = ""
                    status = "Critical"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_HOSTNAME_CHK))
                    break

        '''validation for LPM configuration'''
        powerkvm_validations_dict[LPMCertificatesFound] = False
        returnCode, stdout, stderr = self.kvmobj.getLPMClientCertFile()

        if returnCode == 0:
            returnCode, stdout, stderr = self.kvmobj.getLPMServreCertFile()
            if returnCode == 0:
                returnCode, stdout, stderr = self.kvmobj.getLPMClientKeyFile()
            if returnCode == 0:
                returnCode, stdout, stderr = self.kvmobj.getServertKeyFile()

            if returnCode == 0:
                powerkvm_validations_dict[LPMCertificatesFound] = True
                camsgplaceholders = []
                correctiveAction = ''
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog. KVM_REQUIRED_LPM_CERTIFICATES_FOUND,
                    desired_locale)
                messageID = messageIDCatalog.\
                    KVM_REQUIRED_LPM_CERTIFICATES_FOUND
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.append(
                    utils.checkResult(
                        host,
                        correctiveAction,
                        camsgplaceholders,
                        ttvcatalogmsgID,
                        msgplaceholders,
                        messageID,
                        msgSubstitues,
                        status,
                        verbose,
                        POWERKVM_LPM_CERTIFICATES))

            powerkvm_validations_dict[LPMNovaConfig] = False
            count = 0
            returnCode, stdout, stderr = self.kvmobj.getLPMNovaFile()
            if returnCode == 0:
                for line in stdout:
                    line = line.strip()
                    if not line.startswith("#") and not line.\
                            startswith("[") and "=" in line:
                        linesplit = line.split("=")
                        key = linesplit[0].strip()
                        value = linesplit[1].strip()
                        if "live_migration_flag" in key and\
                                "VIR_MIGRATE_UNDEFINE_SOURCE" in value and\
                                "VIR_MIGRATE_PEER2PEER" in value and\
                                "VIR_MIGRATE_TUNNELLED" in value:
                            count = count + 1
                        if "live_migration_uri" in key and\
                                "qemu+tls://%s/system" in value:
                            count = count + 1
            if count == 2:
                powerkvm_validations_dict[LPMNovaConfig] = True
                camsgplaceholders = []
                correctiveAction = ''
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog.KVM_REQUIRED_LPM_NOVA_CONFIG_FOUND,
                    desired_locale)
                messageID = messageIDCatalog.KVM_REQUIRED_LPM_NOVA_CONFIG_FOUND
                msgSubstitues = ""
                status = "Success"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     POWERKVM_LPM_NOVA_CONFIG))
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog. CA_KVM_REQUIRED_LPM_NOVA_CONFIG_NOT_FOUND,
                    desired_locale)
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog. KVM_REQUIRED_LPM_NOVA_CONFIG_NOT_FOUND,
                    desired_locale)
                messageID = messageIDCatalog.\
                    KVM_REQUIRED_LPM_NOVA_CONFIG_NOT_FOUND
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.append(utils.checkResult(host,
                                                     correctiveAction,
                                                     camsgplaceholders,
                                                     ttvcatalogmsgID,
                                                     msgplaceholders,
                                                     messageID,
                                                     msgSubstitues,
                                                     status, verbose,
                                                     POWERKVM_LPM_NOVA_CONFIG))
            powerkvm_validations_dict[LPMLibvirtConfig] = False
            returnCode, stdout, stderr = self.kvmobj.getLPMLibvitrdFile()
            if returnCode == 0:
                for line in stdout:
                    line = line.strip()
                    if not line.startswith("#") and not line.\
                            startswith("[") and "=" in line:
                        linesplit = line.split("=")
                        key = linesplit[0].strip()
                        value = linesplit[1].strip()
                        if "LIBVIRTD_ARGS" in key and "--listen" in value:
                            powerkvm_validations_dict[LPMLibvirtConfig] = True
                            camsgplaceholders = []
                            correctiveAction = ''
                            msgplaceholders = []
                            ttvcatalogmsgID = translate_message(
                                ttvCatalog.
                                KVM_REQUIRED_LPM_LIBVIRT_CONFIG_FOUND,
                                desired_locale)
                            messageID = messageIDCatalog.\
                                KVM_REQUIRED_LPM_LIBVIRT_CONFIG_FOUND
                            msgSubstitues = ""
                            status = "Success"
                            verbose = ""
                            self.result.append(
                                utils.checkResult(
                                    host,
                                    correctiveAction,
                                    camsgplaceholders,
                                    ttvcatalogmsgID,
                                    msgplaceholders,
                                    messageID,
                                    msgSubstitues,
                                    status,
                                    verbose,
                                    POWERKVM_LPM_LIBVIRT_CONFIG))
                            break
                        else:
                            camsgplaceholders = []
                            correctiveAction = translate_message(
                                ttvCatalog.
                                CA_KVM_REQUIRED_LPM_LIBVIRT_CONFIG_NOT_FOUND,
                                desired_locale)
                            msgplaceholders = []
                            ttvcatalogmsgID = translate_message(
                                ttvCatalog.
                                KVM_REQUIRED_LPM_LIBVIRT_CONFIG_NOT_FOUND,
                                desired_locale)
                            messageID = messageIDCatalog.\
                                KVM_REQUIRED_LPM_LIBVIRT_CONFIG_NOT_FOUND
                            msgSubstitues = ""
                            status = "Critical"
                            verbose = ""
                            self.result.append(
                                utils.checkResult(
                                    host,
                                    correctiveAction,
                                    camsgplaceholders,
                                    ttvcatalogmsgID,
                                    msgplaceholders,
                                    messageID,
                                    msgSubstitues,
                                    status,
                                    verbose,
                                    POWERKVM_LPM_LIBVIRT_CONFIG))
            returnCode, stdout, stderr = self.kvmobj.getLPMFirewall()

            if returnCode == 0 and len(stdout) > 0:
                powerkvm_validations_dict[LPMFirewallConfig] = False
                for line in stdout:
                    if 'libvirt-tls' in line:
                        powerkvm_validations_dict[LPMFirewallConfig] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = []
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog. KVM_REQUIRED_LPM_FIREWALL_CONFIG_FOUND,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            KVM_REQUIRED_LPM_FIREWALL_CONFIG_FOUND
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_LPM_FIREWALL_CONFIG))
                        break
            else:
                camsgplaceholders = []
                correctiveAction = translate_message(
                    ttvCatalog. CA_KVM_REQUIRED_LPM_FIREWALL_CONFIG_NOT_FOUND,
                    desired_locale)
                msgplaceholders = []
                ttvcatalogmsgID = translate_message(
                    ttvCatalog. KVM_REQUIRED_LPM_FIREWALL_CONFIG_NOT_FOUND,
                    desired_locale)
                messageID = messageIDCatalog.\
                    KVM_REQUIRED_LPM_FIREWALL_CONFIG_NOT_FOUND
                msgSubstitues = ""
                status = "Critical"
                verbose = ""
                self.result.append(
                    utils.checkResult(
                        host,
                        correctiveAction,
                        camsgplaceholders,
                        ttvcatalogmsgID,
                        msgplaceholders,
                        messageID,
                        msgSubstitues,
                        status,
                        verbose,
                        POWERKVM_LPM_FIREWALL_CONFIG))

        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog. CA_KVM_REQUIRED_LPM_CERTIFICATES_NOT_FOUND,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog. KVM_REQUIRED_LPM_CERTIFICATES_NOT_FOUND,
                desired_locale)
            messageID = messageIDCatalog.\
                KVM_REQUIRED_LPM_CERTIFICATES_NOT_FOUND
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_REQUIRED_LPM_CERTIFICATES))

        '''validation for kvm kernel module availability'''
        returnCode, stdout, stderr = self.kvmobj.getLsmodKVMStatus()
        if returnCode == 0 and len(stdout) > 0:
            powerkvm_validations_dict[lsmod_kvm_status] = True
            camsgplaceholders = []
            correctiveAction = ''
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_HOST_KVM_DRIVER_SUCCESS,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_HOST_KVM_DRIVER_SUCCESS
            msgSubstitues = ""
            status = "Success"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_HOST_KVM_DRIVER))
        else:
            powerkvm_validations_dict[lsmod_kvm_status] = False
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERKVM_HOST_KVM_DRIVER_FAIL,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_HOST_KVM_DRIVER_FAIL,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_HOST_KVM_DRIVER_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_HOST_KVM_DRIVER))

        '''validation for openstack-nova-compute service status'''

        returnCode, stdout, stderr = self.kvmobj.\
            getopenstackNovaComputeStatus()

        powerkvm_validations_dict[openstack_nova_compute_status] = False
        if returnCode == 3:
            for line in stdout:
                if "Active: " in line:
                    powerkvm_validations_dict[
                        openstack_nova_compute_status + "-value"] = \
                        parseServiceStatus(line)
                    camsgplaceholders = []
                    correctiveAction = translate_message(
                        ttvCatalog. CA_KVM_OPENSTACK_NOVA_COMPUTE_STATUS_ERROR,
                        desired_locale)
                    msgplaceholders = [parseServiceStatus(line)]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog. KVM_OPENSTACK_NOVA_COMPUTE_STATUS_ERROR,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        KVM_OPENSTACK_NOVA_COMPUTE_STATUS_FAIL
                    msgSubstitues = ""
                    status = "Critical"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_OPENSTACK_NOVA_COMPUTE_STATUS))
                    break
        elif returnCode == 0:
            for line in stdout:

                if "Active: " in line:

                    powerkvm_validations_dict[
                        openstack_nova_compute_status + "-value"] = \
                        parseServiceStatus(line)
                    if openstack_nova in line:
                        powerkvm_validations_dict[
                            openstack_nova_compute_status] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = [openstack_nova]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog. KVM_OPENSTACK_NOVA_COMPUTE_STATUS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            KVM_OPENSTACK_NOVA_COMPUTE_STATUS_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_OPENSTACK_NOVA_COMPUTE_STATUS))
                    break
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog. CA_KVM_OPENSTACK_NOVA_COMPUTE_STATUS_ERROR,
                desired_locale)
            msgplaceholders = [parseServiceStatus(line)]
            ttvcatalogmsgID = translate_message(
                ttvCatalog. KVM_OPENSTACK_NOVA_COMPUTE_STATUS_ERROR,
                desired_locale)
            messageID = messageIDCatalog.\
                KVM_OPENSTACK_NOVA_COMPUTE_STATUS_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_OPENSTACK_NOVA_COMPUTE_STATUS))
        '''validation for libvirt service status'''

        returnCode, stdout, stderr = self.kvmobj.getlibvirtd_status()
        # returnCode == 3, service is inactive(dead)
        # returnCode == 0, service is running
        powerkvm_validations_dict[libvirtd_status] = False
        if returnCode == 3:
            for line in stdout:
                if "Active: " in line:
                    powerkvm_validations_dict[
                        libvirtd_status + "-value"] = parseServiceStatus(line)
                    camsgplaceholders = []
                    correctiveAction = translate_message(
                        ttvCatalog.CA_POWERKVM_HOST_LIBVIRT_FAIL,
                        desired_locale)
                    msgplaceholders = [parseServiceStatus(line),
                                       libvirtd_status]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.POWERKVM_HOST_LIBVIRT_FAIL,
                        desired_locale)
                    messageID = messageIDCatalog.POWERKVM_HOST_LIBVIRT_FAIL
                    msgSubstitues = ""
                    status = "Critical"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_HOST_LIBVIRT))
                    break
        elif returnCode == 0:
            for line in stdout:

                if "Active: " in line:

                    powerkvm_validations_dict[
                        libvirtd_status + "-value"] = parseServiceStatus(line)
                    if openstack_nova in line:
                        powerkvm_validations_dict[libvirtd_status] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = []
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog. POWERKVM_HOST_LIBVIRT_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            POWERKVM_HOST_LIBVIRT_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_HOST_LIBVIRT))
                    break
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERKVM_HOST_LIBVIRT_FAIL,
                desired_locale)
            msgplaceholders = [parseServiceStatus(line), libvirtd_status]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_HOST_LIBVIRT_FAIL,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_HOST_LIBVIRT_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_HOST_LIBVIRT))

        '''validation for required system release'''

        powerkvm_validations_dict[RequiredSystemRelease] = False
        returnCode, stdout, stderr = self.kvmobj.get_required_system_release()
        if returnCode == 0:
            for line in stdout:
                if "PowerKVM" in line:
                    powerkvm_validations_dict[
                        RequiredSystemRelease + "-value"] = line.strip()
                    if RequiredSystemRelease.lower() in line.lower():
                        powerkvm_validations_dict[RequiredSystemRelease] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = []
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog. POWERKVM_HOST_RELEASE_VERSION_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            POWERKVM_HOST_RELEASE_VERSION_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_HOST_RELEASE_VERSION))
                        break
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERKVM_HOST_RELEASE_VERSION_FAIL,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_HOST_RELEASE_VERSION_FAIL,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_HOST_RELEASE_VERSION_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_HOST_RELEASE_VERSION))

        '''validation for required amount of memory'''

        powerkvm_validations_dict[RequiredAvailableMemory] = False
        returnCode, stdout, stderr = self.kvmobj.get_required_avail_memory()
        if returnCode == 0:
            for line in stdout:
                if "Mem:" in line and len(line.split()) > 1:
                    global actualMemory
                    actualMemory = line.split()[1].strip()
                    powerkvm_validations_dict[
                        RequiredAvailableMemory +
                        "-value"] = actualMemory
                    if int(actualMemory) >= int(expectedMemory):
                        powerkvm_validations_dict[
                            RequiredAvailableMemory] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = []
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog. POWERKVM_HOST_MIN_MEMORY_CHECK_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            POWERKVM_HOST_MIN_MEMORY_CHECK_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_HOST_MIN_MEMORY_CHECK))
                        break
        else:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_POWERKVM_HOST_RELEASE_VERSION_FAIL,
                desired_locale)
            msgplaceholders = [actualMemory, expectedMemory]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.POWERKVM_HOST_MIN_MEMORY_CHECK_FAILURE,
                desired_locale)
            messageID = messageIDCatalog.POWERKVM_HOST_MIN_MEMORY_CHECK_FAILURE
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_HOST_MIN_MEMORY_CHECK))

        '''validation for required no.of cpu cores'''
        expectedCPUs = expectedRequiredCPUs
        powerkvm_validations_dict[RequiredCPUs] = False
        returnCode, stdout, stderr = self.kvmobj.getRequiredCPUs()
        if returnCode == 0:
            for line in stdout:
                powerkvm_validations_dict[
                    RequiredCPUs + "-value"] = line.strip()
                if int(line.strip()) >= int(expectedCPUs):
                    powerkvm_validations_dict[RequiredCPUs] = True
                    camsgplaceholders = []
                    correctiveAction = ''
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog. POWERKVM_HOST_MINIMUM_CPU_CHECK_SUCCESS,
                        desired_locale)
                    messageID = messageIDCatalog.\
                        POWERKVM_HOST_MINIMUM_CPU_CHECK_SUCCESS
                    msgSubstitues = ""
                    status = "Success"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_HOST_MINIMUM_CPU_CHECK))
                else:
                    camsgplaceholders = []
                    correctiveAction = translate_message(
                        ttvCatalog.CA_POWERKVM_MINIMUM_CPU_CHECK,
                        desired_locale)
                    msgplaceholders = [line.strip()]
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.POWERKVM_MIN_CPU_CHECK_FAILURE,
                        desired_locale)
                    messageID = messageIDCatalog.POWERKVM_MIN_CPU_CHECK_FAILURE
                    msgSubstitues = ""
                    status = "Critical"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_HOST_MINIMUM_CPU_CHECK))
                break
        '''validation for required python library and version'''
        expectedVersion = expectedRequiredPythonVersion
        powerkvm_validations_dict[RequiredPythonVersion] = False
        returnCode, stdout, stderr = self.kvmobj.getRequiredPythonVersion()

        global version
        if returnCode == 0:
            for line in stdout:

                if(len(line.strip().split()) > 1):
                    version = line.strip().split()[1]
                    powerkvm_validations_dict[
                        RequiredPythonVersion + "-value"] = version
                    if compare(version, str(expectedVersion)):
                        powerkvm_validations_dict[RequiredPythonVersion] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = []
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.KVM_PYTHON_VERSION_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.KVM_PYTHON_VERSION_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_PYTHON_VERSION))
                    else:
                        camsgplaceholders = [expectedVersion]
                        correctiveAction = translate_message(
                            ttvCatalog. CA_KVM_PYTHON_VERSION_FAIL,
                            desired_locale)
                        msgplaceholders = [expectedVersion, version]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.KVM_PYTHON_VERSION_FAIL,
                            desired_locale)
                        messageID = messageIDCatalog.KVM_PYTHON_VERSION_FAIL
                        msgSubstitues = ""
                        status = "Critical"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_PYTHON_VERSION))
                    break
            for line in stderr:
                if(len(line.strip().split()) > 1):
                    version = line.strip().split()[1]
                    powerkvm_validations_dict[
                        RequiredPythonVersion + "-value"] = version
                    if compare(version, str(expectedVersion)):
                        powerkvm_validations_dict[RequiredPythonVersion] = True

                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = []
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog.KVM_PYTHON_VERSION_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.KVM_PYTHON_VERSION_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_PYTHON_VERSION))
                    else:
                        camsgplaceholders = [expectedVersion]
                        correctiveAction = translate_message(
                            ttvCatalog. CA_KVM_PYTHON_VERSION_FAIL,
                            desired_locale)
                        msgplaceholders = [expectedVersion, version]
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog. KVM_PYTHON_VERSION_FAIL,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            KVM_PYTHON_VERSION_FAIL
                        msgSubstitues = ""
                        status = "Critical"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_PYTHON_VERSION))
                    break

        '''validation for firmware'''
        expectedFirmware = expectedRequiredFirmware
        powerkvm_validations_dict[RequiredFirmware] = False
        returnCode, stdout, stderr = self.kvmobj.getRequiredFirmware()
        if returnCode == 0:
            for line in stdout:
                if("firmware" in line and expectedFirmware in line):
                    powerkvm_validations_dict[
                        RequiredFirmware +
                        "-value"] = line.split(":")[1].strip()
                    powerkvm_validations_dict[RequiredFirmware] = True
                    camsgplaceholders = []
                    correctiveAction = ''
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.KVM_REQUIRED_FIRMWARE_SUCCESS,
                        desired_locale)
                    messageID = messageIDCatalog.KVM_REQUIRED_FIRMWARE_SUCCESS
                    msgSubstitues = ""
                    status = "Success"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_REQUIRED_FIRMWARE))
                    break
        if powerkvm_validations_dict[RequiredFirmware] is False:
            camsgplaceholders = []
            correctiveAction = translate_message(
                ttvCatalog.CA_KVM_REQUIRED_FIRMWARE_FAIL,
                desired_locale)
            msgplaceholders = []
            ttvcatalogmsgID = translate_message(
                ttvCatalog.KVM_REQUIRED_FIRMWARE_FAIL,
                desired_locale)
            messageID = messageIDCatalog.KVM_REQUIRED_FIRMWARE_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(utils.checkResult(host,
                                                 correctiveAction,
                                                 camsgplaceholders,
                                                 ttvcatalogmsgID,
                                                 msgplaceholders,
                                                 messageID,
                                                 msgSubstitues,
                                                 status, verbose,
                                                 POWERKVM_REQUIRED_FIRMWARE))

        '''validation for ProcType'''
        temp = expectedRequiredProcTypes
        expectedProcTypes = temp.replace(",", "\|")
        powerkvm_validations_dict[RequiredProcTypes] = False
        tempCmd = CMD_PROCTYPE.replace("POWER", expectedProcTypes)
        returnCode, stdout, stderr = self.kvmobj.getRequiredProcTypes(tempCmd)
        if returnCode == 0:
            for line in stdout:
                if(len(line.strip().split(":")) > 1):
                    version = line.strip().split(":")[1].split()[0]
                    powerkvm_validations_dict[
                        RequiredProcTypes + "-value"] = version
                    powerkvm_validations_dict[RequiredProcTypes] = True

                    camsgplaceholders = []
                    correctiveAction = ''
                    msgplaceholders = []
                    ttvcatalogmsgID = translate_message(
                        ttvCatalog.KVM_REQUIRED_PROC_TYPE_SUCCESS,
                        desired_locale)
                    messageID = messageIDCatalog.KVM_REQUIRED_PROC_TYPE_SUCCESS
                    msgSubstitues = ""
                    status = "Success"
                    verbose = ""
                    self.result.append(
                        utils.checkResult(
                            host,
                            correctiveAction,
                            camsgplaceholders,
                            ttvcatalogmsgID,
                            msgplaceholders,
                            messageID,
                            msgSubstitues,
                            status,
                            verbose,
                            POWERKVM_REQUIRED_PROC_TYPE))
                    break
        else:
            camsgplaceholders = [expectedRequiredProcTypes]
            correctiveAction = translate_message(
                ttvCatalog.CA_KVM_REQUIRED_PROC_TYPE_FAIL,
                desired_locale)
            msgplaceholders = [expectedRequiredProcTypes]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.KVM_REQUIRED_PROC_TYPE_FAIL,
                desired_locale)
            messageID = messageIDCatalog.KVM_REQUIRED_PROC_TYPE_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_REQUIRED_PROC_TYPE))

        '''validation for required disk size'''

        expectedDiskSize = expectedRequiredDiskSize
        powerkvm_validations_dict[RequiredDiskSize] = False

        returnCode, stdout, stderr = self.kvmobj.get_disk_size()

        if returnCode == 0:
            for line in stdout:
                if line.startswith("total ") and len(line.split()) >= 5:
                    actualDiskSize = line.split()[1].strip()
                    length = len(actualDiskSize)
                    actualDiskSizeGB = 0
                    if actualDiskSize.endswith("K"):
                        actualDiskSizeGB = float(
                            actualDiskSize[0:length - 1]) / (1048576)
                    elif actualDiskSize.endswith("T"):
                        actualDiskSizeGB = float(
                            actualDiskSize[0:length - 1]) * 1024
                    elif actualDiskSize.endswith("M"):
                        actualDiskSizeGB = float(
                            actualDiskSize[0:length - 1]) / (1024)
                    elif actualDiskSize.endswith("G"):
                        actualDiskSizeGB = float(
                            actualDiskSize[0:length - 1])
                    powerkvm_validations_dict[
                        RequiredDiskSize +
                        "-value"] = round(actualDiskSizeGB, 2)
                    if float(actualDiskSizeGB) >= float(expectedDiskSize):
                        powerkvm_validations_dict[
                            RequiredDiskSize] = True
                        camsgplaceholders = []
                        correctiveAction = ''
                        msgplaceholders = []
                        ttvcatalogmsgID = translate_message(
                            ttvCatalog. KVM_REQUIRED_TOTAL_DISK_SIZE_SUCCESS,
                            desired_locale)
                        messageID = messageIDCatalog.\
                            KVM_REQUIRED_TOTAL_DISK_SIZE_SUCCESS
                        msgSubstitues = ""
                        status = "Success"
                        verbose = ""
                        self.result.append(
                            utils.checkResult(
                                host,
                                correctiveAction,
                                camsgplaceholders,
                                ttvcatalogmsgID,
                                msgplaceholders,
                                messageID,
                                msgSubstitues,
                                status,
                                verbose,
                                POWERKVM_REQUIRED_TOTAL_DISK_SIZE))

                    break
        if powerkvm_validations_dict[RequiredDiskSize] is False:
            camsgplaceholders = [expectedDiskSize]
            correctiveAction = translate_message(
                ttvCatalog.CA_KVM_REQUIRED_TOTAL_DISK_SIZE_FAIL,
                desired_locale)
            msgplaceholders = [expectedDiskSize, actualDiskSizeGB]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.KVM_REQUIRED_TOTAL_DISK_SIZE_FAIL,
                desired_locale)
            messageID = messageIDCatalog.KVM_REQUIRED_TOTAL_DISK_SIZE_FAIL
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            self.result.append(
                utils.checkResult(
                    host,
                    correctiveAction,
                    camsgplaceholders,
                    ttvcatalogmsgID,
                    msgplaceholders,
                    messageID,
                    msgSubstitues,
                    status,
                    verbose,
                    POWERKVM_REQUIRED_TOTAL_DISK_SIZE))

        return (status, self.result)


def readConf(logger):
    env = {}
    env["kvm-power-hosts"] = []
    for conf in glob.glob("/etc/nova/nova-*.conf"):
        if conf == "/etc/nova/nova-health.conf":
            continue
        try:
            logger.info("reading %s file" % conf)
            nova_conf = ConfigParser.SafeConfigParser()
            nova_conf.read(conf)
            name = os.path.basename(conf)
            name = name.replace("nova-", "")
            name = name.replace(".conf", "")
            host_ip_name = ''
            kvm_hosts = {}

            try:
                kvm_hosts["name"] = name

                if nova_conf.has_option("DEFAULT", "kvm_host_ip"):
                    host_ip_name = nova_conf.get("DEFAULT", "kvm_host_ip")
                    kvm_hosts["hostname"] = host_ip_name
                if nova_conf.has_option("DEFAULT", "host"):
                    host_ip_name = nova_conf.get("DEFAULT", "host")
                    kvm_hosts["hostname"] = host_ip_name

                """kvm_hosts["hostname"]=nova_conf.get("DEFAULT","host")
                ip=resolveHostname(nova_conf.get("DEFAULT","host"))
                if not ip:
                    logger.info("IP not resolved")"""
            except Exception as e:
                logger.exception(e)
                continue
            kvm_hosts["ip"] = host_ip_name
            user = nova_conf.get("DEFAULT", "kvm_host_user")
            kvm_hosts["userid"] = user
            if nova_conf.has_option("DEFAULT", "kvm_host_private_key"):
                key_file = nova_conf.get("DEFAULT", "kvm_host_private_key")
                if os.path.exists(key_file):
                    kvm_hosts["key_file"] = key_file
            if nova_conf.has_option("DEFAULT", "kvm_host_passwd"):
                kvm_hosts["passwd"] = nova_conf.get("DEFAULT",
                                                    "kvm_host_passwd")
            port = '22'
            kvm_hosts["port"] = port

            # storage type as local or NFS
            host_storage_type = ''
            if nova_conf.has_option("DEFAULT", "host_storage_type"):
                host_storage_type = nova_conf.get("DEFAULT",
                                                  "host_storage_type")
            kvm_hosts["host_storage_type"] = host_storage_type

            if nova_conf.has_option("DEFAULT", "kvm_host_ephemeral_path"):
                kvm_host_ephemeral_path = nova_conf.\
                    get("DEFAULT", "kvm_host_ephemeral_path")
                kvm_hosts["kvm_host_ephemeral_path"] = kvm_host_ephemeral_path

            env["kvm-power-hosts"].append(kvm_hosts)
        except Exception as e:
            logger.exception(e)
    return env["kvm-power-hosts"]


def validateResource(validationProp, resource, resource_dict_cinder,
                     validationProp_Cinder,
                     logger=None):

    # chkResultList = output_dict["ValidationDetail"]["ChecksList"]
    exception = False
    readConf(logger)

    host = resource['ip']
    kvmresourcemgr = KVMResource.KVMResourceManager(resource, logger)
    # svcresourcemgr = KVMResource.
    # SVCStorageResource(resource_dict_cinder,logger)
    checkers = []

    # checkers.append(ReachabilityChecker(kvmresourcemgr))
    # checkers.append(CredentialChecker(kvmresourcemgr))
    checkers.append(
        KVMHostChecker(resource, kvmresourcemgr, validationProp))
    checkers.append(
        EphemeralDiskChecker(resource, kvmresourcemgr, validationProp,
                             validationProp_Cinder))
    checkers.append(
        PowerServerNetworkChecker(resource, kvmresourcemgr, validationProp))
    """checkers.append(
    StorageConnectivityChecker(resource_dict_cinder,svcresourcemgr,\
                               validationProp,validationProp_Cinder))"""

    try:
        rc = ReachabilityChecker(kvmresourcemgr)
        (status, result) = rc.check()
        chkResultList.extend(result)
    except Exception as e:
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog. RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog. RESOURCE_UNREACHABLE_FROM_MANAGEMENTSERVER,
            desired_locale)
        messageID = messageIDCatalog.\
            MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
        msgSubstitues = ""
        status = "Critical"
        verbose = ""
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               POWERKVM_REACHABILITY_CHK))
        logger.exception(e)
    if status == "Success" and not exception:
        try:
            status = ""
            result = []

            (status, result) = NetworkQualityChecker(
                resource["hostname"]).check()
            chkResultList.extend(result)

            (status, result) = CredentialChecker(kvmresourcemgr).check()

            chkResultList.extend(result)
        except Exception as e:
            camsgplaceholders = [host]
            correctiveAction = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            msgplaceholders = [host]
            ttvcatalogmsgID = translate_message(
                ttvCatalog.RESOURCE_CRENDENTIAL_INVALID,
                desired_locale)
            messageID = messageIDCatalog.\
                MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
            msgSubstitues = ""
            status = "Critical"
            verbose = ""
            chkResultList.append(utils.checkResult(host,
                                                   correctiveAction,
                                                   camsgplaceholders,
                                                   ttvcatalogmsgID,
                                                   msgplaceholders,
                                                   messageID,
                                                   msgSubstitues,
                                                   status, verbose,
                                                   POWERKVM_CRED_CHK))
            logger.exception(e)

        if status == "Success":
            for checker in checkers:
                status = ""
                result = []
                try:
                    (status, result) = checker.check()
                    chkResultList.extend(result)
                except Exception as e:
                    logger.exception(e)
                    exception = True

    if exception:
        correctiveAction = "an exception occurred while performing some or"\
            " all validations of this resource. check the log for more details"
        camsgplaceholders = [host]
        correctiveAction = translate_message(
            ttvCatalog.CA_FABRIC_FETCH_ERROR,
            desired_locale)
        msgplaceholders = [host]
        ttvcatalogmsgID = translate_message(
            ttvCatalog.FABRIC_FETCH_ERROR,
            desired_locale)
        messageID = messageIDCatalog.\
            MANAGEMENT_SERVER_RESOURCE_INCORRECT_USERID_PASSWORD
        msgSubstitues = ""
        status = "Critical"
        verbose = "verbose"
        chkResultList.append(utils.checkResult(host,
                                               correctiveAction,
                                               camsgplaceholders,
                                               ttvcatalogmsgID,
                                               msgplaceholders,
                                               messageID,
                                               msgSubstitues,
                                               status, verbose,
                                               POWERKVM_CRED_CHK))


def main():
    rc = 0
    # Parse the command line arguments
    rc = main_func()
    exit(rc)


if __name__ == "__main__":
    main()
