#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# ------------------------------------------------------------------

import os
import datetime

NOTPUBLISH = 0
PUBLISH = 1
PUBLISHERR = 2
SENSU = -1


class ttvformatter:

    def __init__(self, logger):
        self.logger = logger
        self.result = {}
        self.logger.debug("powervcFormatter: ttv formatter inside init")

    def addToFinalResult(self, catMap=None, cat_id=None, val_id=None,
                         ip=None, exec_status=None, flag=None, val_rslt=None,
                         stat_dict=None):
        self.logger.debug("Entering _addValResult.")
        self.logger.debug("input parameters: ")
        self.logger.debug("1. cat_id: <{0}>".format(cat_id))
        self.logger.debug("2. val_id: <{0}>".format(val_id))
        self.logger.debug("3. ip: <{0}>".format(ip))
        self.logger.debug("4. exec_status: <{0}>".format(str(exec_status)))
        self.logger.debug("5. flag: <{0}>".format(str(flag)))
        self.logger.debug("6. val_rslt: <{0}>".format(str(val_rslt)))

        try:
            checks = {}
            if val_rslt:
                checks = val_rslt["ValidationResult"]["ValidationDetail"]

            '''catMap = \
                self._readJSONFile(self.cat_val_conf_file)'''

            if 'prior-results' not in self.result.keys():
                self.result['prior-results'] = []
                priorresultsdict = {}
                self.result['prior-results'].append(priorresultsdict)
                priorresultsdict['check-groups'] = {}

            checkgroups = self.result['prior-results'][0]['check-groups']

            self.result['executing-tasks'] = []
            if stat_dict:
                completed = 0
                stat_dict_items = stat_dict['tasks'].items()
                total = len(stat_dict_items)
                for name, (status, flag) in stat_dict_items:
                    if flag == 1:
                        completed = completed + 1

                percent = "percent-complete-estimate"
                if total != completed:
                    executingtasksdict = {}
                    self.result['executing-tasks'].\
                        append(executingtasksdict)
                    executingtasksdict["completed-categories"] = completed
                    executingtasksdict["total-categories"] = total
                    executingtasksdict["task-id"] = "1"
                    executingtasksdict["version"] = "v1"
                    executingtasksdict["status"] = {}
                    executingtasksdict["status"]["current-check"] = val_id
                    executingtasksdict["status"]["current-group"] = cat_id
                    executingtasksdict["status"][percent] = (
                        (completed * 100) / total)

            ''' calculate the last verified time '''
            runs_dir = "/var/log/ttv-validation/runs/validators"
            timestamp_format = "%Y-%m-%dT%H:%M:%SZ"
            utc_date = datetime.datetime.utcfromtimestamp(
                os.path.getmtime(
                    runs_dir))
            self.result['prior-results'][0]["stop-time"] =\
                utc_date.strftime(timestamp_format)
            if checks and len(checks["ChecksList"]) > 0 and\
                    type(checks["ChecksList"]) is list:
                if checkgroups and val_id not in checkgroups.keys():
                    checkgroups[val_id] = {}
                checkgroups[val_id] = {"ExecStatus": exec_status}
                host_dict = checkgroups[val_id]
                host_dict['group-description'] = \
                    val_rslt["ValidationResult"]["Description"]
                host_dict['refs'] = \
                    val_rslt["ValidationResult"]['refs']
                '''if exec_status == 1:
                    host_dict["checks"] = {}
                if flag == PUBLISH:'''
                host_dict["ValidationStatus"] = \
                    val_rslt["ValidationResult"]["ValidationStatus"]
                host_dict["checks"] = checks["ChecksList"]
            elif checkgroups and val_id in checkgroups.keys():
                del checkgroups[val_id]

        except Exception as err:
            msg = "Error: " + str(err)
            self.logger.exception(err)
            self.logger.error(msg)
        self.logger.debug("Exiting _addValResult.")

    def getFinalResult(self):
        self.logger.debug("getFinalResult self.result: {0}".
                          format(self.result))
        return self.result


def main():
    rc = 0
    exit(rc)

if __name__ == "__main__":
    main()
