#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_ttv.utils import utils
# from powervc_keystone.encrypthandler import EncryptHandler

base_nova_conf_file = "/etc/nova/nova.conf"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"

CMD_SWITCH_SHOW = 'switchshow'
CMD_SHOW_VERSION = 'version'

'''
Brocade fabric resource class
'''


class BrocadeFabricResource():

    def __init__(self, resource, logger=None, timeout=60):
        self.ip = resource["hostname"]
        self.port = 22
        self.user = resource["fc_fabric_user"]
        self.pwd = resource["fc_fabric_password"]
        self.keyFile = resource["key-file"]
        self.fabric_name = resource["fabric_name"]
        self.LOG = logger
        self.ssh = None

    def checkCredential(self):
        self.LOG.debug("checking credential for resource: " + self.ip)
        credential = 1
        if self.ssh is None:
            self.ssh = utils.getSSHConnection(self.ip, self.user, self.port,
                                              self.pwd, self.keyFile)
            credential = 0
        return credential

    '''
    Retrieve logged in initiator ports
    '''

    def get_loggedin_host_ports(self):

        if self.ssh is None:
            self.ssh = utils.getSSHConnection(self.ip, self.user, self.port,
                                              self.pwd, self.keyFile)

        fc_host_connectivity_dict = {}
        wwpn_set = set()
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set

        cmd = "nsshow | grep 'N '"
        if self.ssh:
            returnCode, stdout, stderr = utils.runSSHCommand(self.ssh, cmd)
            if returnCode == 0 and stdout:
                for line in stdout:
                    elements = line.split(";")
                    wwpn = elements[2].replace(":", "")
                    wwpn_set.add(wwpn.upper())

        cmd = "nscamshow | grep 'N '"
        returnCode, stdout, stderr = utils.runSSHCommand(self.ssh, cmd)
        if returnCode == 0 and stdout:
            for line in stdout:
                elements = line.split(";")
                wwpn = elements[2].replace(":", "")
                wwpn_set.add(wwpn.upper())

        self.LOG.debug("LoggedInHostPorts for brocadefabric:%s" % self.ip)
        self.LOG.debug(fc_host_connectivity_dict)
        return wwpn_set

    '''
    get the firmware of brocade fabric
    '''

    def getFirmware(self):
        self.LOG.debug("retrieving BROCADE storage firmware")
        if self.ssh is None:
            self.ssh = utils.getSSHConnection(self.ip, self.user, self.port,
                                              self.pwd, self.keyFile)
        firmware = ""
        try:
            returnCode, stdout, stderr = utils.runSSHCommand(self.ssh,
                                                             CMD_SHOW_VERSION)
            if returnCode == 0:
                for line in stdout:
                    line = line.strip()
                    if "Fabric OS:" in line:
                        if len(line.split(":")) >= 2:
                            firmware = line.split(":")[1].strip()
                            if firmware[0].isalpha():
                                firmware = firmware[1:]
                            if firmware[len(firmware) - 1].isalpha():
                                firmware = firmware[0:len(firmware) - 1]
                            break
        except Exception as e:
            self.LOG.exception(
                "Error while retrieving BROCADE firmware" + str(e))
        return firmware

    '''
    get the role and zoning status of brocade fabric
    '''

    def getZoningAndRoleStatus(self):
        self.LOG.debug("retrieving BROCADE storage firmware")
        if self.ssh is None:
            self.ssh = utils.getSSHConnection(self.ip, self.user, self.port,
                                              self.pwd, self.keyFile)
        zoning = ""
        role = ""
        try:
            returnCode, stdout, stderr = utils.runSSHCommand(self.ssh,
                                                             CMD_SWITCH_SHOW)
            if returnCode == 0:
                for line in stdout:
                    line = line.strip()
                    if "zoning" in line and len(line.split(":")) >= 2:
                        zoningstr = line.split(":")[1]
                        if len(zoningstr.split()) >= 2:
                            zoning = zoningstr.split()[0].strip()
                    if "switchRole" in line and len(line.split(":")) >= 2:
                        role = line.split(":")[1].strip()
        except Exception as e:
            self.LOG.exception(
                "Error while retrieving BROCADE firmware" + str(e))
        return (zoning, role)
