#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from cinder import exception
from oslo_utils import excutils
from powervc_cinder.volume.drivers.ds8k.ds8k_product import DS8KRESTConnector
from powervc_ttv import _
from powervc_ttv.utils import utils
from xiv_ds8k_openstack.utils.powervc_ds8k import DS8KPowerVCHelpers


def _get_ds8k_exception_class(self):
    """Temporary Workaround for DS8K bug loading exception class"""
    return exception.CinderException

__import__('powervc_oslo.xiv_xsf')

base_nova_conf_file = "/etc/nova/nova.conf"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"


class DS8KStorageResource():

    def __init__(self, resource, logger, timeout=60):
        self.ip = resource["hostname"]
        self.port = 22
        self.user = resource["userid"]
        self.pwd = resource["passwd"]
        # self.keyFile = resource["key-file"]
        self.LOG = logger
        self.ssh = None
        self.pool = resource['san_clustername']
        self.context = None

    '''
    Get connection
    '''

    def getConnection(self):

        try:
            self.LOG.info("Create resource context object ..")
            passwd = utils.getClearTextString(self.pwd).decode('base64')
            self.context = {'username': self.user, 'password': passwd,
                            'address': self.ip, 'pool': self.pool,
                            'http_connector': DS8KRESTConnector()}
        except Exception as e:
            self.LOG.exception(_("Error while retrieving ds8k details:" +
                                 str(e)))
        return self.context

    '''
    get Model & Firmware of this storage
    '''

    def getModelFirmware(self):
        '''fetch details for machine model and firmware'''
        try:
            if self.context is None:
                self.context = self.getConnection()
        except Exception as e:
            with excutils.save_and_reraise_exception():
                self.LOG.error("An error has occurred "
                               "while establishing connection")
                self.LOG.error(e)

        # Get machine details from helper class
        try:
            connection = None
            helpers = DS8KPowerVCHelpers()
            self.LOG.info("Fetching machine details for storage %s.. "
                          % self.context['address'])
            connection = helpers.get_machine_details(self.context)
            self.LOG.info(connection)
        except Exception as e:
            with excutils.save_and_reraise_exception():
                self.LOG.error("An error has occurred "
                               "fetching Machine details"
                               " for this storage.")
                self.LOG.error(e)
        return connection
