#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_ttv.utils import utils
import logging
from HMCResource import *  # noqa

SUCCESS = 0
WARN = 1
CRITICAL = 2

CodeToStatus_map = {SUCCESS: "Success", WARN: "Warning", CRITICAL: "Critical"}
StatusToCode_map = {"Success": SUCCESS, "Warning": WARN, "Critical": CRITICAL}
DEFAULT_SECTION = "DEFAULT"
KVMHostCredentialValid = 'KVMHostCredentialValid'
KVMHostReachable = 'KVMHostReachable'
LPMConfiguration = 'LPMConfiguration'
LPMCertificatesFound = 'LPMCertificatesFound'
LPMNovaConfig = 'LPMNovaConfig'
LPMLibvirtConfig = 'LPMLibvirtConfig'
LPMFirewallConfig = 'LPMFirewallConfig'
openstack_nova_compute_status = 'openstack-nova-compute-status'
lsmod_kvm_status = "lsmod-kvm-status"
HostnameCheck = "HostnameCheck"
libvirtd_status = "libvirtd_status"
RequiredSystemRelease = "RequiredSystemRelease"
RequiredAvailableMemory = "RequiredAvailableMemory"
RequiredCPUs = "RequiredCPUs"
RequiredPythonVersion = "RequiredPythonVersion"
RequiredFirmware = "RequiredFirmware"
RequiredProcTypes = "RequiredProcTypes"
CHECK_LOG = "Check log for details."
NOT_APPLICABLE = "Not applicable"
LOG_FILENAME = 'kvm.log'
LOG = logging.getLogger("kvm_details")
# from powervc_keystone.encrypthandler import EncryptHandler

base_nova_conf_file = "/etc/nova/nova.conf"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"

LPM_CLIENT_CERT_FILE = '[ -e /etc/pki/libvirt/clientcert.pem ]'
LPM_SERVER_CERT_FILE = '[ -e /etc/pki/libvirt/servercert.pem ]'
LPM_CLIENT_KEY_FILE = '[ -e /etc/pki/libvirt/private/clientkey.pem ]'
LPM_SERVER_KEY_FILE = '[ -e /etc/pki/libvirt/private/serverkey.pem ]'
LPM_LIBVIRTD_FILE = 'cat /etc/sysconfig/libvirtd'
CMD_SELINUX_NFS_FLAG = "getsebool virt_use_nfs"
FILE_SEP_CHAR = '/'

ephemeral_path_valid = "host-ephemeral-path-valid"
host_prop_eph_path = "kvm_host_ephemeral_path"
ephemeral_path_accessible = "host-ephemeral-path-accessible"

CMD_NFS_CHECK = "cat /etc/mtab"
EPH_PATH_TEST_FILE = "/ephDiscVal.txt"
CMD_LSPORTIP = 'svcinfo lsportip -nohdr -filtervalue'\
    ' "link_state=active":"state=configured" -delim !'
CMD_LSFABRIC = 'svcinfo lsfabric -nohdr -delim :'
CMD_SHOW_CLUSTER = 'svcinfo lscluster -nohdr -filtervalue name=* -delim :'
CMD_SHOW_VERSION = 'svcinfo lscluster '

"""
CLI Commands for KVM operations.
"""
PING_COMMAND = "ping -c 4 -w 10 "
PING6_COMMAND = "ping6 -c 4 -w 10 "
CMD_OPENSTACK_NOVA_COMPUTE_STATUS = "service openstack-nova-compute status"
CMD_ISCSI_SERVICE_STATUS = "service iscsid status"
CMD_MGMT_IP = 'cat /etc/nova/nova.conf | grep "^auth_uri ="'
CMD_SYSTEM_RELEASE = "cat /etc/system-release"
CMD_SYSTEM_MEMORY = "free -g"
CMD_SYSTEM_CPU = "grep -c processor /proc/cpuinfo"
CMD_LIBVIRTD_STATUS = "service libvirtd status"
CMD_LSMOD_KVM_STATUS = 'lsmod | grep "^kvm\s"'
CMD_OPENVSWITCH = "ovs-vsctl show"
CMD_LSHW = "lshw -class network -xml"
CMD_SERVICE_IPTABLES = "service iptables status"
CMD_OPENSTACK_NEUTRON_AGENT_STATUS = "service neutron-openvswitch-agent status"
CMD_SYSTEM_DISK = "df -lh --total"
CMD_PYTHON_VERSION = "python -V"
CMD_FIRMWARE = "cat /proc/cpuinfo | grep 'firmware'"
CMD_PROCTYPE = "cat /proc/cpuinfo | grep 'POWER'"
CMD_FIREWALL = "firewall-cmd --list-services"
CMD_HOSTNAME = 'hostname'
CMD_VSWITCH_NAMES = 'ovs-vsctl list-br'


class KVMResourceManager(object):

    def __init__(self, resource, logger=None, timeout=60):
        self.ip = resource["hostname"]
        self.port = 22
        self.user = resource["userid"]
        self.password = resource['passwd']
        self.kvm_host_private_key = None
        if ('key_file' in resource):
            self.kvm_host_private_key = resource['key_file']
        # self.password = EncryptHandler().decode(resource['passwd'])
        self.LOG = logger
        self.ssh = None

    def checkCredential(self):
        self.LOG.debug("checking credential for resource: " + self.ip)
        credential = 1
        if self.ssh is None:
            self.ssh = self._getKVMConnection()
            credential = 0
        return credential

    def _getKVMConnection(self):
        self.ssh = utils.getSSHConnection(self.ip, self.user, self.port,
                                          self.password,
                                          self.kvm_host_private_key)
        return self.ssh

    def execute_command(self, cmd):
        ssh_conn = utils.getSSHConnection(self.ip, self.user, self.port,
                                          self.password,
                                          self.kvm_host_private_key)
        if ssh_conn:
            return utils.runSSHCommand(ssh_conn, cmd)

    def is_reachable(self):
        if utils.is_ipv6_install():
            cmdping = ["ping6", "-c 4 ", "-w 10 ", self.ip]
        else:
            cmdping = ["ping", "-c 4 ", "-w 10 ", self.ip]
        p = subprocess.Popen(cmdping, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        out, err = p.communicate()
        p_status = p.wait()
        if p_status == 0:
            return True
        else:
            return False

    def login(self, user, password=None):
        if password is not None:
            ssh_conn = utils.getSSHConnection(self.ip, self.user,
                                              self.port, self.password,
                                              self.kvm_host_private_key)
        if ssh_conn:
            return ssh_conn

    def getHostName(self):
        rc, stdout, stderr = self.execute_command(CMD_HOSTNAME)
        return rc, stdout, stderr

    def getLPMClientCertFile(self):
        '''validation for LPM configuration'''
        rc, stdout, stderr = self.execute_command(LPM_CLIENT_CERT_FILE)
        return rc, stdout, stderr

    def getLPMServreCertFile(self):
        rc, stdout, stderr = self.execute_command(LPM_SERVER_CERT_FILE)
        return rc, stdout, stderr

    def getLPMClientKeyFile(self):
        rc, stdout, stderr = self.execute_command(LPM_CLIENT_KEY_FILE)
        return rc, stdout, stderr

    def getServertKeyFile(self):
        rc, stdout, stderr = self.execute_command(LPM_SERVER_KEY_FILE)
        return rc, stdout, stderr

    def getLPMNovaFile(self):
        rc, stdout, stderr = self.execute_command("cat /etc/nova/nova.conf")
        return rc, stdout, stderr

    def getLPMLibvitrdFile(self):
        rc, stdout, stderr = self.execute_command(LPM_LIBVIRTD_FILE)
        return rc, stdout, stderr

    def getLPMFirewall(self):
        rc, stdout, stderr = self.execute_command(CMD_FIREWALL)
        return rc, stdout, stderr

    def getLsmodKVMStatus(self):
        rc, stdout, stderr = self.execute_command(CMD_LSMOD_KVM_STATUS)
        return rc, stdout, stderr

    def getopenstackNovaComputeStatus(self):
        rc, stdout, stderr = self.\
            execute_command(CMD_OPENSTACK_NOVA_COMPUTE_STATUS)
        return rc, stdout, stderr

    def getlibvirtd_status(self):
        rc, stdout, stderr = self.execute_command(CMD_LIBVIRTD_STATUS)
        return rc, stdout, stderr

    def get_required_system_release(self):
        rc, stdout, stderr = self.execute_command(CMD_SYSTEM_RELEASE)
        return rc, stdout, stderr

    def get_required_avail_memory(self):
        rc, stdout, stderr = self.execute_command(CMD_SYSTEM_MEMORY)
        return rc, stdout, stderr

    def get_mngt_ip(self):
        rc, stdout, stderr = self.execute_command(CMD_MGMT_IP)
        return rc, stdout, stderr

    def getRequiredCPUs(self):
        rc, stdout, stderr = self.execute_command(CMD_SYSTEM_CPU)
        return rc, stdout, stderr

    def getRequiredPythonVersion(self):
        rc, stdout, stderr = self.execute_command(CMD_PYTHON_VERSION)
        return rc, stdout, stderr

    def getRequiredFirmware(self):
        rc, stdout, stderr = self.execute_command(CMD_FIRMWARE)
        return rc, stdout, stderr

    def getRequiredProcTypes(self, cmd):
        rc, stdout, stderr = self.execute_command(cmd)
        return rc, stdout, stderr

    def getLshw(self):
        rc, stdout, stderr = self.execute_command(CMD_LSHW)
        return rc, stdout, stderr

    def getSelinuxFlag(self):
        rc, stdout, stderr = self.execute_command(CMD_SELINUX_NFS_FLAG)
        return rc, stdout, stderr

    def getopenvswitch(self):
        rc, stdout, stderr = self.execute_command(CMD_OPENVSWITCH)
        return rc, stdout, stderr

    def getCmdServiceIptables(self):
        rc, stdout, stderr = self.execute_command(CMD_SERVICE_IPTABLES)
        return rc, stdout, stderr

    def getCmdOpenstackNeutronAgentStatus(self):
        rc, stdout, stderr = self.\
            execute_command(CMD_OPENSTACK_NEUTRON_AGENT_STATUS)
        return rc, stdout, stderr

    def getCmdvSwitchNames(self):
        rc, stdout, stderr = self.execute_command(CMD_VSWITCH_NAMES)
        return rc, stdout, stderr

    def getCmdNfsCheck(self):
        rc, stdout, stderr = self.execute_command(CMD_NFS_CHECK)
        return rc, stdout, stderr

    def getCreateFileCommand(self, cmd):
        rc, stdout, stderr = self.execute_command(cmd)
        return rc, stdout, stderr

    def getRemoveFileCmd(self, cmd):
        rc, stdout, stderr = self.execute_command(cmd)
        return rc, stdout, stderr

    def get_iscsi_service_status(self):
        rc, stdout, stderr = self.execute_command(CMD_ISCSI_SERVICE_STATUS)
        return rc, stdout, stderr

    def get_disk_size(self):
        rc, stdout, stderr = self.execute_command(CMD_SYSTEM_DISK)
        return rc, stdout, stderr


class SVCStorageResource():

    def __init__(self, resource, logger=None, timeout=60):
        self.svcip = resource["hostname"]
        self.port = 22
        self.svcUser = resource["userid"]
        self.svcPwd = resource["passwd"]
        self.keyFile = resource["key-file"]
        self.LOG = logger
        self.ssh = None

    def _getSVCConnection(self):
        self.ssh = utils.getSSHConnection(self.svcip, self.svcUser, self.port,
                                          self.svcPwd, self.keyFile)
        return self.ssh

    def getNodeISCSIIPs(self):
        '''validation for svc node ISCSI IPs'''
        if self.ssh is None:
            self.ssh = self._getSVCConnection()
        returnCode, stdout, stderr = utils.runSSHCommand(self.ssh,
                                                         CMD_LSPORTIP)
        nodeIPDict = {}
        ipindex = 3
        if utils.is_ipv6_install():
            ipindex = 6
        if returnCode == 0 and len(stdout) > 0:
            for line in stdout:
                nodeID = line.split("!")[1]
                nodeip = line.split("!")[ipindex]
                if(len(nodeip) > 0):
                    if nodeIPDict.get(nodeID) is not None:
                        existingnodeiplist = nodeIPDict.get(nodeID)
                        existingnodeiplist.append(nodeip)
                        nodeIPDict[nodeID] = existingnodeiplist
                    else:
                        nodeiplist = []
                        nodeiplist.append(nodeip)
                        nodeIPDict[nodeID] = nodeiplist
        output = self.svcip + "----"
        for nodeid in nodeIPDict:
            output += "Node" + nodeid + "!!"
            output += ",".join(nodeIPDict[nodeid])
            output += "###"
        return output
