#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from HMCResource import cmdHandler
import ast
from powervc_ttv import _


CLUSTER_ID = "cluster_id"
DESCRIPTION = "description"
MESSAGE = "message"
STATUS = "status"
TITLE = "title"
STORAGE_HOST = "storage_host"
CLUSTER_SSP_ACCESSIBLE = "Cluster and SharedStoragePool accessibility"
SUCCESS = "SUCCESS"
MSG_ID = "MSG080"


class SSPResource():

    def __init__(self, hmcip, hmcUser, hmcPasswd, logger, timeout):
        self.hmcip = hmcip
        self.hmcUser = hmcUser
        self.hmcPasswd = hmcPasswd
        self.logger = logger
        self.cmdHandle = cmdHandler(self.logger, timeout)

    def getSSPDetails(self):
        sspChkList = {}
        # Invokes the script that calls K2 api.
        cmd = 'python /usr/lib/python2.7/site-packages/powervc_ttv/utils/' + \
            'powervc_k2_scripts/ssp_details.py -i \"' + \
            self.hmcip + '\" -u \"' + self.hmcUser + \
            '\" -p \"' + self.hmcPasswd + '\"'
        self.logger.info(_("Going to execute ssp_details.py"))
        # gets back the return code and results in the output stream
        (rc, cmdOutput, cmdErr) = self.cmdHandle.cmd_run(cmd)
        if cmdOutput is not None and len(cmdOutput) > 0:
            cmdOutput = ast.literal_eval(cmdOutput)
        if rc == 0:
            # parse the results string and get the information in a dict
            sspChkList = self.parse_command_output(cmdOutput)
        self.logger.info(_("SSP details for hmc {0} from K2 call is {1}".
                           format(self.hmcip, sspChkList)))
        return (rc, cmdOutput, cmdErr, sspChkList)

    def parse_command_output(self, cmdOutput):
        retList = []
        # Check for multiple entries of cinder-*.conf files
        # Below logic assumes we have only one SSP configured with one HMC.
        # But it will work with multi SSP and multi HMC scenario.
        for entry in cmdOutput.keys():
            innerDict = cmdOutput[entry]
            listOfDicts = innerDict["tests"]
            for innerEntry in listOfDicts:
                if innerEntry[MESSAGE] == MSG_ID and \
                        innerEntry[STATUS] == SUCCESS:
                    retList = self.filterSSP(listOfDicts)
                    break
        return retList

    def filterSSP(self, listOfDicts):
        retList = []
        for innerEntry in listOfDicts:
            sspResource = {}
            if innerEntry[CLUSTER_ID] is not None:
                sspResource[CLUSTER_ID] = innerEntry[CLUSTER_ID]
            if innerEntry[DESCRIPTION] is not None:
                sspResource[DESCRIPTION] = innerEntry[DESCRIPTION]
            if innerEntry[MESSAGE] is not None:
                sspResource[MESSAGE] = innerEntry[MESSAGE]
            if innerEntry[STATUS] is not None:
                sspResource[STATUS] = innerEntry[STATUS]
            if innerEntry[TITLE] is not None:
                sspResource[TITLE] = innerEntry[TITLE]
            if innerEntry[STORAGE_HOST] is not None:
                sspResource[STORAGE_HOST] = innerEntry[STORAGE_HOST]
            retList.append(sspResource)
        return retList
