#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================

from powervc_ttv.utils import utils
from xiv_ds8k_openstack.utils.powervc_helper import XIVPowerVCHelpers
from pyne.dtypes import Bunch
from cinder import exception
from powervc_ttv import _
# from powervc_keystone.encrypthandler import EncryptHandler


def _get_xiv_exception_class(self):
    """Temporary Workaround for XIV bug loading exception class"""
    return exception.CinderException

XIVPowerVCHelpers._get_exception = _get_xiv_exception_class

__import__('powervc_oslo.xiv_xsf')


base_nova_conf_file = "/etc/nova/nova.conf"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"


class XIVStorageResource():

    def __init__(self, resource, logger=None, timeout=60):
        self.ip = resource["hostname"]
        self.port = 22
        self.user = resource["userid"]
        self.pwd = resource["passwd"]
        # self.keyFile = resource["key-file"]
        self.LOG = logger
        self.ssh = None
        self.pool = resource['san_clustername']
        self.context = None

    '''
    Get connection
    '''

    def getConnection(self):

        try:
            passwd = utils.getClearTextString(self.pwd).decode('base64')
            self.context = {'user': self.user, 'password': passwd,
                            'address': self.ip, 'pool': self.pool}
        except Exception as e:
            self.LOG.exception(_("Error while retrieving xiv  details:" +
                                 str(e)))
        return self.context

    '''
    get FC port wwpns of initiators in this storage
    '''

    def getConnectivityList(self):
        '''return FCAndHostConnectivityList'''
        helpers = XIVPowerVCHelpers()
        if self.context is None:
            self.context = self.getConnection()
        # context = Bunch(user=self.user, password=pwd, address=self.ip)
        connection = helpers._get_connection(Bunch(self.context))
        fc_host_connectivity_dict = {}
        wwpn_set = set()
        host_wwpn_dict = {}
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set
        fc_host_connectivity_dict["host_wwpn_dict"] = host_wwpn_dict

        hosts_list = connection.\
            cmd.fc_connectivity_list(role='Initiator').as_list
        for host in hosts_list:
            wwpn_set.add(host.wwpn)
        hosts_list = connection.cmd.host_connectivity_list().as_list
        for host in hosts_list:
            if host.host in host_wwpn_dict:
                host_wwpn_dict.get(host.host).add(host.host_port)
            else:
                wwn_set = set()
                wwn_set.add(host.host_port)
                host_wwpn_dict[host.host] = []
                host_wwpn_dict[host.host] = wwn_set
            host_wwpn_dict[host.host] = host_wwpn_dict.get(host.host)

        self.LOG.debug("fc_host_connectivity_dict for xiv:%s" % self.ip)
        self.LOG.debug(fc_host_connectivity_dict)
        return fc_host_connectivity_dict

    '''
    get FC port wwpns of this storage
    '''

    def getSystemFCPorts(self):
        '''return SystemFCPorts'''
        helpers = XIVPowerVCHelpers()
        if self.context is None:
            self.context = self.getConnection()

        connection = helpers._get_connection(Bunch(self.context))
        fc_host_connectivity_dict = {}

        wwpn_set = set()
        fc_host_connectivity_dict["wwpn_set"] = wwpn_set

        hosts_list = connection.cmd.fc_port_list().as_list
        for host in hosts_list:
            if host.port_state.lower() == 'online' and\
                    host.role.lower() == 'target':
                wwpn_set.add((host.wwpn).upper())

        self.LOG.debug("fc_host_connectivity_dict for xiv:%s" % self.ip)
        self.LOG.debug(fc_host_connectivity_dict)
        return wwpn_set

    '''
    get Model of this storage
    '''

    def getModel(self):
        '''validation for model'''
        if self.context is None:
            self.context = self.getConnection()
        helpers = XIVPowerVCHelpers()
        connection = helpers.get_machine_details(self.context)
        return connection.model.split("-")[0]

    '''
    get firmware version of this storage
    '''

    def getFirmware(self):
        '''validation for model'''
        if self.context is None:
            self.context = self.getConnection()
        helpers = XIVPowerVCHelpers()
        connection = helpers.get_machine_details(self.context)
        '''validation for firmware'''
        micro_version = connection.microcode_version.split(".")
        version = micro_version[0] + "." + micro_version[1]
        return version

    '''
    get max host limit of this storage
    '''

    def getMaxHostLimit(self):
        if self.context is None:
            self.context = self.getConnection()
        helpers = XIVPowerVCHelpers()
        limit = int(helpers.get_max_hosts(self.context))
        currentCount = 0
        try:
            currentCount = int(len(helpers.get_hosts_data(self.context)))
        except AttributeError:
            currentCount = int(len(helpers.get_hosts_list(self.context)))
        return (limit, currentCount)
