#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import traceback
from powervc_oslo.k2.k2operator import K2Error
from powervc_oslo.k2.k2operator import K2Session
from powervc_oslo.k2.k2operator import K2Operator
import sys
import optparse
import time
from powervc_ttv.utils import utils

logFile = "hmc_details.log"
home_dir = "/var/log/ttv-validation/validators"
LOG_FILENAME = "{0}/{1}".format(home_dir.strip(), logFile)
chkLogger = utils.checkerLogger(LOG_FILENAME)
LOG = chkLogger.getLogger()


def get_managed_systems_info(oper, cecidList):
    '''
    Gets the ManagedSystem details
    '''
    managed_systems = []
    i = 1
    for cecid in cecidList:
        cec_k2id = cecid
        managedSystem = oper.read('ManagedSystem', rootId=cec_k2id)
        entry = managedSystem.entry

        if entry is not None:
            sys = {}
            print 'Start querying for ManagedSystem %d : ' % i

            sys['k2id'] = cec_k2id  # entry.properties['id']
            print '  mgmt_id=', sys['k2id']

            mtm = entry.element.find('./MachineTypeModelAndSerialNumber')

            sys['MachineType'] = _find_child_text(mtm, './MachineType')
            print '  mgmt_MachineType=', sys['MachineType']

            sys['Model'] = _find_child_text(mtm, './Model')
            print '  mgmt_Model=', sys['Model']

            sys['SerialNumber'] = _find_child_text(mtm, './SerialNumber')
            print '  mgmt_SerialNumber=', sys['SerialNumber']

            sys['MTMS'] = sys['MachineType'] + "-" + \
                sys['Model'] + "-" + sys['SerialNumber']
            print '  mgmt_MTMS=', sys['MTMS']

            sys['SystemName'] = _find_child_text(entry.element, './SystemName')
            print '  mgmt_SystemName=', sys['SystemName']

            sys['State'] = _find_child_text(entry.element, './State')
            print '  mgmt_State=', sys['State']

            sys['PrimaryIPAddress'] = _find_child_text(entry.element,
                                                       './PrimaryIPAddress')
            print '  mgmt_PrimaryIPAddress=', sys['PrimaryIPAddress']

            memory = entry.element.find(
                './AssociatedSystemMemoryConfiguration')

            sys['ConfigurableSystemMemory'] = _find_child_text(
                memory, './ConfigurableSystemMemory')
            print '  mgmt_ConfigurableSystemMemory=', \
                sys['ConfigurableSystemMemory']

            proc = entry.element.find(
                './AssociatedSystemProcessorConfiguration')

            sys['ConfigurableSystemProcessorUnits'] = _find_child_text(
                proc, './ConfigurableSystemProcessorUnits')
            print '  mgmt_ConfigurableSystemProcessorUnits=', \
                sys['ConfigurableSystemProcessorUnits']

            compmodes = proc.findall(
                'SupportedPartitionProcessorCompatibilityModes')
            modes = ''
            for mode in compmodes:
                modes += (' ' + mode.gettext())

            sys['SupportedPartitionProcessorCompatibilityModes'] = modes
            print '  mgmt_SupportedPartitionProcessorCompatibilityModes=', \
                sys['SupportedPartitionProcessorCompatibilityModes']

            syscap = entry.element.find('./AssociatedSystemCapabilities')

            sys['VirtualIOServerCapable'] = _find_child_text(
                syscap, './VirtualIOServerCapable')
            print '  mgmt_VirtualIOServerCapable=', \
                sys['VirtualIOServerCapable']

            sys['MicroLogicalPartitionCapable'] = _find_child_text(
                syscap, './MicroLogicalPartitionCapable')
            print '  mgmt_MicroLogicalPartitionCapable=', \
                sys['MicroLogicalPartitionCapable']

            sys['ActiveLogicalPartitionMobilityCapable'] = _find_child_text(
                syscap, './ActiveLogicalPartitionMobilityCapable')
            print '  mgmt_ActiveLogicalPartitionMobilityCapable=', \
                sys['ActiveLogicalPartitionMobilityCapable']

            host_srr_capability = _find_child_text(
                syscap,
                './PowerVMLogicalPartitionSimplifiedRemoteRestartCapable')
            sys['PowerVMLogicalPartitionSimplifiedRemoteRestartCapable'] = \
                host_srr_capability
            print '  PowerVMLogicalPartitionSimplifiedRemoteRestartCapable=', \
                sys['PowerVMLogicalPartitionSimplifiedRemoteRestartCapable']

            vioss = list_vios_by_cec(oper, sys['k2id'])

            print '*' * 100

            if vioss.get('AssociatedVirtualIOServers') is not None:
                for vios in vioss.get('AssociatedVirtualIOServers'):
                    # print vios
                    show_vios_by_k2id(oper, vios)
                    print '#' * 100

            managed_systems.append(sys)
            print 'End querying for ManagedSystem %d : ' % i
            i += 1
    return managed_systems


def list_vios_by_cec(oper, cec_k2id):
    '''
    Gets all the VIOS under a ManagedSystem
    '''
    managedSystem = oper.read('ManagedSystem', rootId=cec_k2id)

    vios = {}
    entry = managedSystem.entry

    vios['AssociatedVirtualIOServers'] =\
        _get_vios_k2id_from_sys_element(entry.element)
    return vios


def _find_child_text(elemet, search_str):
    '''
    Gets the given element data
    '''
    child = elemet.find(search_str)
    if child is not None:
        LOG.debug("search_str: %s, value: %s ",
                  search_str, child.gettext())
        return child.gettext()
    else:
        return ' '


def _get_vios_k2id_from_sys_element(sys_element):
    vios = sys_element.find('./AssociatedVirtualIOServers')
    if vios is not None:
        return [_get_uuid_from_href(vio.getattrib()['href'],
                                    'VirtualIOServer')['VirtualIOServer']
                for vio in vios.iter(tag='link')]


def _get_uuid_from_href(href, keyStr):
    """
    look for keyStr in the href and find the UUID right next to it
    https://vmcorea03.austin.ibm.com:12443/rest/api/uom/ManagedSystem/
        967811c5-6286-347b-9555-1ec8dd1f3dd4/VirtualIOServer/
        6B699CBB-18F9-4A4F-844A-D4EA49688D76
    """
    href_list = href.split('/')
    href_list[0:href_list.index(keyStr)] = []
    # create a dict with {'<keyStr>': <keyValue>}
    return dict([(href_list[0], href_list[1])])


def show_vios_by_k2id(oper, vios_k2id):
    '''
    Gets the given VIOS details
    '''
    if not vios_k2id:
        return
    vios = oper.read('VirtualIOServer', rootId=vios_k2id)

    vioserver = {}
    vioselem = vios.entry.element

    vioserver['k2id'] = vios_k2id
    print 'Start querying details of VIOS:'

    print '    vios_id=', vioserver['k2id']

    vioserver['PartitionName'] = _find_child_text(vioselem, "./PartitionName")
    print '    vios_PartitionName=', vioserver['PartitionName']

    vioserver['PartitionID'] = _find_child_text(vioselem, "./PartitionID")
    print '    vios_PartitionID=', vioserver['PartitionID']

    virtualSlotsTag = "./PartitionIOConfiguration/CurrentMaximumVirtualIOSlots"
    vioserver['MaximumVirtualIOSlots'] = _find_child_text(vioselem,
                                                          virtualSlotsTag)
    print '    vios_MaximumVirtualIOSlots=', vioserver['MaximumVirtualIOSlots']

    vioserver['OperatiingSystemVersion'] =\
        _find_child_text(vioselem, "./OperatingSystemVersion")
    print '    vios_OperatiingSystemVersion=', \
        vioserver['OperatiingSystemVersion']

    vioserver['CurrentProfileSync'] = _find_child_text(vioselem,
                                                       "./CurrentProfileSync")
    print '    vios_CurrentProfileSync=', vioserver['CurrentProfileSync']

    vioserver['CurrentProcessorCompatibilityMode'] = \
        _find_child_text(vioselem, "./CurrentProcessorCompatibilityMode")
    # print
    # 'CurrentProcessorCompatibilityMode=',
    # vioserver['CurrentProcessorCompatibilityMode']

    vioserver['PartitionType'] = _find_child_text(vioselem, "./PartitionType")
    # print 'PartitionType=',vioserver['PartitionType']

    vioserver['PartitionState'] = _find_child_text(vioselem,
                                                   "./PartitionState")
    print '    vios_PartitionState=', vioserver['PartitionState']

    vioserver['ResourceMonitoringIPAddress'] =\
        _find_child_text(vioselem, "./ResourceMonitoringIPAddress")
    print '    vios_ResourceMonitoringIPAddress=', \
        vioserver['ResourceMonitoringIPAddress']

    vioserver['ResourceMonitoringControlState'] =\
        _find_child_text(vioselem, "./ResourceMonitoringControlState")
    print '    vios_ResourceMonitoringControlState=', \
        vioserver['ResourceMonitoringControlState']

    vioserver['MoverServicePartition'] =\
        _find_child_text(vioselem, "./MoverServicePartition")
    # print 'MoverServicePartition=',vioserver['MoverServicePartition']

    vioserver['ManagerPassthroughCapable'] =\
        _find_child_text(vioselem, "./ManagerPassthroughCapable")
    # print 'ManagerPassthroughCapable=',vioserver['ManagerPassthroughCapable']

    media_repo_path = str('./MediaRepositories/VirtualMediaRepository/')
    if vioselem.find(media_repo_path):
        vioserver['MediaRepositoryExists'] = True
        print '    vios_MediaRepositoryExists=', \
            vioserver['MediaRepositoryExists']

    repository_size = vioselem.find(media_repo_path + "RepositorySize")
    if repository_size is not None:
        vioserver['RepositorySize'] = float(repository_size.gettext())
        print '    vios_RepositorySize=', vioserver['RepositorySize']

    total_media_size = 0.0000000
    media_sizes = vioselem.findall(media_repo_path + "Size")
    if media_sizes:
        for entry in media_sizes:
            media_size = float(entry.gettext())
            if media_size == 0:
                # setting media size to smallest granularity of K2 API value
                media_size = 0.0000005
            total_media_size = total_media_size + media_size
    vioserver['TotalMediaSize'] = total_media_size
    print '    vios_TotalMediaSize=', vioserver['TotalMediaSize']

    wwpns = ''
    WWPNs_tag = './VirtualFibreChannelMappingCollection' \
        '/VirtualFibreChannelMapping/ClientAdapter/WWPNs'
    elements = vioselem.findall(WWPNs_tag)
    wwpns = ''
    for element in elements:
        wwpns += (' ' + element.gettext())

    vioserver['VirtualFiberChannelClientWWPNs'] = wwpns
    print '    vios_VirtualFiberChannelClientWWPNs=', \
        vioserver['VirtualFiberChannelClientWWPNs']

    wwpns = ''
    if not wwpns:
        pfcwwpnelements = []
        adapterPath = str('./PartitionIOConfiguration/ProfileIOSlots/'
                          'ProfileIOSlot/AssociatedIOSlot/RelatedIOAdapter/'
                          'PhysicalFibreChannelAdapter')
        fcAdapters = vioselem.findall(adapterPath)
        if fcAdapters:
            for fcAdapter in fcAdapters:
                LOG.debug('Found element: %s.' % adapterPath)
                portElements = fcAdapter.findall(
                    './PhysicalFibreChannelPorts/PhysicalFibreChannelPort/')
                if portElements:
                    for portElement in portElements:
                        LOG.debug('Found PhysicalFibreChannelPort element')
                        pfcwwpnelements.append(portElement)
        wwpns = ''
        for element in pfcwwpnelements:
            wwpns += (' ' + (_find_child_text(element, "./WWPN")))

    if not wwpns:
        WWPNs_tag = './VirtualFibreChannelMappings' \
            '/VirtualFibreChannelMapping'
        elements = vioselem.findall(WWPNs_tag)
        pfcwwpnelements = []
        for element in elements:
            if(element.find("./ClientAdapter") is None and
               element.find("./ServerAdapter") is None and
               element.find("./AssociatedLogicalPartition") is None):
                pfcwwpnelements.append(element)

        wwpns = ''
        for element in pfcwwpnelements:
            wwpns += (' ' + (_find_child_text(element, "./Port/WWPN")))

    vioserver['PhysicalFiberChannelWWPNs'] = wwpns
    print '    vios_PhysicalFiberChannelWWPNs=', \
        vioserver['PhysicalFiberChannelWWPNs']

    VEA_tag = './TrunkAdapters/TrunkAdapter'
    vea_elements = vioselem.findall(VEA_tag)

    for element in vea_elements:
        print 'Start querying VirtualEthernetAdapter properties:'
        PortVLANID = _find_child_text(element, './PortVLANID')
        AdditionalVLANIDs = _find_child_text(element, './TaggedVLANIDs')
        LOG.debug('VEA PortVLANIDs %s' % PortVLANID)
        LOG.debug('VEA AdditionalVLANIDs %s' % AdditionalVLANIDs)
        print '      vios_vea_AdditionalVLANIDs=', AdditionalVLANIDs
        print '      vios_vea_PortVLANID=', PortVLANID
        print 'End querying VirtualEthernetAdapter properties'

    SEA_tag = './SharedEthernetAdapters/SharedEthernetAdapter'
    sea_elements = vioselem.findall(SEA_tag)
    # print 'sea_elements',sea_elements
    if(len(sea_elements) > 0):
        sea = []
        PortVLANID = ''
        for element in sea_elements:
            print 'Start querying SharedEthernetAdapter properties:'
            PortVLANID = _find_child_text(element, './PortVLANID')
            print '      vios_sea_PortVLANID=', PortVLANID
            print '      vios_sea_HighAvailabilityMode=', \
                _find_child_text(element, './HighAvailabilityMode')
            print '      vios_sea_InterfaceName=', \
                _find_child_text(element, './DeviceName')
            print '      vios_sea_JumboFramesEnabled=', \
                _find_child_text(element, './JumboFramesEnabled')
            print '      vios_sea_QualityOfServiceMode=', \
                _find_child_text(element, './QualityOfServiceMode')
            print '      vios_sea_ThreadModeEnabled=', \
                _find_child_text(element, './ThreadModeEnabled')

            trunkadapters = element.findall('./TrunkAdapters/TrunkAdapter')
            for trunkadapter in trunkadapters:
                trunkPortVLANID = _find_child_text(
                    trunkadapter, './PortVLANID')
                AdditionalVLANIDs = _find_child_text(
                    trunkadapter, './TaggedVLANIDs')
                print '      trunkPortVLANID=', trunkPortVLANID
                LOG.debug('AdditionalVLANIDs %s' % AdditionalVLANIDs)
                print '      AdditionalVLANIDs=', AdditionalVLANIDs
                LOG.debug('trunkPortVLANID %s' % trunkPortVLANID)
                #if PortVLANID == trunkPortVLANID:
                    # print '      vios_sea_state=', \
                        # _find_child_text(trunkadapter, './VariedOn')
                    #break

            if vioserver['PartitionState'].lower() == 'running' and \
                    vioserver['ResourceMonitoringControlState'].lower() == \
                    'active':
                print '      vios_sea_state= true'
            else:
                print '      vios_sea_state= false'

            BackingDevice_tag = './SharedEthernetAdapters'\
                '/SharedEthernetAdapter/EthernetBackingDevices'
            backing_device_elements = vioselem.findall(BackingDevice_tag)
            if len(backing_device_elements) > 0:
                for backing_device_element in backing_device_elements:
                    print '      vios_sea_BackingDeviceName=', \
                        _find_child_text(
                            backing_device_element,
                            './DeviceName')
                    print '      vios_sea_BackingDevice_PhysicalLocation=',\
                        _find_child_text(
                            backing_device_element,
                            './PhysicalLocation')
            vioserver['SharedEthernetAdapters'] = sea
            print 'End querying SharedEthernetAdapter properties'

    PEA_tag = './SharedEthernetAdapters'\
        '/SharedEthernetAdapter/EthernetBackingDevices'
    pea_elements = vioselem.findall(PEA_tag)
    if len(pea_elements) > 0:
        for element in pea_elements:
            print 'Start querying PhysicalEthernetAdapter properties:'
            print '      vios_pea_AdapterID=', \
                _find_child_text(element, './AdapterID')
            print '      vios_pea_DeviceName=', \
                _find_child_text(element, './DeviceName')
            print '      vios_pea_PhysicalLocation=', \
                _find_child_text(element, './PhysicalLocation')
            print '      vios_pea_InterfaceName=', \
                _find_child_text(element, './InterfaceName')
            print '      vios_pea_State=', _find_child_text(element, './State')
            print 'End querying PhysicalEthernetAdapter properties'
    print 'End querying details of VIOS'
    return vioserver


def get_management_console_info(oper):
    try:
        managementConsole = oper.read('ManagementConsole')
        for entry in managementConsole.feed.entries:
            print 'Start querying for ManagementConsole'
            print '  hmc_id=', entry.properties['id']
            print '  HMC_Name=', \
                _find_child_text(entry.element, './ManagementConsoleName')
            print '  HMC_Version=', \
                _find_child_text(entry.element, './VersionInfo/Version')
            print '  HMC_Release=', \
                _find_child_text(entry.element, './VersionInfo/Release')
            print '  HMC_Maintenance=', \
                _find_child_text(entry.element, './VersionInfo/Maintenance')
            print 'End querying for ManagementConsole '
        return True
    except K2Error:
        print traceback.format_exc()
        sys.exit(1)
        return False


def getAllHMCInfo(host, user, pwd, cecidList):

    try:
        conn = K2Session(host, user, pwd)
        # dest = 'https://%s:%i' % (conn.host, k2o.port)
        oper = K2Operator(conn)
        # small logic and working
        get_management_console_info(oper)

        get_managed_systems_info(oper, cecidList)
        return True
    except K2Error:
        print traceback.format_exc()
        return False


def my_main():
    parser = optparse.OptionParser()
    parser.add_option('-i', '--ip', dest='hmc_ip', help='hmc ip')
    parser.add_option('-u', '--user', dest='hmc_user', help='hmc user name')
    parser.add_option('-p', '--passwd', dest='hmc_passwd',
                      help='hmc users password')
    parser.add_option('-c', '--cecids', dest='cecids',
                      help='cecIDs')

    help_msg = "hmc_details.py -i <hmc_ip/hmc_hostname> -u <hmc_user> "\
        "-p <hmc_password> -c <comma separated CEC ids>"

    LOG.debug('is when this event was logged.')
    LOG.debug("Log file with name " + LOG_FILENAME + " created")

    (options, args) = parser.parse_args()
    if options.hmc_ip is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_user is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_passwd is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.cecids is None:
        print "HELP : " + help_msg
        sys.exit(2)

    LOG.debug("********************************************")
    LOG.debug("*************HMC details****************")
    LOG.debug("********************************************")
    LOG.debug("HMC IP :" + options.hmc_ip)
    LOG.debug("HMC User Name : " + options.hmc_user)

    LOG.debug("********************************************")
    hmcip = options.hmc_ip
    hmcUser = options.hmc_user
    if options.hmc_passwd is not None:
        hmcPasswd = utils.getClearTextString(options.hmc_passwd)
    cecidList = options.cecids.split(",")

    # Processing of command line arguments end
    start_time = time.time()
    if(getAllHMCInfo(hmcip, hmcUser, hmcPasswd, cecidList)):
        LOG.debug("HMC details retrieved successfully")
        stop_time = time.time()
        print 'total time taken for hmc details:', \
            round(stop_time - start_time, 1)
        sys.exit(0)
    else:
        stop_time = time.time()
        print 'total time taken for hmc details:', \
            round(stop_time - start_time, 1)
        sys.exit(1)
    LOG.debug("Done with retrieve")

if __name__ == '__main__':
    my_main()
