#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import traceback
from powervc_oslo.k2.k2operator import K2Error
from powervc_oslo.k2.k2operator import K2Session
from powervc_oslo.k2.k2operator import K2Operator
from powervc_ttv.utils import utils
import sys
import optparse


# Logger Initialization
logFile = "hmc_managed_all_host_details.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)
chkLogger = utils.checkerLogger(log_file)
logger = chkLogger.getLogger()


def getHMCDetails(hmcip, hmcuser, hmcpassword):

    conn = K2Session(hmcip, hmcuser, hmcpassword)
    oper = K2Operator(conn)
    try:
        k2resp = oper.read('ManagedSystem')
        for i in range(len(k2resp.feed.entries)):
            print '\nManagedSystem %d : ' % i
            entry = k2resp.feed.entries[i]
            print 'id=', entry.properties['id']
            elem = entry.element
            name = elem.findtext('SystemName')
            if name:
                print 'name=', name
            state = elem.findtext('State')
            if state:
                print 'state=', state
            primary_ip_addr = elem.findtext('PrimaryIPAddress')
            if primary_ip_addr:
                print 'primaryIPaddress=', primary_ip_addr
            machine_type = elem.findtext('MachineTypeModelAndSerialNumber/'
                                         'MachineType')
            if machine_type:
                print 'MachineType=', machine_type
            model = elem.findtext('MachineTypeModelAndSerialNumber/Model')
            if model:
                print 'Model=', model
            serial_number = elem.findtext('MachineTypeModelAndSerialNumber/'
                                          'SerialNumber')
            if serial_number:
                print 'SerialNum=', serial_number
            print 'MTM=', machine_type + '-' + model + '-' + serial_number
            mem = elem.findtext('AssociatedSystemMemoryConfiguration/'
                                'ConfigurableSystemMemory')
            if mem:
                print 'ConfigurableSystemMemory=', mem
            proc = elem.findtext('AssociatedSystemProcessorConfiguration/'
                                 'ConfigurableSystemProcessorUnits')
            if proc:
                print 'ConfigurableSystemProcUnits=', proc
            vios_cap = elem.findtext('AssociatedSystemCapabilities/'
                                     'VirtualIOServerCapable')
            if vios_cap:
                print 'VirtualIOServerCapable=', vios_cap
            mlp_cap = elem.findtext('AssociatedSystemCapabilities/'
                                    'MicroLogicalPartitionCapable')
            if mlp_cap:
                print 'MicroLogicalPartitionCapable=', mlp_cap
            alpm_cap = elem.findtext('AssociatedSystemCapabilities/'
                                     'ActiveLogicalPartitionMobilityCapable')
            if alpm_cap:
                print 'ActiveLogicalPartitionMobilityCapable=', alpm_cap
        return True
    except K2Error:
        print traceback.format_exc()
        sys.exit(2)
        return False


def main():
    parser = optparse.OptionParser()
    parser.add_option('-i', '--ip', dest='hmc_ip', help='hmc ip')
    parser.add_option('-u', '--user', dest='hmc_user', help='hmc user name')
    parser.add_option('-p', '--passwd', dest='hmc_passwd',
                      help='hmc users password')

    help_msg = "hmc_managed_all_hosts_details.py -i <hmc_ip> -u <hmc_user> "\
        "-p <hmc_password>"

    logger.debug('is when this event was logged.')
    logger.debug("Log file with name " + log_file + " created")

    (options, args) = parser.parse_args()
    if options.hmc_ip is None:
        print "HELP : " + help_msg
        sys.exit(1)
    if options.hmc_user is None:
        print "HELP : " + help_msg
        sys.exit(1)
    if options.hmc_passwd is None:
        print "HELP : " + help_msg
        sys.exit(1)

    logger.debug("********************************************")
    logger.debug("*************HMC details****************")
    logger.debug("********************************************")
    logger.debug("HMC IP :" + options.hmc_ip)
    logger.debug("HMC User Name : " + options.hmc_user)
    logger.debug("********************************************")
    hmcip = options.hmc_ip
    hmcUser = options.hmc_user
    if options.hmc_passwd is not None:
        hmcPasswd = utils.getClearTextString(options.hmc_passwd)

    if getHMCDetails(hmcip, hmcUser, hmcPasswd):
        logger.debug("HMC ManagedSystem details retrieved successfully")
        sys.exit(0)
    logger.debug("Done with retrieve")

if __name__ == "__main__":
    try:
        sys.exit(main())
    except Exception, e:
        print e
