#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import sys
import traceback
from powervc_oslo.k2.k2operator import K2Error
from powervc_oslo.k2.k2operator import K2Session
from powervc_oslo.k2.k2operator import K2Operator
from powervc_ttv.utils import utils
import optparse
import time


# Logger Initialization
logFile = "hmc_managedsystem.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)
chkLogger = utils.checkerLogger(log_file)
logger = chkLogger.getLogger()


def get_managed_systems_info(oper, cecidList):
    '''
    Gets the ManagedSystem details
    '''
    managed_systems = []
    i = 1
    for cecid in cecidList:
        cec_k2id = cecid
        managedSystem = oper.read('ManagedSystem', rootId=cec_k2id)
        entry = managedSystem.entry

        if entry is not None:
            sys = {}
            print 'Start querying for ManagedSystem %d : ' % i

            sys['k2id'] = cec_k2id  # entry.properties['id']
            print '  mgmt_id=', sys['k2id']

            mtm = entry.element.find('./MachineTypeModelAndSerialNumber')

            sys['MachineType'] = _find_child_text(mtm, './MachineType')
            print '  mgmt_MachineType=', sys['MachineType']

            sys['Model'] = _find_child_text(mtm, './Model')
            print '  mgmt_Model=', sys['Model']

            sys['SerialNumber'] = _find_child_text(mtm, './SerialNumber')
            print '  mgmt_SerialNumber=', sys['SerialNumber']

            sys['MTMS'] = sys['MachineType'] + "-" + \
                sys['Model'] + "-" + sys['SerialNumber']
            print '  mgmt_MTMS=', sys['MTMS']

            sys['SystemName'] = _find_child_text(entry.element, './SystemName')
            print '  mgmt_SystemName=', sys['SystemName']

            sys['State'] = _find_child_text(entry.element, './State')
            print '  mgmt_State=', sys['State']

            sys['PrimaryIPAddress'] = _find_child_text(entry.element,
                                                       './PrimaryIPAddress')
            print '  mgmt_PrimaryIPAddress=', sys['PrimaryIPAddress']

            memory = entry.element.find(
                './AssociatedSystemMemoryConfiguration')

            sys['ConfigurableSystemMemory'] = _find_child_text(
                memory, './ConfigurableSystemMemory')
            print '  mgmt_ConfigurableSystemMemory=', \
                sys['ConfigurableSystemMemory']

            proc = entry.element.find(
                './AssociatedSystemProcessorConfiguration')

            sys['ConfigurableSystemProcessorUnits'] = _find_child_text(
                proc, './ConfigurableSystemProcessorUnits')
            print '  mgmt_ConfigurableSystemProcessorUnits=', \
                sys['ConfigurableSystemProcessorUnits']

            compmodes = proc.findall(
                'SupportedPartitionProcessorCompatibilityModes')
            modes = ''
            for mode in compmodes:
                modes += (' ' + mode.gettext())

            sys['SupportedPartitionProcessorCompatibilityModes'] = modes
            print '  mgmt_SupportedPartitionProcessorCompatibilityModes=', \
                sys['SupportedPartitionProcessorCompatibilityModes']

            syscap = entry.element.find('./AssociatedSystemCapabilities')

            sys['VirtualIOServerCapable'] = _find_child_text(
                syscap, './VirtualIOServerCapable')
            print '  mgmt_VirtualIOServerCapable=', \
                sys['VirtualIOServerCapable']

            sys['MicroLogicalPartitionCapable'] = _find_child_text(
                syscap, './MicroLogicalPartitionCapable')
            print '  mgmt_MicroLogicalPartitionCapable=', \
                sys['MicroLogicalPartitionCapable']

            sys['ActiveLogicalPartitionMobilityCapable'] = _find_child_text(
                syscap, './ActiveLogicalPartitionMobilityCapable')
            print '  mgmt_ActiveLogicalPartitionMobilityCapable=', \
                sys['ActiveLogicalPartitionMobilityCapable']

            host_srr_capability = _find_child_text(
                syscap,
                './PowerVMLogicalPartitionSimplifiedRemoteRestartCapable')
            sys['PowerVMLogicalPartitionSimplifiedRemoteRestartCapable'] = \
                host_srr_capability
            print '  PowerVMLogicalPartitionSimplifiedRemoteRestartCapable=', \
                sys['PowerVMLogicalPartitionSimplifiedRemoteRestartCapable']

            managed_systems.append(sys)
            print 'End querying for ManagedSystem %d : ' % i
            i += 1
    return managed_systems


def _find_child_text(elemet, search_str):
    '''
    Gets the given element data
    '''
    child = elemet.find(search_str)
    if child is not None:
        logger.debug("search_str: %s, value: %s ",
                     search_str, child.gettext())
        return child.gettext()
    else:
        return ' '


def getAllHMCInfo(host, user, pwd, cecidList):

    try:
        conn = K2Session(host, user, pwd)
        oper = K2Operator(conn)
        managedsystems = []
        managedsystems = get_managed_systems_info(oper, cecidList)
        print("managed systems details start")
        print(managedsystems)
        print("managed systems details end")
        return True
    except K2Error:
        print traceback.format_exc()
        return False


def main():

    parser = optparse.OptionParser()
    parser.add_option('-i', '--ip', dest='hmc_ip', help='hmc ip')
    parser.add_option('-u', '--user', dest='hmc_user', help='hmc user name')
    parser.add_option('-p', '--passwd', dest='hmc_passwd',
                      help='hmc users password')
    parser.add_option('-c', '--cecids', dest='cecids',
                      help='cecIDs')

    help_msg = "hmc_managedsystem.py -i <hmc_ip/hmc_hostname> -u <hmc_user> "\
        "-p <hmc_password> -c <comma separated CEC ids>"

    logger.debug('is when this event was logged.')
    logger.debug("Log file with name " + log_file + " created")

    (options, args) = parser.parse_args()
    if options.hmc_ip is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_user is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_passwd is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.cecids is None:
        print "HELP : " + help_msg
        sys.exit(2)

    logger.debug("********************************************")
    logger.debug("*************HMC details****************")
    logger.debug("********************************************")
    logger.debug("HMC IP :" + options.hmc_ip)
    logger.debug("HMC User Name : " + options.hmc_user)

    logger.debug("********************************************")
    hmcip = options.hmc_ip
    hmcUser = options.hmc_user
    if options.hmc_passwd is not None:
        hmcPasswd = utils.getClearTextString(options.hmc_passwd)
    cecidList = options.cecids.split(",")

    # Processing of command line arguments end
    start_time = time.time()
    if(getAllHMCInfo(hmcip, hmcUser, hmcPasswd, cecidList)):
        logger.debug("HMC details retrieved successfully")
        stop_time = time.time()
        print 'total time taken for hmc details:', \
            round(stop_time - start_time, 1)
        sys.exit(0)
    else:
        stop_time = time.time()
        print 'total time taken for hmc details:', \
            round(stop_time - start_time, 1)
        sys.exit(1)

if __name__ == "__main__":
    main()
