#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import traceback
from powervc_oslo.k2.k2operator import K2Error
from powervc_oslo.k2.k2operator import K2Session
from powervc_oslo.k2.k2operator import K2Operator
from powervc_ttv.utils import utils
import sys
import optparse
import time


# Logger Initialization
logFile = "hmc_version.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)
chkLogger = utils.checkerLogger(log_file)
logger = chkLogger.getLogger()


def _find_child_text(elemet, search_str):
    '''
    Gets the given element data
    '''
    child = elemet.find(search_str)
    if child is not None:
        return child.gettext()
    else:
        return ' '


def get_management_console_info(oper):
    try:
        managementConsole = oper.read('ManagementConsole')
        for entry in managementConsole.feed.entries:
            print 'Start querying for ManagementConsole'
            print '  hmc_id=', entry.properties['id']
            print '  HMC_Name=', \
                _find_child_text(entry.element, './ManagementConsoleName')
            print '  HMC_Version=', \
                _find_child_text(entry.element, './VersionInfo/Version')
            print '  HMC_Release=', \
                _find_child_text(entry.element, './VersionInfo/Release')
            print '  HMC_Maintenance=', \
                _find_child_text(entry.element, './VersionInfo/Maintenance')
            print 'End querying for ManagementConsole '
        return True
    except K2Error:
        print traceback.format_exc()
        sys.exit(1)
        return False


def getAllHMCInfo(host, user, pwd):

    try:
        conn = K2Session(host, user, pwd)
        oper = K2Operator(conn)
        get_management_console_info(oper)
        return True
    except K2Error:
        print traceback.format_exc()
        return False


def my_main():
    parser = optparse.OptionParser()
    parser.add_option('-i', '--ip', dest='hmc_ip', help='hmc ip')
    parser.add_option('-u', '--user', dest='hmc_user', help='hmc user name')
    parser.add_option('-p', '--passwd', dest='hmc_passwd',
                      help='hmc users password')

    help_msg = "hmc_version.py -i <hmc_ip/hmc_hostname> -u <hmc_user> "\
        "-p <hmc_password>"

    logger.debug('is when this event was logged.')
    logger.debug("Log file with name " + log_file + " created")

    (options, args) = parser.parse_args()
    if options.hmc_ip is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_user is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_passwd is None:
        print "HELP : " + help_msg
        sys.exit(2)

    logger.debug("********************************************")
    logger.debug("*************HMC details****************")
    logger.debug("********************************************")
    logger.debug("HMC IP :" + options.hmc_ip)
    logger.debug("HMC User Name : " + options.hmc_user)

    logger.debug("********************************************")
    hmcip = options.hmc_ip
    hmcUser = options.hmc_user
    if options.hmc_passwd is not None:
        hmcPasswd = utils.getClearTextString(options.hmc_passwd)

    # Processing of command line arguments end
    start_time = time.time()
    if(getAllHMCInfo(hmcip, hmcUser, hmcPasswd)):
        logger.debug("HMC details retrieved successfully")
        stop_time = time.time()
        print 'total time taken for hmc details:', \
            round(stop_time - start_time, 1)
        sys.exit(0)
    else:
        stop_time = time.time()
        print 'total time taken for hmc details:', \
            round(stop_time - start_time, 1)
        sys.exit(1)
    logger.debug("Done with retrieve")

if __name__ == '__main__':
    my_main()
