#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2013, 2014 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
from powervc_cinder.k2aclient import client
import json
import optparse
from powervc_ttv.utils import utils
import sys
import traceback

# Logger Initialization
logFile = "ssp_details.log"
home_dir = "/var/log/ttv-validation/validators"
log_file = "{0}/{1}".format(home_dir.strip(), logFile)
chkLogger = utils.checkerLogger(log_file)
logger = chkLogger.getLogger()


def ssp_health_check(cs):

    check_results = cs.powervc.check_ssp_conf("/etc/cinder")
    logger.debug("check_results are")
    logger.debug(check_results)
    check_results_for_json = cs.powervc.result_output_for_ttv(check_results)
    result = json.dumps(check_results_for_json, sort_keys=True, indent=4)
    logger.debug("result json is")
    logger.debug(result)
    print result


def my_main():
    parser = optparse.OptionParser()
    parser.add_option('-i', '--ip', dest='hmc_ip', help='hmc ip')
    parser.add_option('-u', '--user', dest='hmc_user', help='hmc user name')
    parser.add_option('-p', '--passwd', dest='hmc_passwd',
                      help='hmc users password')

    help_msg = "ssp_details.py -i <hmc_ip/hmc_hostname> -u <hmc_user> "\
        "-p <hmc_password>"

    (options, args) = parser.parse_args()
    if options.hmc_ip is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_user is None:
        print "HELP : " + help_msg
        sys.exit(2)
    if options.hmc_passwd is None:
        print "HELP : " + help_msg
        sys.exit(2)

    hmcip = options.hmc_ip
    hmcUser = options.hmc_user
    if options.hmc_passwd is not None:
        hmcPasswd = utils.getClearTextString(options.hmc_passwd)

    k2_api_version = 1
    k2_url = hmcip
    k2_username = hmcUser
    k2_password = hmcPasswd

    cs = client.Client(k2_api_version,
                       k2_url,
                       k2_username,
                       k2_password,
                       retries=30,
                       k2_certpath='/opt/ibm/powervc/data/certs/')

    try:
        ssp_health_check(cs)

    except Exception as e:
        logger.debug("SSP validations failed")
        logger.debug(str(e))
        print (traceback.format_exc())


if __name__ == '__main__':
    my_main()
