#!/usr/bin/python
# vim: tabstop=4 shiftwidth=4 softtabstop=4
# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2015 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
import ConfigParser
import os
import glob
import paramiko
from powervc_keystone.encrypthandler import EncryptHandler
import subprocess
import time
import logging.handlers
import logging
import json
from sqlalchemy import create_engine
import re
import argparse
from powervc_ttv import _
from distutils.version import LooseVersion as V

LOG = logging.getLogger(__name__)

base_nova_conf_file = "/etc/nova/nova.conf"
base_cinder_conf_file = "/etc/cinder/cinder.conf"
ttv_validation_api_paste_file = "/etc/ttv-validation/api-paste.ini"
powervc_known_host_path_param = "ibmpowervm_known_hosts_path"
powervc_known_host_file = "/opt/ibm/powervc/data/known_hosts"
PING_COMMAND = "ping -c 4 -w 10 "
DEFAULT_SECTION = "DEFAULT"
hmc_query = "select * from ibm_hmcs where deleted = 0"
hmc_clusters_query = "select * from ibm_hmc_clusters where deleted =0"


def is_ipv6_install():
    conf = glob.glob(base_nova_conf_file)
    base_nova_conf = ConfigParser.ConfigParser()
    base_nova_conf.read(conf)
    if base_nova_conf.has_option("DEFAULT", "install_ip_version"):
        install_ip_version = base_nova_conf.get(
            "DEFAULT",
            "install_ip_version")
    else:
        install_ip_version = "4"

    use_ipv6 = None
    if base_nova_conf.has_option("DEFAULT", "use_ipv6"):
        use_ipv6 = base_nova_conf.get("DEFAULT", "use_ipv6")

    if(install_ip_version == '6' and (use_ipv6 == 'True')):
        return True
    else:
        return False


def checkReachability(ip):
    reachability = -1
    if is_ipv6_install():
        cmdping = ["ping6", "-c 4 ", "-w 10 ", ip]
    else:
        cmdping = ["ping", "-c 4 ", "-w 10 ", ip]
    p = subprocess.Popen(cmdping, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    out, err = p.communicate()
    reachability = p.wait()
    return reachability


def checkNetworkQuality(ip):
    if is_ipv6_install():
        cmdping = ["ping6", "-c 100 ", "-w 60000 ", ip]
    else:
        cmdping = ["ping", "-c 100 ", "-w 60000 ", ip]
    p = subprocess.Popen(cmdping, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    out, err = p.communicate()
    p.wait()
    percentage_val = -1
    if len(err) != 0:
        LOG.error("CmdExeError in checkNetworkQuality {0}".format(err))
    if out is not None and len(out) != 0:
        percentage_val = re.search('received,(.+?)% packet loss', out).group(1)
    return percentage_val


def result_msg_dict(corrAction="", msg="", msgId="", msgReplcmnt=None,
                    status=""):
    result_dict = {}
    result_dict["CorrectiveAction"] = corrAction
    result_dict["Message"] = msg
    result_dict["MessageId"] = msgId
    result_dict["MsgSubstitutes"] = msgReplcmnt
    result_dict["Status"] = status
    result_dict["Verbose"] = "null"
    result_dict["TimeStamp"] = getCurrSysTime()
    return result_dict


def getCurrSysTime():
    utcTime = str(int(time.time()))
    return utcTime


base_nova_conf_file = "/etc/nova/nova.conf"

SUCCESS = 0
WARN = 1
CRITICAL = 2
CodeToStatus_map = {SUCCESS: "Success", WARN: "Warning", CRITICAL: "Critical"}
StatusToCode_map = {"Success": SUCCESS, "Warning": WARN, "Critical": CRITICAL}
Status_map = {"Success": "pass", "Warning": "warning",
              "Critical": "fail", "Info": "info"}


# CheckerLogger class will handle the logging.


class checkerLogger:

    def __init__(self, log_file=None):
        self.logger = logging.getLogger(__name__)
        self.log_file = log_file
        self.logger.setLevel(logging.DEBUG)
        handler = logging.handlers.RotatingFileHandler(self.log_file,
                                                       maxBytes=1 *
                                                       1024 * 1024,
                                                       backupCount=5)
        handler.setLevel(logging.INFO)
        handler.setLevel(logging.ERROR)
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s -\
            %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)

    def getLogger(self):
        return self.logger

# Exception


def checkResult(ip="", corrAction="", caplaceholder=[], msg="",
                msgplaceholder=[], msgId="", msgReplcmt="", status="",
                verbTxt="", val_check_id=""):
    result_msg = {}
    result_msg["ip"] = ip
    if "" != corrAction:
        result_msg["response"] = corrAction.format(*caplaceholder)
    if "" != msg:
        result_msg["msg"] = msg.format(*msgplaceholder)
    result_msg["refs"] = []
    if "" != msgId:
        if len(msgId.split(":")) >= 2:
            result_msg["msg-id"] = msgId.split(":")[0]
            refdict = {"type": "help-info-link", "value": msgId.split(":")[1]}
            result_msg["refs"].append(refdict)
        else:
            result_msg["msg-id"] = msgId

    result_msg["MsgSubstitutes"] = msgReplcmt
    result_msg["status"] = Status_map.get(status)
    result_msg["Verbose"] = verbTxt
    result_msg["val_check_id"] = val_check_id
    return result_msg


def readJSONFile(jsonfile):
    fobj = open(jsonfile, "r")
    data = json.load(fobj)
    data = convert(data)
    fobj.close()
    return data


def convert(data):
    if isinstance(data, dict):
        return dict([(convert(key), convert(value)) for key,
                     value in data.iteritems()])
    if isinstance(data, list):
        return [convert(element) for element in data]
    elif isinstance(data, unicode):
        return data.encode('utf-8')
    else:
        return data


def get_host_name():
    # Read the HOST_NAME from ttv_validation_api_paste_file file.
    # Setting default hostname
    host_name = "localhost"
    ttv_api_paste = glob.glob(ttv_validation_api_paste_file)
    ttv_api_paste_conf = ConfigParser.ConfigParser()
    ttv_api_paste_conf.read(ttv_api_paste)
    if ttv_api_paste_conf.has_option("filter:audit", "service_name"):
        service_name_value = ttv_api_paste_conf.get(
            "filter:audit", "service_name")
        if service_name_value is not None and service_name_value != "":

            # For IPV6 Scenario
            if is_ipv6_install():
                splitted_values = service_name_value.split("]")

                # If IP is present into conf file
                if len(splitted_values) >= 2:
                    host_name = splitted_values[0].lstrip('[')

                # If hostname is present into conf file
                elif len(splitted_values) == 1:
                    splitted_values = service_name_value.split(":")
                    if len(splitted_values) >= 2:
                        host_name = splitted_values[0]

            # For IPV4 Scenario irrespective of IP or Hostname
            else:
                splitted_values = service_name_value.split(":")
                if len(splitted_values) >= 2:
                    host_name = splitted_values[0]
    return host_name.strip()


def getClearTextString(text):
    """Convert an encrypted string to a clear text one. Clear text will be
    unchanged"""
    encrypt_handler = EncryptHandler()
    return encrypt_handler.decode(text)


def runSSHCommand(ssh, cmd, debug=True):
    LOG.debug("executing ssh command: " + cmd)

    try:
        #TODO: Defect 47986 - this was flagged in a Bandit Scan.
        #Will need to fix subprocess issue in 2Q
        stdin, stdout, stderr = ssh.exec_command(cmd)
        returnCode = stdout.channel.recv_exit_status()
        output = stdout.read().splitlines()
        err_output = stderr.read().splitlines()
        if debug:
            LOG.debug(
                "SSH command [%s] returned returnCode: "
                "%s stdout: %s stderr: %s" %
                (cmd, returnCode, output, err_output))
        return (returnCode, output, err_output)
    except Exception as e:
        LOG.exception(
            "Error while running command %s: %s" % (cmd, str(e)))

        return None, None, None


def getSSHConnection(ip, user, port=22, pwd=None, key=None):
    ssh = None
    ssh = paramiko.SSHClient()
    base_nova_conf = ConfigParser.ConfigParser()
    base_nova_conf.read(glob.glob(base_nova_conf_file))
    if base_nova_conf.has_option(DEFAULT_SECTION,
                                 powervc_known_host_path_param):
        powervc_known_hosts = base_nova_conf.get(
            DEFAULT_SECTION,
            powervc_known_host_path_param)
        if os.path.exists(powervc_known_hosts):
            ssh.load_host_keys(powervc_known_hosts)
    elif os.path.exists(powervc_known_host_file):
        ssh.load_host_keys(powervc_known_host_file)

    clearpwd = getClearTextString(pwd)
    ssh.set_missing_host_key_policy(paramiko.RejectPolicy())
    ssh.connect(
        ip,
        username=user,
        password=clearpwd,
        port=port,
        key_filename=key)
    return ssh

'''
Compares versions
'''


def compareVersion(actual_version, expected_version):
    try:
        actual = V(actual_version)
        expected = V(expected_version)
        return actual >= expected
    except Exception as exp:
        LOG.exception("Error occurred while comparing versions" + str(exp))
        return False


def getAllHMCInfo():
    env = {"HMC": [], }
    conf = glob.glob(base_nova_conf_file)
    base_nova_conf = ConfigParser.ConfigParser()
    base_nova_conf.read(conf)
    text = base_nova_conf.get("database", "connection")
    sql_connection = getClearTextString(text)
    result = executeOnDB(sql_connection, hmc_query)
    for row in result:
        hmc = {}
        hmc["id"] = row['id']
        hmc["uuid"] = row['uuid']
        hmc["display_name"] = row['display_name']
        hmc["ip"] = row['access_ip']
        hmc["hostname"] = row['access_ip']
        hmc["userid"] = row['user_id']
        hmc["passwd"] = row['user_credentials']
        hmc["registered_at"] = row['registered_at']
        env["HMC"].append(hmc)
    return env


def getAllSSPInfo():
    env = {"SSP": [], }
    # SSP providers
    cinderconf = glob.glob(base_cinder_conf_file)
    base_cinder_conf = ConfigParser.ConfigParser()
    base_cinder_conf.read(cinderconf)

    cindertext = base_cinder_conf.get("database", "connection")
    cinder_sql_connection = getClearTextString(cindertext)
    cinderres = executeOnDB(cinder_sql_connection, hmc_clusters_query)

    if cinderres:
        for row in cinderres:
            ssp = {}
            ssp["HMC_UUID"] = row['hmc_uuid']
            ssp["HOST_NAME"] = row['host_name']

            env["SSP"].append(ssp)
    return env


def executeOnDB(sqlconnction, query):
    """Executes a sql query on DB and returns the result"""
    engine = create_engine(sqlconnction, echo=False)

    connection = engine.connect()
    result = connection.execute(query)
    # connection.close()
    return result


def getPowerVCEdition():
    conf = glob.glob(base_nova_conf_file)
    base_nova_conf = ConfigParser.ConfigParser()
    base_nova_conf.read(conf)
    powervc_edition = ""
    if base_nova_conf.has_option("DEFAULT", "powervm_mgr_type"):
        powervc_edition = base_nova_conf.get("DEFAULT", "powervm_mgr_type")
    elif base_nova_conf.has_option("DEFAULT", "host_type"):
        powervc_edition = base_nova_conf.get("DEFAULT", "host_type")
    return powervc_edition


def read_nova_conf(logger):
    env = {}
    env["novalink-partitions"] = []
    for conf in glob.glob("/etc/nova/nova-*.conf"):
        if conf == "/etc/nova/nova-health.conf":
            continue
        try:
            logger.info(_("reading nova.conf file {0}".format(conf)))
            nova_conf = ConfigParser.ConfigParser()
            nova_conf.read(conf)
            host_type = nova_conf.get('DEFAULT', 'host_type').lower()
            pvm_mgr_type = nova_conf.get('DEFAULT', 'powervm_mgr_type').lower()
            if host_type == 'powervm' and pvm_mgr_type == 'pvm':
                novalink_partitions = {}

                novalink_partitions["host_display_name"] =\
                    nova_conf.get("DEFAULT", "host_display_name")

                novalink_partitions["dhcp_domain"] =\
                    nova_conf.get("DEFAULT", "dhcp_domain")

                novalink_partitions["powervm_mgr_type"] =\
                    nova_conf.get("DEFAULT", "powervm_mgr_type")

                novalink_partitions["host_uniqueid"] =\
                    nova_conf.get("DEFAULT", "host_uniqueid")

                novalink_partitions["powervm_host_ip"] =\
                    nova_conf.get("DEFAULT", "powervm_host_ip")

                novalink_partitions["powervm_host_user"] =\
                    nova_conf.get("DEFAULT", "powervm_host_user")

                novalink_partitions["powervm_host_passwd"] = None
                novalink_partitions["powervm_host_private_key"] = None

                if nova_conf.has_option("DEFAULT", "powervm_host_private_key"):
                    if os.path.exists(nova_conf.get(
                            "DEFAULT", "powervm_host_private_key")):
                        novalink_partitions["powervm_host_private_key"] =\
                            getClearTextString(nova_conf.get(
                                "DEFAULT", "powervm_host_private_key"))
                if nova_conf.has_option("DEFAULT", "powervm_host_passwd"):
                    novalink_partitions["powervm_host_passwd"] = nova_conf.get(
                        "DEFAULT", "powervm_host_passwd")
                env["novalink-partitions"].append(novalink_partitions)
        except Exception as e:
            logger.exception(e)
    return env


def parse_arguments():
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument('-a', '--auth_token', dest='auth_token',
                            help='Auth Token has not been passed')
        parser.add_argument('-t', '--tenant_id', dest='tenant_id',
                            help='Tenant ID has not been passed')
        parser.add_argument('-l', '--locale', dest='locale',
                            help='locale  has not been passed')
        args = parser.parse_args()
    except Exception as e:
        LOG.error(e)
        return None
    return args
